*************************************************
* Download data from the Flow of Funds website
* and create a stata version of the time serie
*
* This script downloads all the time series avaiable
* for non-financial corporate businesses, from
* 1945 to 2014, at the quarterly frequency
*
* Written to be called from another script, with two arguments:
*	- `1': full path of temporary file where output is saved
*	- `2': full path of permanent file to store the download (if necessary)
*
* Last modification:
* March 12th, 2015
* Problem: the URL for the package seems to change over time
*******************************************************

**********************************************************************************************************************************************************************************************************************************
* The Url for the package is:
* "http://www.federalreserve.gov/datadownload/Output.aspx?rel=Z1&series=88c9e1429fc465763d744cbc76615dfa&lastObs=&from=12/01/1945&to=12/31/2014&filetype=csv&label=include&layout=seriescolumn"
**********************************************************************************************************************************************************************************************************************************

***********************************************************************
* The list of variables names and labels are available on:
* http://www.federalreserve.gov/apps/fof/DisplayTable.aspx?t=l.102
***********************************************************************

local TempData = "`1'"
local PermData = "`2'"
local UrlFlowOfFundsNFCB = "https://www.federalreserve.gov/datadownload/output.aspx?rel=z1&series=88c9e1429fc465763d744cbc76615dfa&lastobs=&from=12/01/1945&to=12/31/2014&filetype=csv&label=include&layout=seriescolumn"

import delimited "`UrlFlowOfFundsNFCB'", varnames(6) clear

*		fl102050003
*		fl104004005	
# delimit ;
keep    timeperiod
		fl104090005		
		fl103091003	
		fl103020005	
		fl103030003
		fl103034003		
		fl103069100	
		fl103061103	
		fl103061703	
		fl103062003	
		fl103065003	
		fl103066005
		fl103064203	
		fl103070005	
		fl103092005	
		fl103090005	
		fl103076005	
		fl103092405	
		fl103094105	
		fl103093005	
		fl104190005	
		fl104104005	
		fl103169100	
		fl103162000	
		fl103163003	
		fl103168005	
		fl103169005	
		fl103165005	
		fl103170005	
		fl103178000	
		fl103192005	
		fl103190005	
		fl103193005	
		fl103164103
		fa106012005
		fa106000135
		fa105050005
		fa105090005
		fa105019005
		fa105019985
		fa105060005
		fa105061005
		fu105012005
		fu105013005
		;
# delimit cr

gen date = quarterly(timeperiod,"YQ")
format date %tq
gen fyear = year(dofq(date))
gen quart = quarter(dofq(date))
drop timeperiod

label var fl104090005 "total financial assets"
label var fl103091003 "private foreign deposits"
label var fl103020005 "checkable deposits and currency"
label var fl103030003 "total time and savings deposits"
label var fl103034003 "money market mutual fund shares"
*label var fl102050003 "security repurchase agreements"
label var fl104004005 "credit market instruments"
label var fl103069100 "commercial paper"
label var fl103061103 "Treasury securities"
label var fl103061703 "agency- and GSE-backed securities"
label var fl103062003 "municipal securities and loans"
label var fl103065003 "total mortgages"
label var fl103066005 "consumer credit"
label var fl103064203 "mutual fund shares"
label var fl103070005 "trade receivables"
label var fl103092005 "U.S. direct investment abroad"
label var fl103090005 "total miscellaneous assets"
label var fl103076005 "insurance receivables due from property-casualty insurance companies"
label var fl103092405 "equity in Fannie Mae"
label var fl103094105 "equity investment in finance company subsidiaries"
label var fl103093005 "unidentified miscellaneous assets"
label var fl104190005 "total liabilities"
label var fl104104005 "credit market instruments"
label var fl103169100 "commercial paper"
label var fl103162000 "municipal securities and loans"
label var fl103163003 "corporate bonds"
label var fl103168005 "depository institution loans n.e.c."
label var fl103169005 "other loans and advances"
label var fl103165005 "total mortgages"
label var fl103170005 "trade payables"
label var fl103178000 "taxes payable"
label var fl103192005 "foreign direct investment in U.S."
label var fl103190005 "total miscellaneous liabilities"
label var fl103193005 "unidentified miscellaneous liabilities"
label var fl103164103 "corporate equities"
label var fa106012005 "disposable income, net; net saving (IMA)"
label var fa106000135 "US internal funds, book"
label var fa105050005 "total capital expenditures"
label var fa105090005 "gross investment"
label var fa105019005 "gross fixed investment"
label var fa105019985 "gross fixed investment with equity REITs and inventories (IMA)"
label var fa105060005 "net investment"
label var fa105061005 "net physical investment"
label var fu105012005 "gross fixed investment, residential equipments and structures (unadjusted)"
label var fu105013005 "gross fixed investment, nonresidential structures, equipment and intellectual property products"

rename fl104090005 a_tot_fin
rename fl103091003 a_private_dep
rename fl103020005 a_check_dep
rename fl103030003 a_time_savings_dep
rename fl103034003 a_mmf_shares
*rename fl102050003 a_repos
rename fl104004005 a_credit_market_instr
rename fl103069100 a_compap
rename fl103061103 a_treasuries
rename fl103061703 a_agencydebt
rename fl103062003 a_muinipcalsec
rename fl103065003 a_mortgages
rename fl103066005 a_conscredit
rename fl103064203 a_mutualfundshares
rename fl103070005 a_tradereceivables
rename fl103092005 a_invest_abroad
rename fl103090005 a_miscellaneous
rename fl103076005 a_insureance_receiv
rename fl103092405 a_equity_GSE
rename fl103094105 a_invest_financialcomp_subs
rename fl103093005 a_othermiscellaneous
rename fl104190005 l_tot_fin
rename fl104104005 l_creditmarketinstruments
rename fl103169100 l_compap
rename fl103162000 l_municip
rename fl103163003 l_corpbonds
rename fl103168005 l_depositoryinstitloans
rename fl103169005 l_otherloans
rename fl103165005 l_mortgages
rename fl103170005 l_tradepayables
rename fl103178000 l_taxespayable
rename fl103192005 l_FDI
rename fl103190005 l_miscellaneous
rename fl103193005 l_unidentifiedmiscellaneous
rename fl103164103 mkt_equities
rename fa106012005 f_disposableincome
rename fa106000135 f_internalfunds
rename fa105050005 f_capex
rename fa105090005 f_grossinvestment 
rename fa105019005 f_grossfixedinvestment 
rename fa105019985 f_grossfixedinvestmentalt 
rename fa105060005 f_netinvestment
rename fa105061005 f_netphysicalinvestment
rename fu105012005 f_unadjgrossinvest1 
rename fu105013005 f_unadjgrossinvest2

foreach varname of varlist _all{
	destring `varname', ignore("ND") replace
}

/* Save to temporary file */
save "`TempData'", replace

/* Save to permanent file if a permanent file path was passed as argument */
if strlen("`PermData'") > 1 {
	save "`PermData'", replace
}
