******************************************************************
* Download and extract time series from the QFR,
* and save them to a csv file for matlab use
*
* Arguments:
*   - `1': name of the CSV file where the extract should be saved
*
* Last modified: February 25th, 2015
******************************************************************

# delimit cr
clear
clear mata 
set scheme s2color

foreach package in "labutil" {
 		capture which `package'
 		if _rc==111 ssc install `package'
}

global RootPath `"`c(pwd)'"'
global matlabExtract = "$RootPath/`1'.csv"
global TempQFR = "$RootPath/tempdata.dta"

/* Download and format QFR data */
do "$RootPath/downloadQFR.do" "$TempQFR"
use "$TempQFR", clear

/* Create id and balance sheet variable lists */
global idVars assetSize year quarter dateq format industry
qui ds
global bsVars `r(varlist)'
global bsVars: list global(bsVars) - global(idVars)

/* Keep only manufacturing, wholesale and retail firms */
keep if (industry == 6)
/* Drop pre-aggregated asset size classes */
keep if assetSize != 4 & assetSize != 5 & assetSize != 11
collapse (mean) year quarter (sum) $bsVars, by(dateq assetSize)

/* Create debt measures */
gen cc1 = STBANK + INSTBANKS
lab var cc1 "Current bank debt" 
gen cc3 = COMPAPER + INSTBONDS + STDEBTOTH + INSTOTHER
lab var cc3 "Current non-bank debt"
gen tt1 = STBANK + INSTBANK + LTBNKDEBT
lab var tt1 "Total bank debt"
gen tt3 = COMPAPER + INSTBONDS + LTBNDDEBT + STDEBTOTH + INSTOTHER + LTOTHDEBT
lab var tt3 "Total non-bank debt"
gen ccfin = cc1 + cc3
lab var ccfin "Total current debt"
gen ttfin = tt1 + tt3
lab var ttfin "Total debt"

rename TOTASSET totasset

sort year quarter assetSize
keep year quarter assetSize cc1 cc3 tt1 tt3 ccfin ttfin totasset

outsheet year quarter assetSize cc1 cc3 tt1 tt3 ccfin ttfin totasset using "$matlabExtract", comma nolabel replace
erase "$TempQFR"
