function [entry,distribution,aggregates,FD_moments] = aggregate_implications(policy_functions,A,B,ecv,steady_state,current_mu,next_e,pars)

N_z      = pars.N_z;
e        = policy_functions.e;
z        = policy_functions.z;
phi_d_r  = policy_functions.phi_d_r;
phi_b    = policy_functions.phi_b;
k_hat_r  = policy_functions.k_hat_r;
phi_d_nr = policy_functions.phi_d_nr;
k_hat_nr = policy_functions.k_hat_nr;
s_hat_r  = policy_functions.s_hat_r;
if steady_state
    next_e   = e;
end;
e_eq     = next_e(1,:)';
e_div    = next_e(end,:)';

[e_entry,NPV_entry,index_entry] = deal(NaN(N_z,1));
return_warning{1} = true; return_warning{2} = 'entry scale'; 

% Entry scale
if ~pars.onei
    
    parfor m=1:N_z

        z_entry = pars.z_augmented_nosubsidy(m,:);
        e_min = e_eq(m); e_max = policy_functions.e_switch_nosubsidy(m);
        return_more = true; MV_first = true; 
        [dVde_min,~,~,V_min] = optimal_policy_r( e_min, z_entry, A, B, ecv, return_more, MV_first, pars.algorithm_r, pars);
        [dVde_max,~,~,V_max] = optimal_policy_r( e_max, z_entry, A, B, ecv, return_more, MV_first, pars.algorithm_r, pars);

        if dVde_min <= (1+pars.gamma_e_ipo)*pars.beta^(-1)*((1-pars.eta)^(-1))

            fprintf('Entry value is maximized at e = e_eq for m = %d, setting e_entry = e_eq \n',m);
            e_entry(m) = e_eq(m);
            NPV_entry(m) = pars.beta*(1-pars.eta)*V_min - (1+pars.gamma_e_ipo)*e_eq(m);

        else
            
            if dVde_max > (1+pars.gamma_e_ipo)*pars.beta^(-1)*((1-pars.eta)^(-1))
            
                fprintf('Firms enter in the no-renegotiation region for m = %d \n',m);
                x_low = e_max; x_high = e_div(m); tol_f = pars.tol_f_thr_min; tol_x = pars.tol_x_thr_min;
                ff = @(x,indexes)  -pars.beta*(1-pars.eta)*optimal_policy_nr( x, z_entry, A, B, ecv, false, pars) + (1 + pars.gamma_e_ipo);
                dummy = bissection(x_low,x_high,ff,tol_f,tol_x,return_warning);
                e_entry(m) = dummy;
                [~,V] = optimal_policy_nr( e_entry(m), z_entry, A, B, ecv, false, pars);
                NPV_entry(m) = pars.beta*(1-pars.eta)*V - (1+pars.gamma_e_ipo)*e_entry(m);
                
            else

                x_low = e_min; x_high = e_max; tol_f = pars.tol_f_thr_min; tol_x = pars.tol_x_thr_min;
                ff = @(x,indexes) -pars.beta*(1-pars.eta)*optimal_policy_r( x, z_entry, A, B, ecv, return_more, MV_first, pars.algorithm_r, pars) + (1 + pars.gamma_e_ipo);
                dummy = bissection(x_low,x_high,ff,tol_f,tol_x,return_warning);
                e_entry(m) = dummy;
                [~,~,~,V] = optimal_policy_r( e_entry(m), z_entry, A, B, ecv, return_more, MV_first, pars.algorithm_r, pars);
                NPV_entry(m) = pars.beta*(1-pars.eta)*V - (1+pars.gamma_e_ipo)*e_entry(m);
                
            end;

        end;

        [~,index_entry(m)]         = min(abs(next_e(:,m) - repmat(e_entry(m)',[pars.N-1,1])));
        %index_entry(m) = max(2,index_entry(m)-1);

    end;
    entry = {};
    entry.size  = e_entry;
    entry.NPV   = NPV_entry;
    entry.index = index_entry;
    
else

    parfor m=1:N_z

        z_entry = pars.z_augmented_nosubsidy(m,:);

        return_VF = false;
        [dVde_min,V_min] = optimal_policy_nr( e_eq(m), z_entry, A, B, ecv, return_VF, pars);

        if dVde_min <= (1+pars.gamma_e_ipo)*pars.beta^(-1)/(1-pars.eta)

            fprintf('Entry value is maximized at e = e_eq for m = %d, setting e_entry = e_eq \n',m);
            e_entry(m) = e_eq(m);
            NPV_entry(m) = pars.beta*(1-pars.eta)*V_min - (1+pars.gamma_e_ipo)*e_eq(m);

        else

            x_low = e_eq(m); x_high = e_div(m); tol_f = pars.tol_f_thr_min; tol_x = pars.tol_x_thr_min;
            ff = @(x,indexes) -pars.beta*(1-pars.eta)*optimal_policy_nr( x, z_entry, A, B, ecv, return_VF, pars) + (1 + pars.gamma_e_ipo);
            dummy = bissection(x_low,x_high,ff,tol_f,tol_x,return_warning);
            e_entry(m) = dummy;
            [~,V] = optimal_policy_nr( e_entry(m), z_entry, A, B, ecv, return_VF, pars);
            NPV_entry(m) = pars.beta*(1-pars.eta)*V - (1+pars.gamma_e_ipo)*e_entry(m);

        end;

        [~,index_entry(m)]         = min(abs(next_e(:,m) - repmat(e_entry(m)',[pars.N-1,1])));
        index_entry(m) = index_entry(m) - 1;

    end;
    entry = {};
    entry.size  = e_entry;
    entry.NPV   = NPV_entry;
    entry.index = index_entry;
    if max(entry.NPV) < 0
        fprintf('Negative NPV of entry for all indices; assuming entry at 0 w/ ergodic distro \n');
    end;

    %{
    index_entry = 2;
    e_entry = e(2,1);
    z_entry = pars.z_augmented_nosubsidy(1,:);
    [~,V] = optimal_policy_nr( e_entry, z_entry, A, B, ecv, return_VF, pars);
    NPV_entry(m) = pars.beta*(1-pars.eta)*V - (1+pars.gamma_e_ipo)*e_entry;
    %}
    
end;    

% Transition mapping
MM = compute_transition_matrix_new( next_e, z, phi_d_r, phi_b, k_hat_r, phi_d_nr, k_hat_nr, s_hat_r, pars );

% Vector nu for measure of entering firms
% Entry only at the lowest productivity level
nu = zeros(size(MM,1),1); N = size(e,1); 
if max(entry.NPV) > 0
    for m=1:N_z
        nu((m-1)*N+index_entry(m))     = pars.N_e*pars.pi_ergodic(m)*(1 - exp( - pars.lambda*NPV_entry(m)));
        if NPV_entry(m) <= 0
            fprintf('Found negative NPV for entry \n');
            nu((m-1)*N+index_entry(m))     = 0;
        end;
    end;
else
    for m=1:N_z
        nu((m-1)*N+index_entry(m))     = pars.N_e*pars.pi_ergodic(m);
    end;  
end;

% Solve for resulting steady-state distribution
if steady_state
    VV = MM'; II = eye(size(VV,1));
    mu = ((II-VV)^(-1))*nu; 
else
    mu = (MM.')*current_mu + nu;
end;
distribution    = {};
distribution.mu = mu;
distribution.nu = nu;

% Aggregates
if steady_state
   mu_for_aggregate = mu;
else
   mu_for_aggregate = current_mu;
end;
    
s = s_hat_r;
s(isnan(s)) = 0; s = s(:);
k = k_hat_r;
indexes = isnan(k); k(indexes) = k_hat_nr(indexes); k = k(:);
E_phi                   = repmat(pars.E_phi',[size(e,1),1]); E_phi = E_phi(:);
capital                 = sum(mu_for_aggregate.*k);
output                  = sum(mu_for_aggregate.*(E_phi.*(k.^(pars.zeta))));
Total_B                 = sum(mu_for_aggregate.*(s.*(k-e(:))));
Total_M                 = sum(mu_for_aggregate.*((1-s).*(k-e(:))));
%index_nonzero          = (k>1e-2);
index_nonzero           = (k>0);
av_lev                  = sum(mu_for_aggregate(index_nonzero).*((k(index_nonzero)-e(index_nonzero))./k(index_nonzero)))/sum(mu_for_aggregate(index_nonzero));
ag_lev                  = (Total_B + Total_M)/capital;
indexes_entry           = ((1:pars.N_z)'-1)*N+index_entry;
freq_entr               = sum(mu_for_aggregate(indexes_entry))/sum(mu_for_aggregate);
BS                      = Total_B/(Total_B+Total_M);
distrib                 = mu_for_aggregate/sum(mu_for_aggregate);
Mixed_Frac              = sum(distrib(s>0));
indexes_small           = k <= pars.threshold_SL;
S_B                     = sum(mu_for_aggregate(indexes_small).*(s(indexes_small).*(k(indexes_small)-e((indexes_small)))));
S_M                     = sum(mu_for_aggregate(indexes_small).*((1-s(indexes_small)).*(k(indexes_small)-e((indexes_small)))));
indexes_large           = k > pars.threshold_SL;
L_B                     = sum(mu_for_aggregate(indexes_large).*(s(indexes_large).*(k(indexes_large)-e((indexes_large)))));
L_M                     = sum(mu_for_aggregate(indexes_large).*((1-s(indexes_large)).*(k(indexes_large)-e((indexes_large)))));
aggregates              = {};
aggregates.capital      = capital;
aggregates.output       = output;
aggregates.Total_B      = Total_B;
aggregates.Total_M      = Total_M;
aggregates.BS           = BS;
aggregates.Mixed_Frac   = Mixed_Frac;
aggregates.S_B          = S_B;
aggregates.S_M          = S_M;
aggregates.L_B          = L_B;
aggregates.L_M          = L_M;
aggregates.av_lev       = av_lev;
aggregates.freq_entr    = freq_entr;
  
% Compute firm dynamics moments
%{
FD_index   = pars.N_z;
FD_moments = computing_FD_moments( FD_index, policy_functions, pars );
%}
FD_moments = {};

%fprintf(' \n');
%fprintf('Steady-state aggregate ratios: \n')
fprintf('   Bank share                    = %7.4f \n',BS);
fprintf('   Fraction mixed-debt firms     = %7.4f \n',Mixed_Frac);
%fprintf('   Output                       = %7.3f \n',output);
%fprintf('   Capital                      = %7.3f \n',capital);
%fprintf('   Bank borrowing               = %7.3f \n',Total_B);
%fprintf('   Market borrowing             = %7.3f \n',Total_M);
%fprintf('   Bank borrowing by small      = %9.5f \n',S_B);
%fprintf('   Market borrowing by small    = %9.5f \n',S_M);
%fprintf('   Bank borrowing by large      = %7.3f \n',L_B);
%fprintf('   Market borrowing by large    = %7.3f \n',L_M);
%fprintf('Leverage: \n')
%fprintf('   Steady-state average         = %7.3f \n',av_lev);
fprintf('   Liabilities to assets         = %7.4f \n',ag_lev);
fprintf('   Frequency of entry            = %7.4f \n',freq_entr);
%{
fprintf('   (Min,Max) in PF, by exogenous state:        \n');
kk = reshape(k,[N,N_z]);
ee = reshape(e,[N,N_z]);
for ii=1:pars.N_z
    dummy = (kk(:,ii)-ee(:,ii))./kk(:,ii); levz = dummy(~isnan(dummy));
    levmin = min(levz); levmax = max(levz);
    fprintf('       Min(%d)              = %7.3f \n',ii,levmin)
    fprintf('       Max(%d)              = %7.3f \n',ii,levmax)
end;
fprintf(' \n');
%}



end

