function [k_min_f,dk_min_fds,dk_min_fde] = borrowing_constraint_f_min(s,e,z,pars)

%Useful values
r_m = z(:,end);
chi_m = z(:,(end-2));
s_r_tilde = 1./(1+(pars.chi_L-chi_m)./(1+r_m)*(1+pars.r_b)./chi_m);

nogood = (e < 0 | s >= s_r_tilde); 

if sum(nogood) > 0
    error('e < 0 in borrowing_constraint_f \n');
end;

a = z(:,1); b = z(:,2); r_m = z(:,end);
r_tilde_b = (1+pars.r_b)*s/pars.chi - 1;
r_tilde_m = (1+r_m).*(1-s)./(1-pars.chi) - 1;

% upper bound
x_high      = borrowing_constraint_m(s,e,z,pars);

% lower bound
x_low       = (1+r_tilde_b)./(r_tilde_b+pars.delta_k).*e;

case1 = (e == 0);
case2 = (e > 0);

k_min_f = NaN(size(s));

if sum(case1) > 0

    x_low1 = x_low(case1); x_high1 = x_high(case1); a1 = a(case1); b1 = b(case1); r_tilde_b1 = r_tilde_b(case1); r_tilde_m1 = r_tilde_m(case1); e1 = e(case1); s1 = s(case1);
    z1 = z(case1,:);
    x_low1(r_tilde_b1 > r_tilde_m1) = x_low1(r_tilde_b1 > r_tilde_m1)+min(pars.tol_x_maxk_f,1e-12*(x_high1(r_tilde_b1 > r_tilde_m1) - x_low1(r_tilde_b1 > r_tilde_m1)));
    r_m1 = r_m(case1);
    
    % Find the borrowing limit
    ff = @(x,indexes) -( HN_fun(e1(indexes),s1(indexes),x,a1(indexes),b1(indexes),r_m1(indexes),z1(indexes,:),pars) -  x.^(1-pars.zeta).*( r_tilde_b1(indexes) + pars.delta_k ) );
    
    tol_f = pars.tol_f_maxk_f; tol_x = pars.tol_x_maxk_f;
    return_warning{1} = true; return_warning{2} = 'borrowing_constraint_f';
    k_min_f(case1) = bissection(x_low1,x_high1,ff,tol_f,tol_x,return_warning);
    k_min_f(case1) = max(0,k_min_f(case1));
    
end;

if sum(case2) > 0
    
    x_low2 = x_low(case2); x_high2 = x_high(case2); a2 = a(case2); b2 = b(case2); r_tilde_b2 = r_tilde_b(case2); e2 = e(case2); s2 = s(case2);
    r_m2 = r_m(case2); z2 = z(case2,:);
    % Find the borrowing limit
    ff = @(x,indexes) -( HN_fun(e2(indexes),s2(indexes),x,a2(indexes),b2(indexes),r_m2(indexes),z2(indexes,:),pars) - x.^(-pars.zeta).*( r_tilde_b2(indexes) + pars.delta_k ).*( x - ( 1 + r_tilde_b2(indexes) )./( pars.delta_k + r_tilde_b2(indexes) ).*e2(indexes) ) );   
    
    tol_f = pars.tol_f_maxk_f; tol_x = pars.tol_x_maxk_f;
    return_warning{1} = true; return_warning{2} = 'borrowing_constraint_f';
    k_min_f(case2) = bissection(x_low2,x_high2,ff,tol_f,tol_x,return_warning);
    k_min_f(case2) = max((1+r_tilde_m(case2))./(r_tilde_m(case2)+pars.delta_k).*e(case2),k_min_f(case2));
    
end;

% Check the market borrowing constraint is not violated
k_min_f(k_min_f > x_high) = x_high(k_min_f > x_high);

% Check that phi_d <= phi_b at the frontier; otherwise, adjust inward
phi_d   = lending_contract_m(k_min_f,s,e,a,b,z,pars);
phi_b   = lending_contract_b(k_min_f,s,e,a,b,pars);
case_modify = (phi_d > phi_b);
k_min_f(case_modify) = (1-1e-7)*k_min_f(case_modify);

% Other output
if nargout > 1
    [phi,~,~,~,~,~,~,dphidk,dphids,dphide]   = lending_contract_m(k_min_f,s,e,a,b,z,pars);
    dHdphi                                   = exp( -( (phi./a).^(b) ) );
    H                                        = H_fun(phi,a,b);
    dk_min_fds                               = ( (1+pars.r_b)/pars.chi*(k_min_f - e).*k_min_f.^(-pars.zeta) - dHdphi.*dphids ).*( 1./(  dHdphi.*dphidk - (k_min_f).^(-pars.zeta).*( (1-pars.zeta)*(r_tilde_b+pars.delta_k) + pars.zeta*(e/k_min_f)*(1+r_tilde_b) ) ) );
    dk_min_fde                               = ( dHdphi.*dphide + (1+r_tilde_b).*k_min_f.^(-pars.zeta)).*( 1./( (pars.delta_k+r_tilde_b).*(k_min_f.^(-pars.zeta)).*( 1 - pars.zeta + pars.zeta*((1+r_tilde_b).*e)./((pars.delta_k+r_tilde_b).*k_min_f) ) - dHdphi.*dphidk ));
end;

end
