function [k_max,dk_maxds,dk_maxde] = borrowing_constraint_m(s,e,z,pars)

nogood = (e < 0); 

if sum(nogood) > 0
    fprintf('e < 0 in borrowing_constraint_m \n');
end;

a = z(:,1); b = z(:,2); 
r_m = z(:,end);
r_tilde_m = (1+r_m).*(1-s)./(1-pars.chi) - 1;
chi_m = z(:,(end-2));
delta_hat = 1-(pars.chi_L - chi_m)./(1-chi_m).*(1-pars.delta_k);

case1 =  (s >= 1-(1-pars.chi)./(1+r_m).*(1-delta_hat));
case2 = (~(case1))&(e == 0);
case3 = (~(case1))&(e > 0);

k_max = NaN(numel(e),1); dk_maxde = k_max; dk_maxds = k_max;

if sum(case1) > 0
    
    k_max(case1)    = Inf(sum(case1),1);
    dk_maxds(case1) = Inf(sum(case1),1);
    dk_maxde(case1) = Inf(sum(case1),1);
    
end;

if sum(case2) > 0

    % upper bound
    phi_high     = (a(case2).^(-(1-b(case2))./b(case2))).*( b(case2)./a(case2) ).^(-1./b(case2));
    A            = (exp(-(phi_high./a(case2)).^(b(case2))).*phi_high)./(delta_hat(case2)+r_tilde_m(case2));
    B            = (1+r_tilde_m(case2))./(r_tilde_m(case2)+delta_hat(case2)).*e(case2);
    tol_x = pars.tol_x_fractionalroot; tol_f = pars.tol_f_fractionalroot;
    x_high       = find_root_fractional(A,B,tol_x,tol_f,pars);
    
    % lower bound
    x_low       = (1+r_tilde_m(case2))./(r_tilde_m(case2)+delta_hat(case2)).*e(case2);

    % Find the borrowing limit
    a2 = a(case2,:); b2 = b(case2,:); r_tilde_m2 = r_tilde_m(case2); z2 = z(case2,:); delta_hat2 = delta_hat(case2);
    ff = @(x,indexes) (delta_hat2(indexes)+r_tilde_m2(indexes)).*(x.^(1-pars.zeta)) - N_fun(maximum_default_m(x,a2(indexes),b2(indexes),z2(indexes,:),pars),-(1-delta_hat2(indexes)).*(x.^(1-pars.zeta)),a2(indexes),b2(indexes),pars);
    tol_f = pars.tol_f_maxk; tol_x = pars.tol_x_maxk;
    return_warning{1} = true; return_warning{2} = 'borrowing_constraint_m';
    
    % Borrowing limit
    k_max(case2)    = bissection(x_low,x_high,ff,tol_f,tol_x,return_warning);
    
end;

if sum(case3) > 0

    % upper bound
    phi_high     = (a(case3).^(-(1-b(case3))./b(case3))).*( b(case3)./a(case3) ).^(-1./b(case3));
    A            = (exp(-(phi_high./a(case3)).^(b(case3))).*phi_high)./(delta_hat(case3)+r_tilde_m(case3));
    B            = (1+r_tilde_m(case3))./(r_tilde_m(case3)+delta_hat(case3)).*e(case3);
    tol_x = pars.tol_x_fractionalroot; tol_f = pars.tol_f_fractionalroot;
    x_high       = find_root_fractional(A,B,tol_x,tol_f,pars);        

    % lower bound
    x_low       = (1+r_tilde_m(case3))./(r_tilde_m(case3)+delta_hat(case3)).*e(case3);

    % Find the borrowing limit
    a3 = a(case3,:); b3 = b(case3,:); r_tilde_m3 = r_tilde_m(case3); e3 = e(case3); z3 = z(case3,:); delta_hat3 = delta_hat(case3);
    ff = @(x,indexes) (x.^(-pars.zeta)).*(delta_hat3(indexes)+r_tilde_m3(indexes)).*(x - (1+r_tilde_m3(indexes))./(delta_hat3(indexes)+r_tilde_m3(indexes)).*e3(indexes)) - N_fun(maximum_default_m(x,a3(indexes),b3(indexes),z3(indexes,:),pars),-(1-delta_hat3(indexes)).*(x.^(1-pars.zeta)),a3(indexes),b3(indexes),pars);
    tol_f = pars.tol_f_maxk; tol_x = pars.tol_x_maxk;
    return_warning{1} = true; return_warning{2} = 'borrowing_constraint_m';
    
    % Borrowing limit
    k_max(case3) = bissection(x_low,x_high,ff,tol_f,tol_x,return_warning);
    
end;

% Other output
if nargout > 1 && sum(~case1)>0
    phi                     = maximum_default_m(k_max(~case1),a(~case1),b(~case1),z(~case1,:),pars);
    F                       = 1 - exp( -( (phi./a(~case1)).^(b(~case1)) ) );
    dNdk                    = -((1-delta_hat(~case1)).*(1-pars.zeta)).*(k_max(~case1).^(-pars.zeta)).*F;
    dk_maxds(~case1)        = (k_max(~case1).^(-pars.zeta)).*(1+r_m(~case1))./(1-pars.chi).*(k_max(~case1) - e(~case1)).*( 1./( (delta_hat(~case1)+r_tilde_m(~case1)).*(k_max(~case1).^(-pars.zeta)).*( 1 - pars.zeta + pars.zeta*((1+r_tilde_m(~case1)).*e(~case1))./((delta_hat(~case1)+r_tilde_m(~case1)).*k_max(~case1)) ) - dNdk ));
    dk_maxde(~case1)        = (1+r_tilde_m(~case1)).*k_max(~case1).^(-pars.zeta).*( 1./( (delta_hat(~case1)+r_tilde_m(~case1)).*(k_max(~case1).^(-pars.zeta)).*( 1 - pars.zeta + pars.zeta*((1+r_tilde_m(~case1)).*e(~case1))./((delta_hat(~case1)+r_tilde_m(~case1)).*k_max(~case1)) ) - dNdk ));
end;

end