function [k_max,dk_maxds,dk_maxde] = borrowing_constraint_nr(s,e,z,pars)

nogood = (e < 0); 

if sum(nogood) > 0
    fprintf('e < 0 in borrowing_constraint_nr \n');
end;

a = z(:,1); b = z(:,2); 
r_m = z(:,(end-1));
chi_m = z(:,(end-2));
E_phi = a.*gamma(1+1./b);
r_tilde_s =  1 + r_m + (pars.r_b - r_m).*s - 1;
delta_tilde = 1-pars.chi_L*(1-pars.delta_k);
delta_hat   = 1-((pars.chi_L - chi_m)./(1-chi_m)).*(1-delta_tilde);

% upper bound
A           = E_phi./(delta_tilde+r_tilde_s);
B           = (1+r_tilde_s)./(r_tilde_s+delta_tilde).*e;
tol_x = pars.tol_x_fractionalroot; tol_f = pars.tol_f_fractionalroot;
x_high      = find_root_fractional(A,B,tol_x,tol_f,pars);

% lower bound
A           = (chi_m.*E_phi)./(delta_tilde+r_tilde_s+(1-chi_m).*(1-delta_hat));
B           = (1+r_tilde_s)./(r_tilde_s+delta_tilde+(1-chi_m).*(1-delta_hat)).*e;
x_low       = find_root_fractional(A,B,tol_x,tol_f,pars);
x_low       = max(x_low,(1+r_tilde_s)./(r_tilde_s+delta_tilde).*e);

% Find the borrowing limit
ff = @(x,indexes) (delta_tilde+r_tilde_s(indexes)).*(x - (1+r_tilde_s(indexes))./(delta_tilde+r_tilde_s(indexes)).*e(indexes)) - (x.^(pars.zeta)).*M_fun(maximum_default_nr(x,a(indexes),b(indexes),z(indexes,:),pars),-(1-delta_hat(indexes)).*(x).^(1-pars.zeta),a(indexes),b(indexes),z(indexes,:),pars);
tol_f = pars.tol_f_maxk; tol_x = pars.tol_x_maxk;
return_warning{1} = true; return_warning{2} = 'borrowing_constraint_nr';
k_max = bissection(x_low,x_high,ff,tol_f,tol_x,return_warning);
k_max = max((1+r_tilde_s)./(r_tilde_s+delta_tilde).*e,k_max);

% Other output
if nargout > 1
    phi                     = maximum_default_nr(k_max,a,b,z,pars);
    F                       = 1 - exp( -( (phi./a).^(b) ) );
    dMdk                    = -(1-chi_m).*(1-delta_hat).*(1-pars.zeta).*(k_max.^(-pars.zeta)).*F;
    dk_maxds                = -(k_max.^(-pars.zeta)).*(pars.r_b - r_m).*(k_max - e).*( 1./( (delta_tilde+r_tilde_s).*(k_max.^(-pars.zeta)).*( 1 - pars.zeta + pars.zeta*((1+r_tilde_s).*e)./((delta_tilde+r_tilde_s).*k_max) ) - dMdk ));
    dk_maxde                = (1+r_tilde_s).*k_max.^(-pars.zeta).*( 1./( (delta_tilde+r_tilde_s).*(k_max.^(-pars.zeta)).*( 1 - pars.zeta + pars.zeta*((1+r_tilde_s).*e)./((delta_tilde+r_tilde_s).*k_max) ) - dMdk ));
end;

end