function [model,output] = calibration(parameters_calibration)

% Number of parameter combinations to experiment with
NN = size(parameters_calibration.r_b_vec,1);

% Put model characteristics in a structure
model(NN).model_version = 1;
for ii=1:NN
    
    % Irreversibility / fungibility
    model(ii).chi                   = parameters_calibration.chi_P;
    
    % Degree of returns to scale
    model(ii).zeta                  = parameters_calibration.zeta;
    
    % Intermediation cost
    model(ii).rb                    = parameters_calibration.r_b_vec(ii);
    
    % Depreciation rate
    model(ii).delta_k               = parameters_calibration.delta_k;
    
    % Interim equity issuance
    model(ii).interim_equity        = parameters_calibration.interim_equity;
    
     % Consumption upon exit
    model(ii).no_exit_c             = parameters_calibration.no_exit_c;
    
    % Subsidy to market debt
    model(ii).subsidy               = parameters_calibration.subsidy;
    model(ii).e_subsidy             = parameters_calibration.e_subsidy;
    
    % Grids / Optimization
    model(ii).optimize_e            = parameters_calibration.optimize_e;
    model(ii).gamma_e_i             = parameters_calibration.gamma_e_i;
    model(ii).maxiter_e             = parameters_calibration.iter_e;
    model(ii).N                     = parameters_calibration.N;
    model(ii).H                     = parameters_calibration.H;
    model(ii).paralellized_inside   = parameters_calibration.paralellized_inside;

    % Which version of the model to solve
    % 1 : dynamic, two instruments
    % 2 : dynamic, one instrument
    % 3 : static, two instruments
    % 4 : static, one instrument
    model(ii).model_version = 1;

    % Solve all the model, or load a solution and compute moments
    model(ii).solve_everything   = true;
    
    % Print output
    model(ii).print_stuff        = parameters_calibration.print_stuff;

end;

% Print characteristics of the models to solve
fprintf('\n');
fprintf('Model characteristics: \n');
fprintf('\n');
fprintf('H   N    opt e  gamma_e  iter_e      zeta   chi    rb       \n');
fprintf('----------------------------------------------------------- \n');
for ii=1:NN
    fprintf('%2.0f  %3.0f  %g      %6.4f   %g           %5.3f  %5.3f  %7.6f \n',model(ii).H,model(ii).N,model(ii).optimize_e,model(ii).gamma_e_i,model(ii).maxiter_e,model(ii).zeta,model(ii).chi,model(ii).rb);
end;

% Do the run
fprintf('\n');
if ~parameters_calibration.paralellized_inside
    parfor ii=1:NN
        output(ii) = solve_steady_state_functional(model(ii));
    end;
else
    for ii=1:NN
        output(ii) = solve_steady_state_functional(model(ii));
    end;
end;

% Print key moments
fprintf('\n');
fprintf('Results: \n');
fprintf('\n');
fprintf('H   N    opt e  gamma_e  iter_e      zeta   chi    rb             gam_opt    BS      Mfrac  DA     NWE    Adrop \n');
fprintf('--------------------------------------------------------------------------------------------------------------- \n');
for ii=1:NN
    fprintf('%2.0f  %3.0f  %g      %6.4f   %g           %5.3f  %5.3f  %7.6f      %7.6f    %5.4f  %5.3f  %5.3f  %5.3f  %5.3f \n',model(ii).H,model(ii).N,model(ii).optimize_e,model(ii).gamma_e_i,model(ii).maxiter_e,model(ii).zeta,model(ii).chi,model(ii).rb,...
                                                                                                      output(ii).gamma_e_ipo,output(ii).BS,output(ii).mixed_frac,output(ii).ag_lev,output(ii).nw_entry_ratio,output(ii).assets_drop);
end;
fprintf('\n')

end
