function pars = create_optimization_parameters

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%      
% Tolerance parameters for optimization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

pars                  = {};

% tolerance for finding the boundary for dividend issuance 
% this depends on the derivative of the value function, which is of order
% r; the tolerance needs to be set below 
pars.tol_f_thr_div                                = 1e-5;
pars.tol_x_thr_div                                = 1e-5;

% tolerance for finding the boundary for dividend issuance 
% this depends on the derivative of the value function, which is of order
% r; the tolerance needs to be set below 
pars.tol_f_thr_eq                                 = 1e-4;
pars.tol_x_thr_eq                                 = 1e-3;

% Since dVde is of order 1+gamma_e at the equity issuance threshold, the precision there can be loose 
pars.tol_x_thr_min                                = 1e-4;
pars.tol_f_thr_min                                = 1e-3;

% At the dividend issuance threshold, dVde is of order 1+r_m, so need to
% make this more precise
pars.tol_x_thr_max                                = 1e-4;
pars.tol_f_thr_max                                = 1e-6;

% tolerance for finding the boundaries between renegotiation and
% non-renegotiation regimes
% this depends on the level of the value function, so doesn't need to be
% extremely accurate.
pars.tol_f_thr_switch                             = 1e-5;
pars.tol_x_thr_switch                             = 1e-5;

% Tolerance for finding the maximum in k for given (e,s), when using the
% first-order approach
pars.tol_x_obj                              = 1e-4*pars.tol_x_thr_div;
pars.tol_f_obj                              = 1e-4*pars.tol_f_thr_div;

% Tolerance for finding the maximum in k for given (e,z,s), when using the golden
% section search algorithm
pars.tol_f_gss_k                            = 1e-4*pars.tol_f_thr_div;

% Tolerance for finding the maximum in s for given (e,z), when using the golden
% section search algorithm
pars.tol_f_gss_s                            = 1e-6*pars.tol_f_thr_div;

% Step size for numerical approximation of derivatives
pars.de                                     = 1e-6;
pars.ds                                     = 1e-5;

% tol_contr is for finding the solution to the contract
% this needs to be more precise than the tolerance for the firm objective
% can't be smaller than machine precision eps
pars.tol_x_contr                            = 1e-12;
pars.tol_f_contr                            = 1e-12;

% tol_maxk is for the borrowing constraint
% this needs to be more precise than the tolerance for the contract
% can't be smaller than machine precision eps
pars.tol_x_maxk                             = 1e-13;
pars.tol_f_maxk                             = 1e-13;

% tol_maxk_f is for the frontier constraint
% this needs to be less precise than the tolerance for the borrowing
% constraints, because it uses the borrowing constraints as an input
pars.tol_x_maxk_f                           = 1e-10;
pars.tol_f_maxk_f                           = 1e-10;

% tol_maxphi is for finding argmax_{phi} M(phi,k)
% this needs to be more precise than the tolerance for the borrowing
% can't be smaller than machine precision eps
pars.tol_x_maxphi                           = 1e-13;
pars.tol_f_maxphi                           = 1e-13;

% tol_thresholds is for the s-thresholds. Needs to be more precise than for
% finding the borrowing constraints
% can't be smaller than machine precision eps
pars.tol_x_sthr                             = 1e-14;
pars.tol_f_sthr                             = 1e-14;

% tol_fractional root is for solutions to Ak^{zeta} = k - B
% they serve to bound the borrowing constraint and do not need to be very
% tight
pars.tol_x_fractionalroot                   = 1e-8;
pars.tol_f_fractionalroot                   = 1e-8;

% Minimum level allowed for e in the construction of the endogenous
% state-space
pars.e_min = 1e-6;

% Algorithm type for the solution to the max_k problem for given s in the
% renegotiation region
pars.algorithm_r = 1; % global method

% Log-spacing for grid
pars.log_space = 1; % linear grid

end