function [CV_NR,dCV_NRdk,dCV_NRde,saalpha,sbbeta] = integral_T_NR(k,e,z,phi_d,dphidk,dphide,A_mat,B_mat,ecv_mat,pars)

numargs = numel(e); N = size(A_mat,2);

if ~( size(e,1) == size(A_mat,1) && size(e,1) == size(B_mat,1) && size(e,1) == size(ecv_mat,1) && size(A_mat,2) == size(B_mat,2) && size(A_mat,2) == size(ecv_mat,2)-1 );
    error('A_mat, B_mat or ecv_mat have the wrong size in integral_T \n');
end;

if (max(abs(A_mat),[],2) == 0)&(max(abs(B_mat),[],2) == 0)
    error('integral_T was called with zero continuation value \n');
end;

phi_x             = phi_d - (1-pars.chi_L)*(1-pars.delta_k)*(k.^(1-pars.zeta));
dphi_xdk          = dphidk - (1-pars.zeta)*(1-pars.chi_L)*(1-pars.delta_k)*(k.^(-pars.zeta));
dphi_xde          = dphide;
e_d               = (1-pars.chi_L)*(1-pars.delta_k)*k;
dummy             = arrayfun(@(x) find(ecv_mat(x,:)-e_d(x)<=0,1,'last'),(1:numargs)');
A_min             = A_mat(sub2ind([numargs,N],(1:numargs)',dummy));
B_min             = B_mat(sub2ind([numargs,N],(1:numargs)',dummy));
T_min             = A_min.*e_d + B_min;

% Values of (alpha_i^{NR},beta_i^{NR}), i = 1,...,N
phimat       = repmat(phi_x,[1,N+1]) + (repmat(1/(1-pars.tau)*k,[1,N+1]).^(-pars.zeta)).*ecv_mat;
phimat       = max( repmat(phi_d,[1,N+1]), phimat);
c = phimat(:,1:(end-1))'; c = c(:); d = phimat(:,2:end)'; d = d(:); x = repmat(phi_x,[1,N])'; x = x(:);
a = repmat(z(:,1),[1,N])'; a = a(:); b = repmat(z(:,2),[1,N])'; b = b(:);
[IE,IP] = integral_components(c,d,x,a,b);
IE = reshape(IE,[N,numargs])'; 
IP = reshape(IP,[N,numargs])'; 
aalpha = (1-pars.tau)*A_mat.*IE; bbeta = B_mat.*IP; ggamma = (1-pars.tau)*A_mat.*IP;

% Necessary to compute derivatives
a = z(:,1); b = z(:,2);
f =  (b./a).*(phi_d./a).^(b-1).*exp( -( (phi_d./a).^(b) ) ); f(phi_d <= 0) = 0;

% Components of value function and derivatives
saalpha   = sum(aalpha,2);
sbbeta    = sum(bbeta,2);
sggamma   = sum(ggamma,2);

% Objective function and derivatives
CV_NR   = saalpha.*(k.^(pars.zeta)) + sbbeta;
   
% Derivative with respect to k   
dCV_NRdk   = -(T_min).*f.*dphidk ...
          -sggamma.*(k.^(pars.zeta)).*dphi_xdk ...
          +saalpha.*(pars.zeta.*(k.^(pars.zeta-1)));
dCV_NRdk(dphidk == Inf) = -Inf;     
      
% Derivative with respect to e    
dCV_NRde   = -(T_min).*f.*dphide ...
          -sggamma.*(k.^(pars.zeta)).*dphi_xde;  
dCV_NRde(dphide == -Inf) = Inf;

    % Correction for e > 0, k = 0
    dummy = ( e > 0 & k == 0 );
    if sum(dummy > 0)
            
            Rrep           = repmat((1+pars.r)*e,[1,N]);
            indexes        = ( Rrep >= ecv_mat(:,1:(end-1)) )&( Rrep < ecv_mat(:,2:end));
            Arep           = (1-pars.tau)*A_mat(indexes);
            Brep           = B_mat(indexes);
            CV_NR(dummy)      = (1+pars.r)*Arep(dummy).*e(dummy) + Brep(dummy);

            % In this case, dCV_NRdk ~ A_i*( E_phi*zeta*k^(zeta-1) - (r+delta_k) ) ->
            % +Inf, where i is the index such that e_{i} <= (1+r)*e < e_{i+1}
            dCV_NRdk(dummy)   = Inf;
            
            
            % The derivative is continuous in e and k
            dCV_NRde(dummy)   = (1+pars.r)*Arep(dummy);
      
    end;
    
    % Correction for e = 0, k = 0
    dummy = ( e == 0 & k == 0 );
    if sum(dummy > 0)
            
            CV_NR(dummy)      = B_mat(dummy,1);

            % In this case, dCV_NRdk -> -Inf (since dphidk = Inf at k=0)
            dCV_NRdk(dummy)   = -Inf;

            % The derivative is not continuous in k
            dCV_NRde(dummy)   = Inf;
      
    end;

end

