function [CV,dCVdk,dCVds,dCVde] = integral_T_R(k,e,z,phi_d,phi_b,dphidk,dphi_bdk,dphids,dphi_bds,dphide,dphi_bde,A_mat,B_mat,ecv_mat,pars)

numargs = numel(e); N = size(A_mat,2);

if ~( size(e,1) == size(A_mat,1) && size(e,1) == size(B_mat,1) && size(e,1) == size(ecv_mat,1) && size(A_mat,2) == size(B_mat,2) && size(A_mat,2) == size(ecv_mat,2)-1 );
    error('A_mat, B_mat or ecv_mat have the wrong size in integral_T_R_P \n');
end;

if (max(abs(A_mat),[],2) == 0)&(max(abs(B_mat),[],2) == 0)
    error('integral_T_R_P was called with zero continuation value \n');
end;

%if (min(abs(k)) == 0 || min(abs(e)) == 0 )
%    error('integral_T_R_P does not take k = 0 or e = 0 arguments \n');
%end;

phi_bm            = pars.chi*phi_b + (1-pars.chi)*phi_d;
kmat              = repmat(k,[1,N+1]);
e_d               = (1-pars.chi_L)*(1-pars.delta_k)*k;
dummy             = arrayfun(@(x) find(ecv_mat(x,:)-e_d(x)<=0,1,'last'),(1:numargs)');
A_min             = A_mat(sub2ind([numargs,N],(1:numargs)',dummy));
B_min             = B_mat(sub2ind([numargs,N],(1:numargs)',dummy));
T_min             = A_min.*e_d + B_min;

% Values of (alpha_i^{R_P},beta_i^{R_P}), i = 1,...,N
phimat       = repmat(phi_bm,[1,N+1]) + (repmat(1/(1-pars.tau)*k,[1,N+1]).^(-pars.zeta)).*( ecv_mat - (1-pars.chi_L)*(1-pars.delta_k)*kmat );
phi_bmat     = repmat(phi_b,[1,N]);
indexes_b    = (phi_bmat >= phimat(:,1:(end-1)));
c = phimat(:,1:(end-1)); c(indexes_b) = phi_bmat(indexes_b); c = c'; c = c(:); 
d = phimat(:,2:end)'; d = d(:); x = repmat( phi_bm - (1-pars.chi_L)*(1-pars.delta_k)*k.^(1-pars.zeta) ,[1,N])'; x = x(:);
a = repmat(z(:,1),[1,N])'; a = a(:); b = repmat(z(:,2),[1,N])'; b = b(:);
[IE_RP,IP_RP] = integral_components(c,d,x,a,b);
IE_RP = reshape(IE_RP,[N,numargs])'; 
IP_RP = reshape(IP_RP,[N,numargs])';
aalpha_RP = (1-pars.tau)*A_mat.*IE_RP; bbeta_RP = B_mat.*IP_RP;

% Values of (alpha_i^{R_R},beta_i^{R_R}), i = 1,...,N
phimat       = repmat(phi_d,[1,N+1]) + 1/((1-pars.chi)*(1-pars.tau))*(repmat(k,[1,N+1]).^(-pars.zeta)).*( ecv_mat - (1-pars.chi_L)*(1-pars.delta_k)*kmat );
phi_dmat     = repmat(phi_d,[1,N+1]);
phimat       = max(phimat,phi_dmat);
phi_bmat     = repmat(phi_b,[1,N]);
indexes_b    = (phi_bmat < phimat(:,2:end));
c = phimat(:,1:(end-1))'; c = c(:); 
d = phimat(:,2:end); d(indexes_b) = phi_bmat(indexes_b); d = d'; d = d(:); 
x = repmat(phi_d - (1-pars.chi_L)*(1-pars.delta_k)*k.^(1-pars.zeta)/(1-pars.chi) ,[1,N])'; x = x(:);
a = repmat(z(:,1),[1,N])'; a = a(:); b = repmat(z(:,2),[1,N])'; b = b(:);
[IE_RR,IP_RR] = integral_components(c,d,x,a,b);
IE_RR = reshape(IE_RR,[N,numargs])';
IP_RR = reshape(IP_RR,[N,numargs])';
aalpha_RR = (1-pars.tau)*A_mat.*IE_RR; bbeta_RR = B_mat.*IP_RR;

% Necessary to compute derivatives
a = z(:,1); b = z(:,2);
f =  (b./a).*(phi_d./a).^(b-1).*exp( -( (phi_d./a).^(b) ) ); f(phi_d <= 0) = 0;

% Components of derivatives
M_RR   = sum((1-pars.tau)*A_mat.*IP_RR,2);
M_RP   = sum((1-pars.tau)*A_mat.*IP_RP,2);
N_RR   = sum(aalpha_RR,2);
N_RP   = sum(aalpha_RP,2);

% Objective function and derivatives
CV   = sum(aalpha_RP,2).*(k.^(pars.zeta)) + sum(bbeta_RP,2) + ...
       (1-pars.chi)*N_RR.*(k.^(pars.zeta)) + sum(bbeta_RR,2);
   
% Derivative with respect to k   
dphi_bmdk = pars.chi*dphi_bdk + (1-pars.chi)*dphidk;  
dCVdk   = -(T_min).*f.*dphidk ...
          -M_RR.*((1-pars.chi)*(k.^(pars.zeta)).*dphidk - (1-pars.zeta)*(1-pars.chi_L)*(1-pars.delta_k) )...
          +(1-pars.chi)*N_RR.*(pars.zeta.*(k.^(pars.zeta-1))) ...
          +N_RP.*(pars.zeta.*(k.^(pars.zeta-1))) ...
          -M_RP.*( (k.^(pars.zeta)).*dphi_bmdk - (1-pars.zeta)*(1-pars.chi_L)*(1-pars.delta_k) );
dCVdk(dphi_bmdk == Inf & dphidk == Inf) = -Inf;     

%dT1 = -B_mat(:,1).*f.*dphidk;
%dT2 = (1-pars.chi)*N_RR.*(pars.zeta.*(k.^(pars.zeta-1)));
%dT3 = -(1-pars.chi)*M_RR.*(k.^(pars.zeta)).*dphidk;
%dT4 = N_RP.*(pars.zeta.*(k.^(pars.zeta-1)));
%dT5 = -M_RP.*(k.^(pars.zeta)).*dphi_bmdk;
                   
% Derivative with respect to s   
dphi_bmds = pars.chi*dphi_bds + (1-pars.chi)*dphids;  
dCVds   = -(T_min).*f.*dphids ...
          -(1-pars.chi)*M_RR.*(k.^(pars.zeta)).*dphids ...
          -M_RP.*(k.^(pars.zeta)).*dphi_bmds;  
dCVds(dphids == -Inf) = Inf;
      
% Derivative with respect to e   
dphi_bmde = pars.chi*dphi_bde + (1-pars.chi)*dphide;  
dCVde   = -(T_min).*f.*dphide ...
          -(1-pars.chi)*M_RR.*(k.^(pars.zeta)).*dphide ...
          -M_RP.*(k.^(pars.zeta)).*dphi_bmde;  
dCVde(dphide == -Inf) = Inf;

%if sum( isnan(dCVdk) ) ~= 0 || sum( isnan(dCVds) ) || sum( isnan(dCVde) )
%    error('NaN obtained in integral_T_R \n');
%end;
      
end

