function [ ecvp, Ap, Bp, policy_functions ] = iterate_model_paralellized( ecv, A, B, pars, exogenous_e, print_output )

N_z = pars.N_z;
N   = pars.N;

[e,V_r,s_hat_r,k_hat_r,phi_d_r,phi_b,V_nr,k_hat_nr,phi_d_nr]                                                                      = deal(NaN(N-1,N_z));
[e_eq,e_switch_nosubsidy,e_div,e_switch_subsidy,index_switch_nosubsidy,index_switch_subsidy,index_subsidy,e_subsidy]              = deal(NaN(N_z,1));
[e_save,z_save]                                                                                                                   = deal({});

if ~pars.onei

        parfor m=1:N_z

           tval = [pars.gamma_e;0]; update_e_min = true;    
           [e_eq(m),e_switch_nosubsidy(m),e_div(m),e_switch_subsidy(m)] = thresholds_e( m, tval, update_e_min, A, B, ecv, pars );
           e_subsidy(m) = pars.e_subsidy; 
            
           % Thresholds for net worth
           % State-space for net worth
           if isempty(exogenous_e)
                 
                dummy_e                                 = intspace(e_eq(m),e_div(m),N-1,pars.log_space);
                if ~isnan(e_switch_subsidy(m))
                    [~,index_switch_subsidy(m)]         = min(abs(dummy_e - repmat(e_switch_subsidy(m)',[N-1,1])));
                    dummy_e(index_switch_subsidy(m))    = e_switch_subsidy(m);
                end;
                [~,index_subsidy(m)]                    = min(abs(dummy_e - repmat(e_subsidy(m)',[N-1,1])));
                if e_subsidy(m) > 0 && ( e_div(m) > e_subsidy(m) )
                    dummy_e(index_subsidy(m))           = e_subsidy(m);
                end;
                if ~isnan(e_switch_nosubsidy(m))
                    dummy                               = find( dummy_e > e_switch_nosubsidy(m),1,'first');
                    if ~isempty(dummy)
                        index_switch_nosubsidy(m)       = dummy;
                    else
                        index_switch_nosubsidy(m)       = NaN;
                    end;
                end;
                
           else
               
               dummy_e    = exogenous_e(:,m);
               if ~isnan(e_switch_subsidy(m))
                    [~,index_switch_subsidy(m)]         = min(abs(dummy_e - repmat(e_switch_subsidy(m)',[N-1,1])));
                    dummy_e(index_switch_subsidy(m))    = e_switch_subsidy(m);
                end;
                [~,index_subsidy(m)]                    = min(abs(dummy_e - repmat(e_subsidy(m)',[N-1,1])));
                if e_subsidy(m) > 0 && ( e_div(m) > e_subsidy(m) )
                    dummy_e(index_subsidy(m))           = e_subsidy(m);
                end;
                if ~isnan(e_switch_nosubsidy(m))
                    dummy                               = find( dummy_e > e_switch_nosubsidy(m),1,'first');
                    if ~isempty(dummy)
                        index_switch_nosubsidy(m)       = dummy;
                    else
                        index_switch_nosubsidy(m)       = NaN;
                    end;
                end;
               
           end;

           % State-space for productivity
           dummy_z = repmat(pars.z_augmented_nosubsidy(m,:),[(N-1),1]); 
           dummy_z(dummy_e <= e_subsidy(m),(2+N_z+1):end) = repmat(pars.z_augmented_subsidy((end-2):end),[sum(dummy_e <= e_subsidy(m)) 1]);

           % Indexes for mixed debt / bond debt regions
           if isempty(exogenous_e)
               if ~isnan(e_switch_subsidy(m)) && ~isnan(e_switch_nosubsidy(m))
                indexes_r  = ((dummy_e >= e_eq(m))&( dummy_e <= e_switch_subsidy(m) ))|((dummy_e >= e_subsidy(m) )&(dummy_e <= e_switch_nosubsidy(m)));
                indexes_nr = ((dummy_e >= e_switch_subsidy(m))&( dummy_e <= e_subsidy(m) ))|((dummy_e >= e_switch_nosubsidy(m) )&(dummy_e <= e_div(m)));
               elseif ~isnan(e_switch_subsidy(m)) && isnan(e_switch_nosubsidy(m))
                indexes_r  = ((dummy_e >= e_eq(m))&( dummy_e <= e_switch_subsidy(m) ));
                indexes_nr = ((dummy_e >= e_switch_subsidy(m))&( dummy_e <= e_subsidy(m) ))|(dummy_e >= e_subsidy(m) );
               elseif isnan(e_switch_subsidy(m)) && ~isnan(e_switch_nosubsidy(m)) && (e_div(m) > e_switch_nosubsidy(m))
                indexes_r  = 1:index_switch_nosubsidy(m);
                indexes_nr  = index_switch_nosubsidy(m):(N-1);
               elseif isnan(e_switch_subsidy(m)) && ~isnan(e_switch_nosubsidy(m)) && (e_div(m) <= e_switch_nosubsidy(m))
                indexes_r  = [];
                indexes_nr = (1:(N-1))';   
               else
                indexes_r  = (1:(N-1))';
                indexes_nr = [];
               end;
           else
               if ~isnan(e_switch_nosubsidy(m))
                   index_switch_nosubsidy(m)          = find( dummy_e > e_switch_nosubsidy(m),1,'first');
                   indexes_r                          = 1:index_switch_nosubsidy(m);
                   indexes_nr                         = index_switch_nosubsidy(m):(N-1);
               else
                   indexes_r                          = (1:(N-1))';
                   indexes_nr                         = [];
               end;
           end;

           % Solution in the mixed-debt region
           if sum(indexes_r) > 0  
               return_more = true; MV_first = false; algorithm = 1; e_r = dummy_e(indexes_r); z_r = dummy_z(indexes_r,:);
               [dummy_V_r,dummy_s_hat_r,dummy_k_hat_r,~,dummy_phi_d_r,dummy_phi_b] = optimal_policy_r( e_r, z_r,  A, B, ecv, return_more, MV_first, algorithm, pars);        
               dummy = NaN(N-1,1); dummy(indexes_r) = dummy_V_r; V_r(:,m) = dummy;
               dummy = NaN(N-1,1); dummy(indexes_r) = dummy_V_r; V_r(:,m) = dummy;
               dummy = NaN(N-1,1); dummy(indexes_r) = dummy_s_hat_r; s_hat_r(:,m) = dummy;
               dummy = NaN(N-1,1); dummy(indexes_r) = dummy_k_hat_r; k_hat_r(:,m) = dummy;
               dummy = NaN(N-1,1); dummy(indexes_r) = dummy_phi_d_r; phi_d_r(:,m) = dummy;
               dummy = NaN(N-1,1); dummy(indexes_r) = dummy_phi_b; phi_b(:,m) = dummy;
           end;

           % Solution in the bond debt region
           if sum(indexes_nr) > 0
               e_nr = dummy_e(indexes_nr); z_nr = dummy_z(indexes_nr,:); return_VF = false;
               [~,dummy_V_nr,dummy_k_hat_nr,dummy_phi_d_nr] = optimal_policy_nr( e_nr, z_nr, A, B, ecv, return_VF, pars);
               dummy = NaN(N-1,1); dummy(indexes_nr) = dummy_V_nr; V_nr(:,m) = dummy;
               dummy = NaN(N-1,1); dummy(indexes_nr) = dummy_k_hat_nr; k_hat_nr(:,m) = dummy;
               dummy = NaN(N-1,1); dummy(indexes_nr) = dummy_phi_d_nr; phi_d_nr(:,m) = dummy;
           end;

           e_save{m} = dummy_e;
           z_save{m} = dummy_z;

        end;
        
else
    
   parfor m=1:N_z
    
       % Thresholds for net worth
       tval = [pars.gamma_e;0]; update_e_min = false;    
       [e_eq(m),e_switch_nosubsidy(m),e_div(m),e_switch_subsidy(m)] = thresholds_e( m, tval, update_e_min, A, B, ecv, pars );
       e_subsidy(m) = pars.e_subsidy;
       
       % State-space for net worth
       if isempty(exogenous_e)
            dummy_e                     = intspace(e_eq(m),e_div(m),N-1,pars.log_space);
            %{
            if ~isnan(e_switch_subsidy(m))
                [~,index_switch_subsidy(m)]         = min(abs(dummy_e - repmat(e_switch_subsidy(m)',[N-1,1])));
                dummy_e(index_switch_subsidy(m))    = e_switch_subsidy(m);
            end;
            [~,index_subsidy(m)]         = min(abs(dummy_e - repmat(e_subsidy(m)',[N-1,1])));
            if e_subsidy(m) > 0
                dummy_e(index_subsidy(m))    = e_subsidy(m);
            end;
            if ~isnan(e_switch_nosubsidy(m))
                index_switch_nosubsidy(m)          = find( dummy_e > e_switch_nosubsidy(m),1,'first');
            end;
            %}
       else
           dummy_e    = exogenous_e(:,m);
       end;
       
       % State-space for productivity
       dummy_z = repmat(pars.z_augmented_nosubsidy(m,:),[(N-1),1]); 
       dummy_z(dummy_e <= e_subsidy(m),(2+N_z+1):end) = repmat(pars.z_augmented_subsidy((end-2):end),[sum(dummy_e <= e_subsidy(m)) 1]);
       
       % Indexes for mixed debt / bond debt regions
       indexes_nr= (1:(N-1))';

       % Solution in the bond debt region
       if sum(indexes_nr) > 0
           e_nr = dummy_e(indexes_nr); z_nr = dummy_z(indexes_nr,:); return_VF = false;
           [~,dummy_V_nr,dummy_k_hat_nr,dummy_phi_d_nr] = optimal_policy_nr( e_nr, z_nr, A, B, ecv, return_VF, pars);
           dummy = NaN(N-1,1); dummy(indexes_nr) = dummy_V_nr; V_nr(:,m) = dummy;
           dummy = NaN(N-1,1); dummy(indexes_nr) = dummy_k_hat_nr; k_hat_nr(:,m) = dummy;
           dummy = NaN(N-1,1); dummy(indexes_nr) = dummy_phi_d_nr; phi_d_nr(:,m) = dummy;
       end;

       e_save{m} = dummy_e;
       z_save{m} = dummy_z;
       
   end;
   
end;    

% Update value function and default threshold
V = V_nr; phi_d = phi_d_nr;
for m=1:N_z
    V(~isnan(s_hat_r(:,m)),m) = V_r(~isnan(s_hat_r(:,m)),m);
    phi_d(~isnan(s_hat_r(:,m)),m) = phi_d_r(~isnan(s_hat_r(:,m)),m);
end;  

% Implied iterates
z = [];
for m=1:N_z
    e(:,m) = e_save{m};
    z = [z;z_save{m}];
end;
[Ap,Bp,ecvp] = make_continuation_matrices(V,e,pars);

%Policy functions
policy_functions                          = {};
policy_functions.e                        = e;
policy_functions.z                        = z;
policy_functions.e_switch_subsidy         = e_switch_subsidy;
policy_functions.e_switch_nosubsidy       = e_switch_nosubsidy;
policy_functions.phi_d_r                  = phi_d_r;
policy_functions.phi_b                    = phi_b;
policy_functions.k_hat_r                  = k_hat_r;
policy_functions.phi_d_nr                 = phi_d_nr;
policy_functions.k_hat_nr                 = k_hat_nr;
policy_functions.s_hat_r                  = s_hat_r;
policy_functions.V                        = V;

% Print net worth thresholds to keep track of convergence
if print_output
    for m=1:N_z
        fprintf('Thresholds for net worth for m = %d  \n',m);
        fprintf('   e_min                   = %7.3f \n',policy_functions.e(1,m));
        fprintf('   e_switch (no subs)      = %7.3f \n',policy_functions.e_switch_nosubsidy(m));
        fprintf('   e_switch (no subs)      = %7.3f \n',policy_functions.e_switch_subsidy(m));
        fprintf('   e_div                   = %7.3f \n',policy_functions.e(end,m));
        indexes_nr = ~isnan(policy_functions.k_hat_nr(:,m)); indexes_r = ~isnan(policy_functions.k_hat_r(:,m)); 
        if sum(indexes_nr) > 0
            af = policy_functions.k_hat_nr(indexes_nr,m); bf = policy_functions.k_hat_r(indexes_r,m);
            fprintf('   Drop in assets          = %7.3f \n',(bf(end)-af(1))/bf(end));
        end;
    end;
end;    
               
end

