function [phi_d,PD,R,LRP,L,dLdphi,dLdk,dphidk,dphids,dphide,N,dNdphi,dNdk] = lending_contract_m(k,s,e,a,b,z,pars)

nogood = (e < 0 | k < 0); numargs_e = numel(e); numargs_k = numel(k); numargs_a = numel(a); numargs_b = numel(b);

if sum(nogood) > 0
    fprintf('e < 0 or k < 0 in lending_contract_m \n');
end;

if ( numargs_e ~= numargs_k || numargs_e ~= numargs_a || numargs_e ~= numargs_b )
    fprintf('e, k, a and b do not have the same number of elements in lending_contract_m \n');
end;

r_m = z(:,end);
r_hat_m = (1+pars.r)*(1-s)/(1-pars.chi) - 1;
r_tilde_m = (1+r_m).*(1-s)./(1-pars.chi) - 1;
chi_m = z(:,(end-2));
delta_hat = 1-(pars.chi_L - chi_m)./(1-chi_m).*(1-pars.delta_k);

case0 = ( e > 0 )&( k > 0 )&( k < e );
case1 = ( r_tilde_m + delta_hat > 0  )&( ( e > 0 )&( k >= e  )&( k < (1+r_tilde_m)./(delta_hat+r_tilde_m).*e ) );
case2 = ( r_tilde_m + delta_hat > 0  )&( ( ( e > 0 )&( k >= ((1+r_tilde_m)./(delta_hat+r_tilde_m).*e) ) ) | ( e == 0 )&( k > 0) );
case3 = ( r_tilde_m + delta_hat <= 0 )&( ( ( e > 0 )&( k >= e  ) | ( e == 0 )&( k > 0 ) ) );

phi_d = NaN(numargs_e,1); N = phi_d;
dNdphi = phi_d; dNdk = phi_d; dphidk = phi_d; dphide = phi_d; dphids = phi_d;
L = phi_d; dLdk = phi_d; dLdphi = phi_d;

if sum(case0) > 0
    
    phi_d(case0)    = (r_hat_m(case0)+delta_hat(case0)).*(k(case0).^(-pars.zeta)).*(k(case0) - (1+r_hat_m(case0))./(delta_hat(case0)+r_hat_m(case0)).*e(case0));
    N(case0)        = phi_d(case0);
    L(case0)        = 0;
    
    dNdphi(case0)   = 1;
    dNdk(case0)     = 0;
    
    dLdphi(case0) = 0;
    dLdk(case0)   = 0;
    
    dphidk(case0) = (r_hat_m(case0)+delta_hat(case0)).*(-pars.zeta*(k(case0).^(-pars.zeta-1)).*(k(case0) - (1+r_hat_m(case0))./(delta_hat(case0)+r_hat_m(case0)).*e(case0)) + k(case0).^(-pars.zeta) );
    dphids(case0) = -(1+pars.r)./(1-pars.chi)*k(case0).^(-pars.zeta).*(k(case0) - e(case0));
    dphide(case0) = -(1+r_hat_m(case0)).*(k(case0).^(-pars.zeta));
    
end;

if sum(case1) > 0
    
    phi_d(case1)  = (r_tilde_m(case1)+delta_hat(case1)).*(k(case1).^(-pars.zeta)).*(k(case1) - (1+r_tilde_m(case1))./(delta_hat(case1)+r_tilde_m(case1)).*e(case1));
    N(case1)      = phi_d(case1);
    L(case1)      = 0;
    
    dNdphi(case0) = 1;
    dNdk(case0)   = 0;
    
    dLdphi(case1) = 0;
    dLdk(case1)   = 0;
    
    dphidk(case1) = (r_tilde_m(case1)+delta_hat(case1)).*(-pars.zeta*(k(case1).^(-pars.zeta-1)).*(k(case1) - (1+r_tilde_m(case1))./(delta_hat(case1)+r_tilde_m(case1)).*e(case1)) + k(case1).^(-pars.zeta) );
    dphids(case1) = -(1+r_m(case1))./(1-pars.chi).*k(case1).^(-pars.zeta).*(k(case1) - e(case1));
    dphide(case1) = -(1+r_tilde_m(case1)).*(k(case1).^(-pars.zeta));
    
end;

if sum(case2) > 0
    
    phi_d_low  = zeros(sum(case2),1);
    phi_d_high = maximum_default_m(k(case2),a(case2),b(case2),z(case2,:),pars);
    k2 = k(case2); e2 = e(case2); a2 = a(case2); b2 = b(case2); r_tilde_m2 = r_tilde_m(case2); delta_hat2 = delta_hat(case2);
    ff = @(x,indexes) ( N_fun(x,-(1-delta_hat2(indexes)).*(k2(indexes).^(1-pars.zeta)),a2(indexes),b2(indexes),pars) - (k2(indexes).^(-pars.zeta)).*(delta_hat2(indexes)+r_tilde_m2(indexes)).*(k2(indexes) - (1+r_tilde_m2(indexes))./(delta_hat2(indexes)+r_tilde_m2(indexes)).*e2(indexes)) );
    x_low = phi_d_low; x_high = phi_d_high; tol_f = pars.tol_f_contr; tol_x = pars.tol_x_contr;
    return_warning{1} = true; return_warning{2} = 'lending_contract';
    phi_d(case2) = bissection(x_low,x_high,ff,tol_f,tol_x,return_warning);
    
    N(case2)      = N_fun(phi_d(case2),-(1-delta_hat2).*(k(case2).^(1-pars.zeta)),a(case2),b(case2),pars);
    L(case2)      = L_fun(phi_d(case2),k(case2),a(case2),b(case2),pars);
    
    F2            = 1 - exp( -( (phi_d(case2)./a(case2)).^(b(case2)) ) );
    f2            =  (b(case2)./a(case2)).*(phi_d(case2)./a(case2)).^(b(case2)-1).*exp( -( (phi_d(case2)./a(case2)).^(b(case2)) ) );
    
    dNdphi(case2) = 1 - F2 - f2.*( phi_d(case2) + (1-delta_hat2).*(k(case2).^(1-pars.zeta)) );
    dNdk(case2)   = -(1-delta_hat2).*(1-pars.zeta).*(k(case2).^(-pars.zeta)).*F2;
    
    dLdphi(case2) = f2.*( phi_d(case2) + (1-delta_hat2).*(k(case2).^(1-pars.zeta)) );
    dLdk(case2)   = (1-delta_hat2).*(1-pars.zeta).*(k(case2).^(-pars.zeta)).*F2;
    
    dphidk(case2) = ( (r_tilde_m(case2)+delta_hat2).*( -pars.zeta*(k(case2).^(-pars.zeta-1)).*(k(case2) - (1+r_tilde_m(case2))./(delta_hat2+r_tilde_m(case2)).*e(case2)) + k(case2).^(-pars.zeta) ) - dNdk(case2) )./max(0,dNdphi(case2));
    dphids(case2) = -(1+r_m(case2))./(1-pars.chi).*k(case2).^(-pars.zeta).*(k(case2) - e(case2)).*1./(max(0,dNdphi(case2)));
    dphide(case2) = -(1+r_tilde_m(case2)).*(k(case2).^(-pars.zeta))./(max(0,dNdphi(case2)));
    
end;

if sum(case3) > 0
    
    phi_d(case3)  = (r_tilde_m(case3)+delta_hat(case3)).*(k(case3).^(1-pars.zeta)) - (1+r_tilde_m(case3)).*e(case3).*(k(case3).^(-pars.zeta));
    N(case3)      = phi_d(case3);
    L(case3)      = 0;
    
    dNdphi(case3) = 1;
    dNdk(case3)   = 0;
    
    dLdphi(case3) = 0;
    dLdk(case3)   = 0;
    
    dphidk(case3) = (r_tilde_m(case3)+delta_hat(case3)).*((1-pars.zeta)*(k(case3).^(-pars.zeta))) - (1+r_tilde_m(case3)).*e(case3).*(-pars.zeta*(k(case3).^(-pars.zeta-1)));
    dphids(case3) = -(1+r_m(case3))./(1-pars.chi).*k(case3).^(-pars.zeta).*(k(case3) - e(case3));
    dphide(case3) = -(1+r_tilde_m(case3)).*(k(case3).^(-pars.zeta));
    
end;

% Extensions at k=0
phi_d( k == 0 & e > 0 ) = -Inf;
phi_d( k == 0 & e == 0 )= 0;
N( k == 0 & e > 0 ) = -Inf;
N( k == 0 & e == 0 )= 0;
dNdphi(k == 0) = 1;
dNdk(k==0) = 0;
dphidk((k == 0)&( r_hat_m + delta_hat > 0))  = Inf;
dphidk((k == 0)&( r_hat_m + delta_hat <= 0)) = -Inf;
dphide(k == 0) = -Inf;
dphids(k == 0) = 0;
dLdk(k == 0) = 0;
dLdphi(k == 0) = 0;

% Other output
R = (1-pars.chi)*(phi_d.*(k.^(pars.zeta)) + (1-delta_hat).*k); R(k==0) = -(1+pars.r).*(1-s(k==0)).*e(k==0);
PD = 1 - exp( -( (phi_d./a).^(b) ) ); PD(phi_d <= 0) = 0;
LRP = (k.^(pars.zeta)).*( phi_d -  N )./(k-e); LRP(k <= (1+r_tilde_m)./(delta_hat+r_tilde_m).*e ) = 0;

end