function [dWdk,dWds,dWde,W,phi_d,phi_b,R,R_b,R_m,PD,PR,LRP,L,dLdphi,dLdk,dphidk,dphids,dphide,dphi_bdk,CV,dCVdk,dCVds,dCVde] = objective_function_old( k, s, e, z, A, B, ecv, pars, return_VF )

% Objective function for the dynamic problem
% January 2016

numargs     = numel(k); 

if numargs ~= numel(e)
    error('k and e have different sizes in objective_function');
end;

%if min(k) == 0 || min(e) == 0
%    error('objective_function does not take k=0 or e=0 as arguments');
%end;

r_m_r = z(:,end);
if sum( ( k == 0 )&( e == 0 )&( s >=  1./(1+(1-pars.chi)./(1+r_m_r).*(1+pars.r_b)./(pars.chi)) ) ) > 0
    error('objective_function does not take k=e=0 and s > s_r as arguments');
end;

a = z(:,1); b = z(:,2); 
[phi_d,phi_b,R,R_b,R_m,PD,PR,LRP,L,dLdphi,dLdk,dphidk,dphids,dphide,dphi_bdk,dphi_bds,dphi_bde] = lending_contract(k,s,e,a,b,z,pars);
E_phi = a.*gamma(1+1./b);

% Current value
Wcur = NaN(numargs,1); dWcurdk = Wcur; dWcurde = Wcur; dWcurds = Wcur;

case1 = ( k <= e );
case2 = ( k >  e );

if sum(case1) > 0
    
    Wcur(case1)    = ( (1-pars.tau).*E_phi(case1).*k(case1).^(pars.zeta) - (-(1-pars.tau).*(1-pars.delta_k)+1+pars.r).*k(case1) + (1+pars.r)*e(case1) );
    dWcurdk(case1) = pars.zeta*(1-pars.tau).*E_phi(case1).*(k(case1).^(pars.zeta-1)) - (-(1-pars.tau).*(1-pars.delta_k)+1+pars.r); 
    dWcurds(case1) = 0;
    dWcurde(case1) = (1+pars.r);
    
end;

if sum(case2) > 0
    
    case21 = (case2)&(s >= 1./(1+(1-pars.chi)./(1+r_m_r).*(1+pars.r_b)./(pars.chi)) )&( s <= 1 );
    case22 = (case2)&( s == 0 );
    
    if sum(case21+case22) ~= sum(case2)
        error('Wrong cases in objective function');
    end;
    
    if sum(case21) > 0
    
            % First term: repayment region
            phi_bm        = (pars.chi).*phi_b(case21) + (1-pars.chi).*phi_d(case21);
            dphi_bmdk     = (pars.chi).*dphi_bdk(case21) + (1-pars.chi).*dphidk(case21);
            dphi_bmds     = (pars.chi).*dphi_bds(case21) + (1-pars.chi).*dphids(case21);
            dphi_bmde     = (pars.chi).*dphi_bde(case21) + (1-pars.chi).*dphide(case21);
            [IE,IP]       = integral_components(phi_b(case21),Inf(sum(case21),1),phi_bm,a(case21),b(case21));
            f_phib        = (b(case21)./a(case21)).*(phi_b(case21)./a(case21)).^(b(case21)-1).*exp( -( (phi_b(case21)./a(case21)).^(b(case21)) ) );
            IEx           = -f_phib.*(phi_b(case21) - phi_bm);
            IEw           = -IP;
            Wcur_P        = (1-pars.tau).*IE.*(k(case21).^(pars.zeta));
            dWcurdk_P     =   pars.zeta*(1-pars.tau).*IE.*(k(case21).^(pars.zeta-1)) ...
                            + (1-pars.tau).*(k(case21).^(pars.zeta)).*IEx.*dphi_bdk(case21) ...
                            + (1-pars.tau).*(k(case21).^(pars.zeta)).*IEw.*dphi_bmdk(case21) ;
            dWcurds_P     =  (1-pars.tau).*(k(case21).^(pars.zeta)).*IEx.*dphi_bds(case21) ...
                            + (1-pars.tau).*(k(case21).^(pars.zeta)).*IEw.*dphi_bmds(case21) ;
            dWcurde_P     =  (1-pars.tau).*(k(case21).^(pars.zeta)).*IEx.*dphi_bde(case21) ...
                            + (1-pars.tau).*(k(case21).^(pars.zeta)).*IEw.*dphi_bmde(case21) ; 
                        
            % Second term: renegotiation region
            [IE,IP]       = integral_components(phi_d(case21),phi_b(case21),phi_d(case21),a(case21),b(case21));
            f_phib        = (b(case21)./a(case21)).*(phi_b(case21)./a(case21)).^(b(case21)-1).*exp( -( (phi_b(case21)./a(case21)).^(b(case21)) ) );
            IEy           = f_phib.*(phi_b(case21) - phi_d(case21));
            IEw           = -IP;
            Wcur_R        = (1-pars.tau).*(1-pars.chi).*IE.*(k(case21).^(pars.zeta));
            dWcurdk_R     =   pars.zeta*(1-pars.tau).*(1-pars.chi).*IE.*(k(case21).^(pars.zeta-1)) ...
                            + (1-pars.tau).*(1-pars.chi).*(k(case21).^(pars.zeta)).*IEy.*dphi_bdk(case21) ...
                            + (1-pars.tau).*(1-pars.chi).*(k(case21).^(pars.zeta)).*IEw.*dphidk(case21) ;
            dWcurds_R     =  (1-pars.tau).*(1-pars.chi).*(k(case21).^(pars.zeta)).*IEy.*dphi_bds(case21) ...
                            + (1-pars.tau).*(1-pars.chi).*(k(case21).^(pars.zeta)).*IEw.*dphids(case21) ;
            dWcurde_R     =  (1-pars.tau).*(1-pars.chi).*(k(case21).^(pars.zeta)).*IEy.*dphi_bde(case21) ...
                            + (1-pars.tau).*(1-pars.chi).*(k(case21).^(pars.zeta)).*IEw.*dphide(case21) ;             

            % Take the sum
            Wcur(case21)          = Wcur_R    + Wcur_P;
            dWcurdk(case21)       = dWcurdk_R + dWcurdk_P;
            dWcurds(case21)       = dWcurds_R + dWcurds_P;
            dWcurde(case21)       = dWcurde_R + dWcurde_P;
                        
    end;
    
    if sum(case22) > 0
                        
            % First term: payment region
            [IE,IP]       = integral_components(phi_d(case22),Inf(sum(case22),1),phi_d(case22),a(case22),b(case22));
            IEw           = -IP;
            Wcur_P        = (1-pars.tau).*IE.*(k(case22).^(pars.zeta));
            dWcurdk_P     =   pars.zeta*(1-pars.tau).*IE.*(k(case22).^(pars.zeta-1)) ...
                            + (1-pars.tau).*(k(case22).^(pars.zeta)).*IEw.*dphidk(case22) ;
            dWcurds_P     = (1-pars.tau).*(k(case22).^(pars.zeta)).*IEw.*dphids(case22) ;
            dWcurde_P     = (1-pars.tau).*(k(case22).^(pars.zeta)).*IEw.*dphide(case22) ;             

            % Take the sum
            Wcur(case22)          = Wcur_P;
            dWcurdk(case22)       = dWcurdk_P;
            dWcurds(case22)       = dWcurds_P;
            dWcurde(case22)       = dWcurde_P;
                        
    end;
    
end;

if ~(max(abs(A(:))) == 0)

    % Net continuation value
    CV = NaN(numargs,size(A,1)); dCVdk = CV; dCVde = CV; dCVds = CV; N_z = pars.N_z;
    
    case3 = (s == 0);
    case4 = ( s >=  1./(1+(1-pars.chi)./(1+r_m_r).*(1+pars.r_b)./(pars.chi)) )&( s <= 1 );

    if sum(case3+case4) ~= numargs
        error('incorrect cases in objective_function');
    end;

    % Contracts without renegotiation
    if sum(case3) > 0

        for mp=1:size(A,1)
            A_mat = repmat(A(mp,:),[numargs,1]); B_mat = repmat(B(mp,:),[numargs,1]); ecv_mat = repmat(ecv(mp,:),[numargs,1]);
           [CV(case3,mp),dCVdk(case3,mp),dCVde(case3,mp)] = integral_T_NR(k(case3),e(case3),z(case3,:),phi_d(case3),dphidk(case3),dphide(case3),A_mat(case3,:),B_mat(case3,:),ecv_mat(case3,:),pars);
        end;
        dCVds(case3,:) = 0;
        
    end;

    % Contracts with renegotiation
    if sum(case4) > 0

        for mp=1:size(A,1)
           A_mat = repmat(A(mp,:),[numargs,1]); B_mat = repmat(B(mp,:),[numargs,1]); ecv_mat = repmat(ecv(mp,:),[numargs,1]);
           [CV(case4,mp),dCVdk(case4,mp),dCVds(case4,mp),dCVde(case4,mp)] = integral_T_R(k(case4),e(case4),z(case4,:),phi_d(case4),phi_b(case4),dphidk(case4),dphi_bdk(case4),dphids(case4),dphi_bds(case4),dphide(case4),dphi_bde(case4),A_mat(case4,:),B_mat(case4,:),ecv_mat(case4,:),pars);
        end;
        
    end;
    
    % Take expectation
    ppi = z(:,3:(3+N_z-1));
    CV = (ppi.*CV); CV = sum(CV,2);
    dCVdk = (ppi.*dCVdk); dCVdk = sum(dCVdk,2);
    dCVde = (ppi.*dCVde); dCVde = sum(dCVde,2);
    dCVds = (ppi.*dCVds); dCVds = sum(dCVds,2);

else

    CV = 0; dCVdk = 0; dCVde = 0; dCVds = 0; 
        
end;

%Total value function
dWdk = dWcurdk + dCVdk;
dWde = dWcurde + dCVde;
dWds = dWcurds + dCVds;
W = Wcur + CV;

if return_VF
     dWdk = W;
     W = dWdk;
end;

end

