function [dVde,V,k,phi_d] = optimal_policy_nr( e, z, A, B, ecv, return_VF, pars)

% Optimal firm policy in the no-renegotiation region   
s = zeros(size(e));

% Borrowing constraint
k_max = borrowing_constraint_nr(s,e,z,pars);

% Try the first-order approach first
%x_low = (1e-8)*ones(size(e)); x_high = k_max; tol_f = pars.tol_f_obj; tol_x = pars.tol_x_obj; return_VF_local = false;
x_low = zeros(size(e)); x_high = k_max; tol_f = pars.tol_f_obj; tol_x = pars.tol_x_obj; return_VF_local = false;
ff = @(x,indexes) -objective_function(x,s,e(indexes),z(indexes,:),A,B,ecv,pars,return_VF_local);
return_warning{1} = false;
[k,fk] = bissection(x_low,x_high,ff,tol_f,tol_x,return_warning);

% Two cases
case1 = ( fk <= pars.tol_f_obj );
case2 = ( fk > pars.tol_f_obj );

dVde = NaN(size(k)); V = NaN(size(k));

if sum(case1) > 0
    
    % If an interior solution was found, use the envelope theorem to compute dVde
    return_VF_local = false;
    [~,~,dVde(case1),V(case1)] = objective_function(k(case1),s(case1),e(case1),z(case1,:),A,B,ecv,pars,return_VF_local); 
    
end;

if sum(case2) > 0
    
    % If no interior solution was found, use a global method
    tol = pars.tol_f_gss_k; ag = zeros(sum(case2),1); bg = k_max(case2); return_VF_local = true;
    ff = @(x) objective_function(x,s(case2),e(case2),z(case2,:),A,B,ecv,pars,return_VF_local); 
    [k(case2),V(case2)] = golden_section_search( ag, bg, ff, tol );
    
    % Find a solution for e+de
    ep = (e(case2)+pars.de); k_maxp = borrowing_constraint_nr(s(case2),ep,z(case2,:),pars); bg = k_maxp;
    ff = @(x) objective_function(x,s(case2),ep,z(case2,:),A,B,ecv,pars,return_VF_local);
    [~,Vp] = golden_section_search( ag, bg, ff, tol );
    
    % Compute dVde numerically
    dVde(case2) = (Vp - V(case2))/pars.de;    
    
end;

if return_VF
    dummy = V;
    V = dVde;
    dVde = dummy;
end;

if nargout > 3
    a = z(:,1); b = z(:,2); s = 0*ones(size(e)); 
    phi_d = lending_contract(k,s,e,a,b,z,pars);
end;

end


