function [V_r,s_hat_r,k_hat_r,dV_rde,phi_d,phi_b] = optimal_policy_r( e, z, A, B, ecv, return_more, MV_first, algorithm_r, pars)

    % Thresholds of contract space
    [s_r,s_fm,s_mb] = thresholds_s( e, z, pars );
    
    % Output
    k_hat_r = NaN(size(e)); V_r = NaN(size(e)); s_hat_r = NaN(size(e)); dV_rde = NaN(size(e));
    
    if algorithm_r == 1
    
        % Use a golden-section search algorithm.
        % Fully vectorized.
        
        tol = pars.tol_f_gss_s; ag = (1.01)*s_fm; bg = ones(size(e)); MV_first_inside = false;
        ff = @(s) optimal_policy_rs( s, e, z, A, B, ecv, pars, MV_first_inside, s_r, s_fm, s_mb );
        [s_hat_r,V_r] = golden_section_search( ag, bg, ff, tol );
        
    elseif algorithm_r == 2

        % Use fmincon's interior-point algorithm; sometimes faster, especially when b -> 1. 
        % Not vectorized; use only for computing thresholds.
        
        error('algorithm_r = 2 not updated yet.');
        
        %{
        N_z = size(z,1); V_r = NaN(N_z,1); s_hat_r = NaN(N_z,1);
        for n_z=1:N_z
        
            fun       = @(x) fmincon_objective_r(x,e(n_z),z(n_z,:),s_r(n_z),s_fm(n_z),s_mb(n_z),A,B,ecv,pars);
            AA         = [];
            bb         = [];
            AAeq       = [];
            bbeq       = [];
            lb         = min(1.1*s_r(n_z),(1/2)*(s_r(n_z)+s_fm(n_z)));
            ub         = 1;
            nonlcon    = [];

            x0 = min(1.1*s_mb(n_z),(1/2)*(s_mb(n_z)+1));

            options = optimoptions('fmincon','Display','off','Algorithm','interior-point');
            options = optimoptions(options,'GradObj','on','GradConstr','off');
            [x,fval] = fmincon(fun,x0,AA,bb,AAeq,bbeq,lb,ub,nonlcon,options);
            s_hat_r(n_z) = x; V_r(n_z) = -fval;

        end;
        %}

    end;
    
    if return_more
    
        MV_first_inside = true;
        [dV_rde,k_hat_r] = optimal_policy_rs( s_hat_r, e, z, A, B, ecv, pars, MV_first_inside, s_r, s_fm, s_mb );
        a = z(:,1); b = z(:,2);
        [phi_d,phi_b] = lending_contract(k_hat_r,s_hat_r,e,a,b,z,pars);
        
    end;  

    if return_more && MV_first
        
        dummy = dV_rde;
        dV_rde = V_r;
        V_r = dummy;
        
    end;
    

        
end

