function [X,k,dXde,dXds,k_min,k_max,active_case,W,dWdk,dWds,dWde,dk_cds,dk_cde] = optimal_policy_rs( s, e, z, A, B, ecv, pars, MV_first, s_r, s_fm, s_mb )

% Optimal policy in the renegotiation region for a given s
numargs = numel(s); X = NaN(size(s)); dXds = NaN(size(s)); dXde = NaN(size(s)); 
k = NaN(size(s)); fk = ones(size(s));
dWdk = k; dWds = k; dWde = k; dk_cds = k; dk_cde = k;

if nargin < 11
    [s_r,s_fm,s_mb] = thresholds_s( e, z, pars );
end;

if sum( s < s_r ) > 0
    error('Arguments with s<s_r in optimal_policy_n');
end;

% Borrowing constraints
[k_max,dk_maxds,dk_maxde] = borrowing_constraint(s,e,z,pars,s_r,s_fm,s_mb);
[k_min,dk_minds,dk_minde] = minimum_borrowing(s,e,z,pars);

% For e = 0, only search for on k > 0
k_min = min(max(k_min,1e-10),(99/100)*k_min+(1/100)*k_max);

% Compute the derivatives at the boundaries
return_VF_inside  = false;
[dWdk_kmax,dWds_kmax,dWde_kmax,W_kmax]  = objective_function( k_max, s, e, z, A, B, ecv, pars, return_VF_inside );
[dWdk_kmin,dWds_kmin,dWde_kmin,W_kmin]  = objective_function( k_min, s, e, z, A, B, ecv, pars, return_VF_inside );

case1 = (~isnan(dWdk_kmin))&(~isnan(dWdk_kmax))&(dWdk_kmin <= 0 & dWdk_kmax <= 0);
case2 = (~isnan(dWdk_kmin))&(~isnan(dWdk_kmax))&(dWdk_kmin >= 0 & dWdk_kmax >= 0);
case3 = (~isnan(dWdk_kmin))&(~isnan(dWdk_kmax))&(dWdk_kmax <  0 & dWdk_kmin >  0);
case4 = (~isnan(dWdk_kmin))&(~isnan(dWdk_kmax))&(dWdk_kmax >  0 & dWdk_kmin <  0);

if sum(case1+case2+case3+case4) ~= numargs
    save('current_args')
    error('Unexpected cases in optimal_policy_rs');
end;

% The minimum constraint is binding
if sum(case1) > 0

    X(case1)        = W_kmin(case1);
    k(case1)        = k_min(case1); 
    dk_cds(case1)   = dk_minds(case1);
    dk_cde(case1)   = dk_minde(case1);
    W(case1)        = W_kmin(case1);
    dWdk(case1)     = dWdk_kmin(case1);
    dWds(case1)     = dWds_kmin(case1);
    dWde(case1)     = dWde_kmin(case1);
    dXds(case1)     = dWdk_kmin(case1).*dk_cds(case1) + dWds_kmin(case1);
    dXde(case1)     = dWdk_kmin(case1).*dk_cde(case1) + dWde_kmin(case1);

end;

% The maximum constraint is binding
if sum(case2) > 0

    X(case2)        = W_kmax(case2);
    k(case2)        = k_max(case2);
    dk_cds(case2)   = dk_maxds(case2);
    dk_cde(case2)   = dk_maxde(case2);
    W(case2)        = W_kmax(case2);
    dWdk(case2)     = dWdk_kmax(case2);
    dWds(case2)     = dWds_kmax(case2);
    dWde(case2)     = dWde_kmax(case2);
    dXds(case2)     = dWdk_kmax(case2).*dk_cds(case2) + dWds_kmax(case2);
    dXde(case2)     = dWdk_kmax(case2).*dk_cde(case2) + dWde_kmax(case2);

end;

% The solution is interior
if sum(case3) > 0

    % Use the first-order approach
    x_low = k_min(case3); x_high = k_max(case3); tol_f = pars.tol_f_obj; tol_x = pars.tol_x_obj; return_VF_inside = false;
    s3 = s(case3); e3 = e(case3); z3 = z(case3,:);

    %ff = @(x,indexes) -(x.^(-pars.zeta)).*(objective_function(x,s3(indexes),e3(indexes),z3(indexes,:),pars,return_VF_inside));
    ff = @(x,indexes) -objective_function(x,s3(indexes),e3(indexes),z3(indexes,:), A, B, ecv, pars,return_VF_inside);
    return_warning{1} = false;
    [k(case3),fk(case3)] = bissection(x_low,x_high,ff,tol_f,tol_x,return_warning);

    % Detect whether the first-order approach worked
    case31 = (case3)&( fk > pars.tol_f_obj );

    if sum(case31) > 0

        % If no interior solution was found using the first-order approach, use a global method
        tol = pars.tol_f_obj; ag = k_min(case31); bg = k_max(case31); return_VF_inside = true;
        ff = @(x) objective_function(x,s(case31),e(case31),z(case31,:), A, B, ecv,pars,return_VF_inside);
        [k(case31),X(case31)] = golden_section_search( ag, bg, ff, tol );

        % If the global optimum found involves either constraint binding, return an error.  
        if ~( sum(case31) == sum( k(case31) < bg & k(case31) > ag ) )      
            fprintf('Cases not expected in optimal_policy_rs \n');
        end;

    end;

    return_VF_inside = false;
    [dWdk(case3),dWds(case3),dWde(case3),W(case3)] = objective_function( k(case3), s(case3), e(case3), z(case3,:), A, B, ecv, pars, return_VF_inside ); 

    X(case3)    = W(case3);
    dXds(case3) = dWds(case3);
    dXde(case3) = dWde(case3);

end;

% Other cases: use a global method
if sum(case4) > 0

    tol = pars.tol_f_obj; ag = k_min(case4); bg = k_max(case4); return_VF_inside = true;
    ff = @(x) objective_function(x,s(case4),e(case4),z(case4,:), A, B, ecv,pars,return_VF_inside);
    [k(case4),X(case4)] = golden_section_search( ag, bg, ff, tol );
    
    % Compare with values at the boundaries, and adjust if necessary
    case4adjmax = case4&(X <= W_kmax)&(W_kmax >= W_kmin);
    if sum(case4adjmax) > 0
        k(case4adjmax) = k_max(case4adjmax);
        X(case4adjmax) = W_kmax(case4adjmax);
    end;
    case4adjmin = case4&(X <= W_kmin)&(W_kmax <= W_kmin);
    if sum(case4adjmin) > 0
        k(case4adjmin) = k_min(case4adjmin);
        X(case4adjmin) = W_kmin(case4adjmin);
    end;

    % To compute the derivatives, check whether the solution is at the
    % boundary
    case40 = case4&( abs( k_min - k_max ) <= 2*tol );
    case41 = case4&(~case40)&( abs( k - k_min ) <= 2*tol );
    case42 = case4&(~case40)&( abs( k - k_max ) <= 2*tol );
    case43 = case4&(~case40)&( (k_min + 2*tol < k) & (k < k_max - 2*tol) );
    
    if sum(case40+case41+case42+case43) ~= sum(case4)
        error('Unexpected cases with global method in optimal_policy_rs');
    end;
    
    if sum(case40) > 0

        X(case40)        = W_kmax(case40);
        k(case40)        = k_max(case40);
        dk_cds(case40)   = dk_maxds(case40);
        dk_cde(case40)   = dk_maxde(case40);
        W(case40)        = W_kmax(case40);
        dWdk(case40)     = dWdk_kmax(case40);
        dWds(case40)     = dWds_kmax(case40);
        dWde(case40)     = dWde_kmax(case40);
        dXds(case40)     = dWdk_kmax(case40).*dk_cds(case40) + dWds_kmax(case40);
        dXde(case40)     = dWdk_kmax(case40).*dk_cde(case40) + dWde_kmax(case40);

    end;
    
    if sum(case41) > 0
        
        dk_cds(case41)   = dk_minds(case41);
        dk_cde(case41)   = dk_minde(case41);
        W(case41)        = W_kmin(case41);
        dWdk(case41)     = dWdk_kmin(case41);
        dWds(case41)     = dWds_kmin(case41);
        dWde(case41)     = dWde_kmin(case41);
        dXds(case41)     = dWdk_kmin(case41).*dk_cds(case41) + dWds_kmin(case41);
        dXde(case41)     = dWdk_kmin(case41).*dk_cde(case41) + dWde_kmin(case41);

    end;
    
    if sum(case42) > 0

        X(case42)        = W_kmax(case42);
        k(case42)        = k_max(case42);
        dk_cds(case42)   = dk_maxds(case42);
        dk_cde(case42)   = dk_maxde(case42);
        W(case42)        = W_kmax(case42);
        dWdk(case42)     = dWdk_kmax(case42);
        dWds(case42)     = dWds_kmax(case42);
        dWde(case42)     = dWde_kmax(case42);
        dXds(case42)     = dWdk_kmax(case42).*dk_cds(case42) + dWds_kmax(case42);
        dXde(case42)     = dWdk_kmax(case42).*dk_cde(case42) + dWde_kmax(case42);

    end;

    if sum(case43) > 0
    
        return_VF_inside = false;
        [dWdk(case43),dWds(case43),dWde(case43),W(case43)] = objective_function( k(case43), s(case43), e(case43), z(case43,:), A, B, ecv, pars, return_VF_inside ); 

        dXds(case43) = dWds(case43);
        dXde(case43) = dWde(case43);
        
    end;

end;

active_case = [case1 case2 case3 case4];

if MV_first
    dummy = X;
    X = dXde;
    dXde = dummy;
end;
 
end


