function print_model_calibration(pars)

fprintf('Static model parameters : \n')
fprintf('------------------------- \n')
fprintf('zeta               = %2.4f \n',pars.zeta)
fprintf('delta_k            = %2.4f \n',pars.delta_k)
fprintf('chi                = %2.4f \n',pars.chi)
fprintf('beta               = %2.4f \n',pars.beta)
fprintf('r                  = %2.4f \n',pars.r)
fprintf('r_m                = %2.4f \n',pars.r_m)
fprintf('r_b                = %2.4f \n',pars.r_b)
fprintf('\n')

fprintf('Dynamic model parameters : \n')
fprintf('-------------------------- \n')
fprintf('gamma_e                                        = %9.4f \n',pars.gamma_e);
fprintf('eta                                            = %9.4f \n',pars.eta);
fprintf('xi                                             = %9.4f \n',pars.xi);
fprintf('Upper bound on xi for no saving                = %9.4f \n',1 - (1-pars.eta)/pars.eta*pars.gamma_m/(1+pars.r));
if pars.xi < 1 - (1-pars.eta)/pars.eta*(pars.gamma_m/(1+pars.r));
    fprintf('   no saving condition satisfied \n');
else
    error('   no saving condition violated \n');
end;
fprintf('\n');

fprintf('Summary of implied exogenous state distribution : \n');
fprintf('------------------------------------------------- \n');
fprintf('\n');
fprintf('a        b       pi((ap,bp)|(a,b)) \n'); 
formatSpec = ['%4.2f     %4.2f    ',repmat(['%4.2f    '],[1,pars.N_z]),'\n'];
fprintf(formatSpec,pars.z');
fprintf('\n');
for ii=1:pars.N_z
    fprintf('a(%d)                  = %8.4f \n',ii,pars.a(ii))
    fprintf('b(%d)                  = %8.4f \n',ii,pars.b(ii))
    fprintf('E(phi|%d)              = %8.4f \n',ii,pars.E_phi(ii))
    fprintf('var(phi|%d)            = %8.4f \n',ii,pars.var_phi(ii))
    fprintf('E(log(phi)|%d)         = %8.4f \n',ii,double(log(pars.a(ii)) + eulergamma/pars.b(ii)))
    fprintf('var(log(phi)|%d)       = %8.4f \n',ii,pi^2/(pars.b(ii)^2*6))
    fprintf('k_star(%d)             = %8.4f \n',ii,pars.k_star(ii))
    fprintf('e_star(%d)             = %8.4f \n',ii,pars.e_star(ii))
    fprintf('\n')
end;

end

