function [ y, ppi, pi_ergodic, moments ] = rouwenhorst_approximation(N,psi,p,q)

% Constructs the Rouwenhorst Markov Chain described in Kopecky and Suen
% (2010 RED), that can be used to match exactly the first and second moments of
% arbitrary first-order auto-regressive processes. The method is more
% general and more accurate than Tauchen's.

% The state-space is centered at 0; any re-normalization needs to be done
% outside of the code.

% Nicolas Crouzet, April 2016

% State-space
y = linspace(-psi,psi,N)';

% Transition matrix
ppi = [p 1-p;1-q q];
if N >= 3
    counter = 3;
    while counter <= N
        ppi =   p*[ppi, zeros(counter-1,1);zeros(1,counter-1),0] + (1-p)*[zeros(counter-1,1),ppi;0,zeros(1,counter-1)] ...
             +  (1-q)*[zeros(1,counter-1),0;ppi, zeros(counter-1,1)] + q*[0,zeros(1,counter-1);zeros(counter-1,1),ppi];
        ppi(2:(counter-1),:) = (1/2)*ppi(2:(counter-1),:);
        counter = counter+1;
    end;
end;

% Ergodic distribution
s = (1-q)/(2-(p+q));
pi_ergodic = binopdf((0:1:(N-1))',N-1,1-s);

% Moments
moments = {};
moments.uncond_mean = (q-p)*psi/(2-(p+q));
moments.uncond_var = psi^2*( 1 - 4*s*(1-s) + 4*s*(1-s)/(N-1) ) - moments.uncond_mean^2;
moments.autocorr = (p+q-1);

end

