function [e_eq,e_switch_nosubsidy,e_div,e_switch_subsidy] = thresholds_e( m, tval, update_e_min, A, B, ecv, pars )
 
[e_eq,e_switch_nosubsidy,e_div,e_switch_subsidy] = deal(NaN(1,1));
e_min = pars.e_min;

if ~pars.onei
            
        if pars.e_subsidy > 0
            
            % Two steps:
            % Compute solution with subsidy
            % Compute solution without subsidy
            % Mesh the two
            
            % No subsidy
            z               = pars.z_augmented_nosubsidy(m,:);
            if update_e_min == 1

                % Check that the equity issuance cost is not too large, so that dVde >
                % 1/(1-eta)*(beta^(-1)*(1+gamma_e) - eta*xi)
                % This only applies when there is interim equity issuance, i.e. when
                % update_e_min == 1
                test_e_min      = e_min; return_more = true; MV_first = true;
                dVde_min        = optimal_policy_r(test_e_min,z,A,B,ecv,return_more,MV_first,pars.algorithm_r,pars);
                thresh_min      = (1/(pars.beta*(1-pars.eta)))*( ( 1 - pars.eta + pars.eta*(1-pars.xi) ) + tval(1) );
                
                if dVde_min < 0
                    fprintf('Error: negative derivative');
                    error;
                end;

                if dVde_min <= thresh_min

                    e_eq = e_min;

                end;

                if dVde_min > thresh_min

                    % This function solves dV/de(e) = 1+gamma_e
                    x_low = e_min; x_high = pars.k_star_B(m); tol_f = pars.tol_f_thr_min; tol_x = pars.tol_x_thr_min; 
                    ff = @(x,indexes) -optimal_policy_r(x,z,A,B,ecv,return_more,MV_first,pars.algorithm_r,pars) + thresh_min;
                    if sum( ecv(m,2) <= 0 ) == 0 
                        e_low = ecv(m,2); ff_low = ff(e_low,1); x_low(ff_low < 0) = e_low;
                    end;
                    return_warning{1} = true; return_warning{2} = 'solve_threshold'; return_warning{3} = 1;
                    e_eq = bissection(x_low,x_high,ff,tol_f,tol_x,return_warning);

                end;

            else

                e_eq = pars.e_min;

            end;

            % Solve for the dividend issuance threshold under the assumption that it is
            % larger than the switching threshold
            x_low = pars.e_min; x_high = pars.e_star(m); tol_f = pars.tol_f_thr_max; tol_x = pars.tol_x_thr_max;
            tt = tval(2);
            return_VF = false;
            ff = @(x,indexes) -pars.beta*(1-pars.eta)*optimal_policy_nr( x, z(indexes,:), A, B, ecv, return_VF, pars) + (1-pars.tau).*( 1 - pars.eta + pars.eta*(1-pars.xi) ) + tt(indexes);
            return_warning{1} = true; return_warning{2} = 'dividend issuance threshold';

            ff_div = ff(x_high,1);
            while ff_div < 0
                x_high = 1.1*x_high;
                ff_div = ff(x_high,1);
            end;

            e_div = bissection_noedges(x_low,x_high,ff,tol_f,tol_x,return_warning);

            % Check whether V_r < V_nr at the dividend issuance and equity issuance thresholds
            return_VF = true; return_more = false; MV_first = false;
            ff = @(x,indexes) optimal_policy_nr( x, z(indexes,:), A, B, ecv, return_VF, pars) - optimal_policy_r(x,z(indexes,:), A, B, ecv, return_more,MV_first,pars.algorithm_r, pars);
            ff_div = ff(e_div,1);
            ff_eq = ff(e_eq,1);

            if ff_div <= 0 && ff_eq <= 0

                % If V_r >= V_nr, re-compute the dividend issuance threshold using
                % V_r and set e_switch = e_div
                x_low = pars.e_min; x_high = pars.k_star_B(m); tol_f = pars.tol_f_thr_max; tol_x = pars.tol_x_thr_max;
                tt = tval(2);
                return_more = true; MV_first = true;
                ff = @(x,indexes) -pars.beta*(1-pars.eta)*optimal_policy_r( x, z(indexes,:), A, B, ecv, return_more,MV_first,pars.algorithm_r,pars) +  (1-pars.tau).*( 1 - pars.eta + pars.eta*(1-pars.xi) ) + tt(indexes);
                return_warning{1} = true; return_warning{2} = 'dividend issuance threshold';
                e_div = bissection_noedges(x_low,x_high,ff,tol_f,tol_x,return_warning);
                e_switch = e_div;

            elseif ff_div > 0 && ff_eq <= 0

                % If V_r < V_nr, compute switching threshold
                x_low = e_eq; x_high = e_div; tol_f = pars.tol_f_thr_switch; tol_x = pars.tol_x_thr_switch;
                return_VF = true; return_more = false; MV_first = false; 
                ff = @(x,indexes) 1/(-ff_eq)*(optimal_policy_nr( x, z(indexes,:), A, B, ecv, return_VF, pars) - optimal_policy_r(x,z(indexes,:), A, B, ecv, return_more,MV_first,pars.algorithm_r, pars));
                return_warning{1} = true; return_warning{2} = 'switching threshold'; 
                e_switch = bissection_noedges(x_low,x_high,ff,tol_f,tol_x,return_warning);

            else 

                % If V_r < V_nr at ff_eq = 0, re-compute the entry size
                x_low = e_min; x_high = pars.k_star_B(m); tol_f = pars.tol_f_thr_min; tol_x = pars.tol_x_thr_min; 
                ff = @(x,indexes) -optimal_policy_nr(x, z(1,:), A, B, ecv, false, pars) + thresh_min;
                if sum( ecv(m,2) <= 0 ) == 0 
                    e_low = ecv(m,2); ff_low = ff(e_low,1); x_low(ff_low < 0) = e_low;
                end;
                return_warning{1} = true; return_warning{2} = 'solve_threshold'; return_warning{3} = 1;
                e_eq = bissection(x_low,x_high,ff,tol_f,tol_x,return_warning);
                e_switch = 2*e_div;

            end;

            e_switch_nosubsidy = e_switch;
            e_div_nosubsidy    = e_div;
            e_eq_nosubsidy     = e_eq;
            
            % Subsidy
            z               = pars.z_augmented_subsidy(m,:);
            if update_e_min == 1

                % Check that the equity issuance cost is not too large, so that dVde >
                % 1/(1-eta)*(beta^(-1)*(1+gamma_e) - eta*xi)
                % This only applies when there is interim equity issuance, i.e. when
                % update_e_min == 1
                test_e_min      = e_min; return_more = true; MV_first = true;
                dVde_min        = optimal_policy_r(test_e_min,z,A,B,ecv,return_more,MV_first,pars.algorithm_r,pars);
                thresh_min      = (1/(pars.beta*(1-pars.eta)))*( ( 1 - pars.eta + pars.eta*(1-pars.xi) ) + tval(1) );
                
                if dVde_min < 0
                    fprintf('Error: negative derivative');
                    error;
                end;

                if dVde_min <= thresh_min

                    e_eq = e_min;

                end;

                if dVde_min > thresh_min

                    % This function solves dV/de(e) = 1+gamma_e
                    x_low = e_min; x_high = pars.k_star_B(m); tol_f = pars.tol_f_thr_min; tol_x = pars.tol_x_thr_min; 
                    ff = @(x,indexes) -optimal_policy_r(x,z,A,B,ecv,return_more,MV_first,pars.algorithm_r,pars) + thresh_min;
                    if sum( ecv(m,2) <= 0 ) == 0 
                        e_low = ecv(m,2); ff_low = ff(e_low,1); x_low(ff_low < 0) = e_low;
                    end;
                    return_warning{1} = true; return_warning{2} = 'solve_threshold'; return_warning{3} = 1;
                    e_eq = bissection(x_low,x_high,ff,tol_f,tol_x,return_warning);

                end;

            else

                e_eq = pars.e_min;

            end;

            % Solve for the dividend issuance threshold under the assumption that it is
            % larger than the switching threshold
            x_low = pars.e_min; x_high = pars.e_star(m); tol_f = pars.tol_f_thr_max; tol_x = pars.tol_x_thr_max;
            tt = tval(2);
            return_VF = false;
            ff = @(x,indexes) -pars.beta*(1-pars.eta)*optimal_policy_nr( x, z(indexes,:), A, B, ecv, return_VF, pars) + (1-pars.tau).*( 1 - pars.eta + pars.eta*(1-pars.xi) ) + tt(indexes);
            return_warning{1} = true; return_warning{2} = 'dividend issuance threshold';

            ff_div = ff(x_high,1);
            while ff_div < 0
                x_high = 1.1*x_high;
                ff_div = ff(x_high,1);
            end;

            e_div = bissection_noedges(x_low,x_high,ff,tol_f,tol_x,return_warning);

            % Check whether V_r < V_nr at the dividend issuance and equity issuance thresholds
            return_VF = true; return_more = false; MV_first = false;
            ff = @(x,indexes) optimal_policy_nr( x, z(indexes,:), A, B, ecv, return_VF, pars) - optimal_policy_r(x,z(indexes,:), A, B, ecv, return_more,MV_first,pars.algorithm_r, pars);
            ff_div = ff(e_div,1);
            ff_eq = ff(e_eq,1);

            if ff_div <= 0 && ff_eq <= 0

                % If V_r >= V_nr, re-compute the dividend issuance threshold using
                % V_r and set e_switch = e_div
                x_low = pars.e_min; x_high = pars.k_star_B(m); tol_f = pars.tol_f_thr_max; tol_x = pars.tol_x_thr_max;
                tt = tval(2);
                return_more = true; MV_first = true;
                ff = @(x,indexes) -pars.beta*(1-pars.eta)*optimal_policy_r( x, z(indexes,:), A, B, ecv, return_more,MV_first,pars.algorithm_r,pars) +  (1-pars.tau).*( 1 - pars.eta + pars.eta*(1-pars.xi) ) + tt(indexes);
                return_warning{1} = true; return_warning{2} = 'dividend issuance threshold';
                e_div = bissection_noedges(x_low,x_high,ff,tol_f,tol_x,return_warning);
                e_switch = e_div;

            elseif ff_div > 0 && ff_eq <= 0

                % If V_r < V_nr, compute switching threshold
                x_low = e_eq; x_high = e_div; tol_f = pars.tol_f_thr_switch; tol_x = pars.tol_x_thr_switch;
                return_VF = true; return_more = false; MV_first = false; 
                ff = @(x,indexes) 1/(-ff_eq)*(optimal_policy_nr( x, z(indexes,:), A, B, ecv, return_VF, pars) - optimal_policy_r(x,z(indexes,:), A, B, ecv, return_more,MV_first,pars.algorithm_r, pars));
                return_warning{1} = true; return_warning{2} = 'switching threshold'; 
                e_switch = bissection_noedges(x_low,x_high,ff,tol_f,tol_x,return_warning);

            else 

                % If V_r < V_nr at ff_eq = 0, re-compute the entry size
                x_low = e_min; x_high = pars.k_star_B(m); tol_f = pars.tol_f_thr_min; tol_x = pars.tol_x_thr_min; 
                ff = @(x,indexes) -optimal_policy_nr(x, z(1,:), A, B, ecv, false, pars) + thresh_min;
                if sum( ecv(m,2) <= 0 ) == 0 
                    e_low = ecv(m,2); ff_low = ff(e_low,1); x_low(ff_low < 0) = e_low;
                end;
                return_warning{1} = true; return_warning{2} = 'solve_threshold'; return_warning{3} = 1;
                e_eq = bissection(x_low,x_high,ff,tol_f,tol_x,return_warning);
                e_switch = 2*e_div;

            end;

            e_switch_subsidy = e_switch;
            e_div_subsidy    = e_div;
            e_eq_subsidy     = e_eq;
            e_eq             = e_eq_subsidy;
            e_subsidy        = pars.e_subsidy;
            
            if e_subsidy > e_div_subsidy
                e_div               = e_div_subsidy;
                e_switch_nosubsidy  = NaN;
            elseif e_subsidy <= e_div_subsidy && e_subsidy > e_div_nosubsidy
                e_div               = e_subsidy;
                e_switch_nosubsidy  = NaN;
            elseif e_subsidy <= e_div_nosubsidy && e_subsidy > e_switch_nosubsidy
                e_div               = e_div_nosubsidy;
                e_switch_nosubsidy  = NaN;
            elseif e_subsidy <= e_switch_nosubsidy && e_subsidy > e_switch_subsidy    
                e_div               = e_div_nosubsidy;
                e_switch_nosubsidy  = NaN;
            elseif e_subsidy <= e_switch_subsidy
                e_div               = e_div_nosubsidy;
                e_switch_subsidy    = NaN;
            else
                error('Non-treated case');
            end;
            
            e_subsidy
            [e_eq e_switch_subsidy e_switch_nosubsidy e_div]

        else

            if update_e_min == 1

                % Check that the equity issuance cost is not too large, so that dVde >
                % 1/(1-eta)*(beta^(-1)*(1+gamma_e) - eta*xi)
                % This only applies when there is interim equity issuance, i.e. when
                % update_e_min == 1
                z               = pars.z_augmented_nosubsidy(m,:);
                test_e_min      = e_min; return_more = true; MV_first = true;
                dVde_min        = optimal_policy_r(test_e_min,z,A,B,ecv,return_more,MV_first,pars.algorithm_r,pars);
                thresh_min      = (1/(pars.beta*(1-pars.eta)))*( ( 1 - pars.eta + pars.eta*(1-pars.xi) ) + tval(1) );
                
                if dVde_min < 0
                    fprintf('Error: negative derivative');
                    error;
                end;

                if dVde_min <= thresh_min

                    e_eq = e_min;

                end;

                if dVde_min > thresh_min

                    % This function solves dV/de(e) = 1+gamma_e
                    z = pars.z_augmented_nosubsidy(m,:);
                    x_low = e_min; x_high = pars.k_star_B(m); tol_f = pars.tol_f_thr_min; tol_x = pars.tol_x_thr_min; 
                    ff = @(x,indexes) -optimal_policy_r(x,z,A,B,ecv,return_more,MV_first,pars.algorithm_r,pars) + thresh_min;
                    if sum( ecv(m,2) <= 0 ) == 0 
                        e_low = ecv(m,2); ff_low = ff(e_low,1); x_low(ff_low < 0) = e_low;
                    end;
                    return_warning{1} = true; return_warning{2} = 'solve_threshold'; return_warning{3} = 1;
                    e_eq = bissection(x_low,x_high,ff,tol_f,tol_x,return_warning);

                end;

            else

                e_eq = pars.e_min;

            end;

            % Solve for the dividend issuance threshold under the assumption that it is
            % larger than the switching threshold
            x_low = pars.e_min; x_high = pars.e_star(m); tol_f = pars.tol_f_thr_max; tol_x = pars.tol_x_thr_max;
            tt = tval(2);
            return_VF = false;
            z = pars.z_augmented_nosubsidy(m,:);
            ff = @(x,indexes) -pars.beta*(1-pars.eta)*optimal_policy_nr( x, z(indexes,:), A, B, ecv, return_VF, pars) + (1-pars.tau).*( 1 - pars.eta + pars.eta*(1-pars.xi) ) + tt(indexes);
            return_warning{1} = true; return_warning{2} = 'dividend issuance threshold';

            ff_div = ff(x_high,1);
            while ff_div < 0
                x_high = 1.1*x_high;
                ff_div = ff(x_high,1);
            end;

            e_div = bissection_noedges(x_low,x_high,ff,tol_f,tol_x,return_warning);

            % Check whether V_r < V_nr at the dividend issuance and equity issuance thresholds
            return_VF = true; return_more = false; MV_first = false;
            z = pars.z_augmented_nosubsidy(m,:); 
            ff = @(x,indexes) optimal_policy_nr( x, z(indexes,:), A, B, ecv, return_VF, pars) - optimal_policy_r(x,z(indexes,:), A, B, ecv, return_more,MV_first,pars.algorithm_r, pars);
            ff_div = ff(e_div,1);
            ff_eq = ff(e_eq,1);
            if ff_div < ff_eq
                fprintf('Error: ff_div > ff_eq');
                error('ff_div > ff_eq');
            end;

            if ff_div <= 0 && ff_eq <= 0

                % If V_r >= V_nr, re-compute the dividend issuance threshold using
                % V_r and set e_switch = e_div
                x_low = pars.e_min; x_high = pars.k_star_B(m); tol_f = pars.tol_f_thr_max; tol_x = pars.tol_x_thr_max;
                tt = tval(2);
                return_more = true; MV_first = true;
                z = pars.z_augmented_nosubsidy(m,:); 
                ff = @(x,indexes) -pars.beta*(1-pars.eta)*optimal_policy_r( x, z(indexes,:), A, B, ecv, return_more,MV_first,pars.algorithm_r,pars) +  (1-pars.tau).*( 1 - pars.eta + pars.eta*(1-pars.xi) ) + tt(indexes);
                return_warning{1} = true; return_warning{2} = 'dividend issuance threshold';
                e_div = bissection_noedges(x_low,x_high,ff,tol_f,tol_x,return_warning);
                e_switch = e_div;

            elseif ff_div > 0 && ff_eq <= 0

                % If V_r < V_nr, compute switching threshold
                x_low = e_eq; x_high = e_div; tol_f = pars.tol_f_thr_switch; tol_x = pars.tol_x_thr_switch;
                return_VF = true; return_more = false; MV_first = false;
                z = pars.z_augmented_nosubsidy(m,:); 
                ff = @(x,indexes) 1/(-ff_eq)*(optimal_policy_nr( x, z(indexes,:), A, B, ecv, return_VF, pars) - optimal_policy_r(x,z(indexes,:), A, B, ecv, return_more,MV_first,pars.algorithm_r, pars));
                return_warning{1} = true; return_warning{2} = 'switching threshold'; 
                e_switch = bissection_noedges(x_low,x_high,ff,tol_f,tol_x,return_warning);

            else 

                % If V_r < V_nr at e = e_eq, re-compute the entry size
                z = pars.z_augmented_nosubsidy(m,:);
                x_low = e_min; x_high = pars.k_star_B(m); tol_f = pars.tol_f_thr_min; tol_x = pars.tol_x_thr_min; 
                ff = @(x,indexes) -optimal_policy_nr(x, z(1,:), A, B, ecv, false, pars) + thresh_min;
                if sum( ecv(m,2) <= 0 ) == 0 
                    e_low = ecv(m,2); ff_low = ff(e_low,1); x_low(ff_low < 0) = e_low;
                end;
                return_warning{1} = true; return_warning{2} = 'solve_threshold'; return_warning{3} = 1;
                e_eq = bissection(x_low,x_high,ff,tol_f,tol_x,return_warning);
                e_switch = NaN;

            end;

            e_switch_nosubsidy = e_switch;

        end;
        
else
            
      if pars.e_subsidy > 0

          error('no policy experiment with only one debt instrument');

      else

            if update_e_min == 1

                % Check that the equity issuance cost is not too large, so that dVde >
                % 1/(1-eta)*(beta^(-1)*(1+gamma_e) - eta*xi)
                % This only applies when there is interim equity issuance, i.e. when
                % update_e_min == 1
                z               = pars.z_augmented_nosubsidy;
                test_e_min      = e_min; return_more = true; MV_first = true;
                dVde_min        = optimal_policy_r(test_e_min,z,A,B,ecv,return_more,MV_first,pars.algorithm_r,pars);
                thresh_min      = (1+pars.r)*(1 + pars.eta/(1-pars.eta)*(1-pars.xi) + 1/(1-pars.eta)*pars.gamma_e);

                if dVde_min <= thresh_min

                    e_eq = e_min;

                end;

                if dVde_min > thresh_min

                    % Prepare vectors
                    tt = tval(1);

                    % This function solves dV/de(e) = 1+gamma_e
                    z = pars.z_augmented_nosubsidy;
                    x_low = e_min; x_high = pars.k_star_B(m); tol_f = pars.tol_f_thr_min; tol_x = pars.tol_x_thr_min; 
                    ff = @(x,indexes) -pars.beta*(1-pars.eta)*optimal_policy_r(x,z(indexes,:),A,B,ecv,return_more,MV_first,pars.algorithm_r,pars) + ( 1 - pars.eta + pars.eta*(1-pars.xi) ) + tt(indexes);
                    if sum( ecv(m,2) <= 0 ) == 0 
                        e_low = ecv(m,2); ff_low = ff(e_low,1); x_low(ff_low < 0) = e_low;
                    end;
                    return_warning{1} = true; return_warning{2} = 'solve_threshold'; return_warning{3} = 1;
                    e_eq = bissection(x_low,x_high,ff,tol_f,tol_x,return_warning);

                end;

            else

                e_eq = pars.e_min;

            end;

            % Solve for the dividend issuance threshold 
            x_low = e_eq; x_high = pars.e_star(m); tol_f = pars.tol_f_thr_max; tol_x = pars.tol_x_thr_max;
            tt = tval(2);
            return_VF = false;
            z = pars.z_augmented_nosubsidy(m,:);
            ff = @(x,indexes) -pars.beta*(1-pars.eta)*optimal_policy_nr( x, z(indexes,:), A, B, ecv, return_VF, pars) + (1-pars.tau).*( 1 - pars.eta + pars.eta*(1-pars.xi) ) + tt(indexes);
            return_warning{1} = true; return_warning{2} = 'dividend issuance threshold';

            ff_div = ff(x_high,1);
            while ff_div < 0
                x_high = 1.1*x_high;
                ff_div = ff(x_high,1);
            end;

            e_div              = bissection_noedges(x_low,x_high,ff,tol_f,tol_x,return_warning);
            e_switch           = -10;
            e_switch_nosubsidy = e_switch;

      end;
            
end;
    
end

