function [s_r,s_fm,s_mb,s_rfm,k_r,k_fm,k_mb,k_rfm] = thresholds_s( e, z, pars )

numargs = numel(e); s_r = NaN(numargs,1); 
[s_fm,s_mb,s_rfm,s_min,k_r,k_fm,k_mb,k_rfm,k_min] = deal(NaN(size(s_r)));

% Check argument size
if numargs ~= size(z,1)
    fprintf('Error in thresholds: wrong size of arguments \n');
end;

%Useful values
a = z(:,1); b = z(:,2); r_m = z(:,end);
E_phi = a.*gamma(1+1./b);
r_hat = 1./((1-pars.chi)./(1+r_m)+pars.chi/(1+pars.r_b)) - 1;
chi_m = z(:,(end-2));
delta_hat = 1-(pars.chi_L - chi_m)./(1-chi_m).*(1-pars.delta_k);

% Solve for s_r
s_r(:) = 1./(1+(1-pars.chi)./(1+r_m)*(1+pars.r_b)/pars.chi);
%{
if sum(e>0) > 0
    k_r(e>0) = borrowing_constraint_f(s_r(e>0),e(e>0),z(e>0,:),pars);
end;
%}
delta_tilde = 1 - s_r.*(1-pars.delta_k) - (1-s_r).*(1-delta_hat);

% Solve for s_rfm
s_rfm(:) = 1 - (1-pars.chi)./(1+r_m).*(1-delta_hat);
k_rfm(:) = borrowing_constraint_b(s_rfm,e,z,pars);

% Solve for s_f and s_bm

% upper bound
A           = E_phi./(pars.delta_k+r_hat);
B           = (1+r_hat)./(r_hat+pars.delta_k).*e;
tol_x = pars.tol_x_fractionalroot; tol_f = pars.tol_f_fractionalroot;
x_high      = find_root_fractional(A,B,tol_x,tol_f,pars);

% lower bound
x_low       = (1+r_hat)./(r_hat+delta_tilde).*e;

case1 = (e == 0);
case2 = (e >  0);

if sum(case1) > 0

    x_low1 = x_low(case1); x_high1 = x_high(case1); a1 = a(case1); b1 = b(case1); s_r1 = s_r(case1); E_phi1 = E_phi(case1); delta_hat1 = delta_hat(case1); delta_tilde1 = delta_tilde(case1);
    z1 = z(case1,:);
    tol_f = pars.tol_f_sthr; tol_x = pars.tol_x_sthr;
    return_warning{1} = true; return_warning{2} = 'thresholds';

    % Solve for s_mb
    ff = @(x,indexes) x.^(1-pars.zeta).*(r_hat + delta_tilde1(indexes)) - s_r1(indexes).*E_phi1(indexes) - (1-s_r1(indexes)).*N_fun(maximum_default_m(x,a1(indexes),b1(indexes),z1(indexes,:),pars),-(1-delta_hat1(indexes)).*((x).^(1-pars.zeta)),a1(indexes),b1(indexes),pars);
    k_mb(case1) = bissection(x_low1,x_high1,ff,tol_f,tol_x,return_warning);
    s_mb(case1) = pars.chi/(1+pars.r_b)*1./(k_mb(case1) - e(case1)).*(E_phi1.*k_mb(case1).^(pars.zeta) + (1-pars.delta_k)*k_mb(case1));
    
    % Solve for s_fm
    ff = @(x,indexes) x.^(1-pars.zeta).*(r_hat + delta_tilde1(indexes)) - gross_return_f(x,s_r1(indexes),a1(indexes),b1(indexes),z1(indexes,:),pars);
    k_fm(case1) = bissection(x_low1,x_high1,ff,tol_f,tol_x,return_warning);
    s_fm(case1) = pars.chi/(1+pars.r_b)*1./(k_fm(case1) - e(case1)).*(H_fun(maximum_default_m(k_fm(case1),a1,b1,z1,pars),a1,b1).*k_fm(case1).^(pars.zeta) + (1-pars.delta_k)*k_fm(case1));
    
    % Solve for s_min
    %{
    ff = @(x,indexes) x.^(1-pars.zeta).*(r_hat + delta_tilde1(indexes)) - (1-s_r1(indexes)).*N_fun(maximum_default_m(x,a1(indexes),b1(indexes),z1(indexes,:),pars),-(1-delta_hat1(indexes)).*((x).^(1-pars.zeta)),a1(indexes),b1(indexes),pars);
    k_min(case1) = bissection(x_low1,x_high1,ff,tol_f,tol_x,return_warning);
    s_min(case1) = pars.chi/(1+pars.r_b)*1./(k_min(case1) - e(case1)).*((1-pars.delta_k)*k_min(case1));
    %}

end;

if sum(case2) > 0

    x_low2 = x_low(case2); x_high2 = x_high(case2); a2 = a(case2); b2 = b(case2); s_r2 = s_r(case2); E_phi2 = E_phi(case2); e2 = e(case2); delta_hat2 = delta_hat(case2); delta_tilde2 = delta_tilde(case2);
    z2 = z(case2,:);
    tol_f = pars.tol_f_sthr; tol_x = pars.tol_x_sthr;
    return_warning{1} = true; return_warning{2} = 'thresholds';

    % Solve for s_mb
    ff = @(x,indexes) (r_hat + delta_tilde2(indexes)).*(x.^(1-pars.zeta)) - (1+r_hat).*e2(indexes).*(x.^(-pars.zeta)) - s_r2(indexes).*E_phi2(indexes) - (1-s_r2(indexes)).*N_fun(maximum_default_m(x,a2(indexes),b2(indexes),z2(indexes,:),pars),-(1-delta_hat2(indexes)).*((x).^(1-pars.zeta)),a2(indexes),b2(indexes),pars);
    k_mb(case2) = bissection(x_low2,x_high2,ff,tol_f,tol_x,return_warning);
    s_mb(case2) = pars.chi/(1+pars.r_b).*1./(k_mb(case2) - e(case2)).*(E_phi2.*k_mb(case2).^(pars.zeta) + (1-pars.delta_k)*k_mb(case2));
    
    % Solve for s_fm
    ff = @(x,indexes) (r_hat + delta_tilde2(indexes)).*(x.^(1-pars.zeta)) - (1+r_hat).*e2(indexes).*(x.^(-pars.zeta))  - gross_return_f(x,s_r2(indexes),a2(indexes),b2(indexes),z2(indexes,:),pars);
    k_fm(case2) = bissection(x_low2,x_high2,ff,tol_f,tol_x,return_warning);
    s_fm(case2) = pars.chi/(1+pars.r_b)*1./(k_fm(case2) - e(case2)).*(H_fun(maximum_default_m(k_fm(case2),a2,b2,z2,pars),a2,b2).*k_fm(case2).^(pars.zeta) + (1-pars.delta_k)*k_fm(case2));
    
    % Solve for s_min
    %{
    ff = @(x,indexes) (r_hat + delta_tilde2(indexes)).*(x.^(1-pars.zeta)) - (1+r_hat).*e2(indexes).*(x.^(-pars.zeta)) - (1-s_r2(indexes)).*N_fun(maximum_default_m(x,a2(indexes),b2(indexes),z2(indexes,:),pars),-(1-delta_hat2(indexes)).*((x).^(1-pars.zeta)),a2(indexes),b2(indexes),pars);
    k_min(case2) = bissection(x_low2,x_high2,ff,tol_f,tol_x,return_warning);
    s_min(case2) = pars.chi/(1+pars.r_b).*1./(k_min(case2) - e(case2)).*((1-pars.delta_k)*k_min(case2));
    %}
    
end;

%s_r_tilde(:) = 1./(1+(pars.chi_L-pars.chi)./(1+r_m)*(1+pars.r_b)/pars.chi);
%s_r(:) = s_r_tilde(:);
%s_r(:) = max(s_r_tilde,s_fm);

end

