function results_transition_path = transition_path(output)

fprintf('\n');
fprintf('---- Transitional dynamics ---- \n');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
% Cell vectors to store output
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

TT = 1+10;

e_cell         = cell(TT,1);   % State-space
ecv_cell       = cell(TT,1);   % Next-period state-space
A_cell         = cell(TT,1);   % Value function
B_cell         = cell(TT,1);   % Value function
phi_d_r_cell   = cell(TT,1);   % Optimal default threshold in R-region
phi_d_nr_cell  = cell(TT,1);   % Optimal default threshold in K-region
phi_b_cell     = cell(TT,1);   % Optimal renegotiation threshold in R-region
k_hat_r_cell   = cell(TT,1);   % Optimal capital in R-region
k_hat_nr_cell  = cell(TT,1);   % Optimal capital in K-region
s_hat_r_cell   = cell(TT,1);   % Optimal debt composition in R-region
mu_cell        = cell(TT,1);   % Distribution of firms
nu_cell        = cell(TT,1);   % Vector representing mass of entering firms
output_cell    = cell(TT,1);   % Aggregate output
capital_cell   = cell(TT,1);   % Capital
Total_B_cell   = cell(TT,1);   % Total bank borrowing
Total_M_cell   = cell(TT,1);   % Total market borrowing
BS_cell        = cell(TT,1);   % Aggregate bank share 
Mixed_Frac_cell= cell(TT,1);   % Fraction of mixed-debt firms in steady-state
S_B_cell       = cell(TT,1);   % Bank borrowing by "small" firms
S_M_cell       = cell(TT,1);   % Market borrowing by "small" firms
L_B_cell       = cell(TT,1);   % Bank borrowing by "large" firms
L_M_cell       = cell(TT,1);   % Market borrowing by "large" firms
PF_cell        = cell(TT,1);   % Cell containing all policy functions

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
% Initial steady-state (before shock)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Note: the indexing convention is that A_cell{tt} represents the A matrix
% summarizing the continuation value at time t+1; likewise for B_cell{tt}
% and ecv_cell{tt}.

policy_functions = output(1).policy_functions;
ecv = output(1).ecv;
A = output(1).A;
B = output(1).B;
pars = output(1).pars;

e_cell{1}         = policy_functions.e; 
ecv_cell{1}       = ecv;
A_cell{1}         = A;
B_cell{1}         = B;
phi_d_r_cell{1}   = policy_functions.phi_d_r; 
phi_d_nr_cell{1}  = policy_functions.phi_d_nr; 
phi_b_cell{1}     = policy_functions.phi_b; 
k_hat_r_cell{1}   = policy_functions.k_hat_r; 
k_hat_nr_cell{1}  = policy_functions.k_hat_nr;
s_hat_r_cell{1}   = policy_functions.s_hat_r; 
PF_cell{1}        = policy_functions;
gamma_b_initial   = pars.gamma_b;

% Shock occurs at tt = 2; at that stage, net worth distribution and
% thresholds are given (policies can change)
PF_cell{2}.e      = e_cell{1};
e_cell{2}         = e_cell{1};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
% Final steady-state (after shock)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Chose horizon, i.e. date after which the exogenous aggregate variable that is shocked (in
% this text) does not vary anymore

TT_H                 = 5;
    
policy_functions = output(2).policy_functions;
ecv = output(2).ecv;
A = output(2).A;
B = output(2).B;
pars = output(2).pars;
pars.paralellized_inside = true;
distribution = output(2).distribution;

ecv_cell{TT}     = ecv;
A_cell{TT}       = A;
B_cell{TT}       = B;

for tt = TT_H:TT
    
    ecv_cell{tt-1}     = ecv;
    A_cell{tt-1}       = A;
    B_cell{tt-1}       = B;
    e_cell{tt}         = policy_functions.e; 
    phi_d_r_cell{tt}   = policy_functions.phi_d_r; 
    phi_d_nr_cell{tt}  = policy_functions.phi_d_nr; 
    phi_b_cell{tt}     = policy_functions.phi_b; 
    k_hat_r_cell{tt}   = policy_functions.k_hat_r; 
    k_hat_nr_cell{tt}  = policy_functions.k_hat_nr;
    s_hat_r_cell{tt}   = policy_functions.s_hat_r; 
    PF_cell{tt}        = policy_functions;

end;

mu_final_reference        = distribution.mu;
gamma_b_final             = pars.gamma_b;

%%%%%%%%%%%%%%%%    
% Choose shock
%%%%%%%%%%%%%%%%
Delta = (gamma_b_final-gamma_b_initial);

shock_vec    = gamma_b_final*ones(TT,1);
shock_vec(1) = gamma_b_initial;
shock_vec(2) = gamma_b_initial -  (0.3/10)*(Delta);
shock_vec(3) = gamma_b_initial +  (2.0/10)*(Delta);
shock_vec(4) = gamma_b_final   -  (0.5/10)*(Delta);
shock_vec(5) = gamma_b_final   -  (0.5/10)*(Delta);
shock_vec(6) = gamma_b_final   -  (0.5/10)*(Delta);
shock_vec(7) = gamma_b_final   -  (0.5/10)*(Delta);

%{
shock_vec    = gamma_b_final*ones(TT,1);
shock_vec(1) = gamma_b_initial;
shock_vec(2) = gamma_b_initial - (0.3/10)*(Delta);
shock_vec(3) = gamma_b_initial + (7/10)*(Delta);
shock_vec(4) = gamma_b_final   -  (0.5/10)*(Delta);
shock_vec(5) = gamma_b_final   -  (0.5/10)*(Delta);
shock_vec(6) = gamma_b_final   -  (0.5/10)*(Delta);
shock_vec(7) = gamma_b_final   -  (0.5/10)*(Delta);
%}

% Uncomment to draw
%{
figure(1)
plot(1:(TT_H),shock_vec(1:(TT_H)),'-x'); title('Path of gamma_b');
hold on;
plot(1:(TT_H),gamma_b_initial*ones(TT_H,1),'--b');
hold on;
plot(1:(TT_H),gamma_b_final*ones(TT_H,1),'--b');
xlim([1 (TT_H)]);
pause(1); drawnow;
%}

% Backward iteration
fprintf('\n'); fprintf('Backward iteration \n'); fprintf('\n');
for tt=(TT_H-1):(-1):2
    
    pars.gamma_b = shock_vec(tt);
    pars.r_b     = pars.r + pars.gamma_b;
    
    fprintf('----------------------------------------------------\n');
    fprintf('Horizon tt = %d, gamma_b = %7.4f, r_b = %7.4f  \n',tt,shock_vec(tt),pars.r_b);
    fprintf('----------------------------------------------------\n');
    
    tt_clock = tic;
    
    % Value function one-step ahead
    A   = A_cell{tt};
    B   = B_cell{tt};
    ecv = ecv_cell{tt};
    
    if tt == 2
        exogenous_e = e_cell{2};
    else
        exogenous_e = [];
    end;
        
    if pars.paralellized_inside
                [ ecvp, Ap, Bp, policy_functions ] = iterate_model_paralellized( ecv, A, B, pars, exogenous_e, true ); 
    else
                [ ecvp, Ap, Bp, policy_functions ] = iterate_model( ecv, A, B, pars, exogenous_e, true ); 
    end;

    % Store results of iterations
    if tt > 2
        A_cell{tt-1}       = Ap;
        B_cell{tt-1}       = Bp;
        ecv_cell{tt-1}     = ecvp;
    end;
    PF_cell{tt}            = policy_functions;

    tt_clock = toc(tt_clock);
    fprintf('Time for horizon %d: %d min and %d seconds \n',tt,floor(tt_clock/60),round(mod(tt_clock,60))); fprintf('\n');
    
end;
fprintf('\n');
save('results_backward','A_cell',...
                        'B_cell',...
                        'ecv_cell',...
                        'PF_cell');            
                    
load('results_backward');

N                               = pars.N;
%N_z                            = pars.N_z;
pars.z_augmented_nosubsidy      = [pars.z repmat(pars.chi_m_nosubsidy,[pars.N_z,1]) repmat(pars.r_m_nosubsidy_NR,[pars.N_z,1]) repmat(pars.r_m_nosubsidy_R,[pars.N_z,1])];
%z                              = kron(pars.z_augmented_nosubsidy,ones(N-1,1));

fprintf('\n'); fprintf('Forward propagation \n'); fprintf('\n');   

for tt=1:(TT-1)
    
    pars.gamma_b = shock_vec(tt);
    pars.r_b     = pars.r + pars.gamma_b;
    
    fprintf('----------------------------------------------------\n');
    fprintf('Horizon tt = %d, gamma_b = %7.4f, r_b = %7.4f  \n',tt,shock_vec(tt),pars.r_b);
    fprintf('----------------------------------------------------\n');

    % Policy functions
    if tt == 1
        steady_state = true;
        current_mu   = [];
        next_e       = []; 
        A            = A_cell{1};
        B            = B_cell{1};
        ecv          = ecv_cell{1};
    else
        steady_state = false;
        current_mu   = mu_cell{tt};
        next_e       = PF_cell{tt+1}.e;
        A            = A_cell{tt+1};
        B            = B_cell{tt+1};
        ecv          = ecv_cell{tt+1};
    end;
        
    % Entry, exit, stationary distribution and firm dynamics moments
    policy_functions = PF_cell{tt};
    if pars.paralellized_inside
        [~,~,~,~,~,distribution,aggregates] = aggregate_implications_for_moment_matching_paralellized(output(1).gamma_e_ipo,policy_functions,A,B,ecv,steady_state,current_mu,next_e,pars,false);
    else
        [~,~,~,~,~,distribution,aggregates] = aggregate_implications_for_moment_matching(output(1).gamma_e_ipo,policy_functions,A,B,ecv,steady_state,current_mu,next_e,pars,false);
    end;
    %[entry,distribution,aggregates,FD_moments] = aggregate_implications(policy_functions,A,B,ecv,steady_state,current_mu,next_e,pars);

    if tt == 1
        mu_cell{1}               = distribution.mu;
    end;
    mu_cell{tt+1}                = distribution.mu;
    nu_cell{tt}                  = distribution.nu;
    BS_cell{tt}                  = aggregates.BS;
    Mixed_Frac_cell{tt}          = aggregates.Mixed_Frac;
    output_cell{tt}              = aggregates.output;
    capital_cell{tt}             = aggregates.capital;
    Total_B_cell{tt}             = aggregates.Total_B;
    Total_M_cell{tt}             = aggregates.Total_M;
    S_B_cell{tt}                 = aggregates.S_B;
    S_M_cell{tt}                 = aggregates.S_M;
    L_B_cell{tt}                 = aggregates.L_B;
    L_M_cell{tt}                 = aggregates.L_M;
    
    if tt > 1
        fprintf('Max difference between current and LR SS distribution = %9.5f \n',max(abs(mu_cell{tt} - mu_final_reference)));
    end;

end;  

[relative_Y_path,relative_K_path,relative_I_path] = deal(NaN(TT+1,1));
delta_k = 0.15;
initial_Y = output(1).aggregates.output;
initial_K = output(1).aggregates.capital;
initial_I = delta_k*output(1).aggregates.capital;
relative_Y_path(1) = initial_Y;
relative_K_path(1) = initial_K;
relative_I_path(1) = initial_I;
for tt=2:TT
    relative_Y_path(tt) = output_cell{tt-1};
    relative_K_path(tt) = capital_cell{tt-1};
    if tt == 2
        relative_I_path(tt) = initial_I;
    else
        relative_I_path(tt) = (capital_cell{tt-1}-(1-delta_k)*capital_cell{tt-2});
    end;
end;
relative_Y_path(end) = output(2).aggregates.output;
relative_K_path(end) = output(2).aggregates.capital;
relative_I_path(end) = delta_k*output(2).aggregates.capital;
relative_Y_path = relative_Y_path/relative_Y_path(3);
relative_K_path = relative_K_path/relative_K_path(3);
relative_I_path = relative_I_path/relative_I_path(3);

% Show transition path
fprintf('\n');
fprintf('Transition path: \n');
fprintf('\n');
fprintf('T    zeta     chi      rb          BS      Y/Y_2   K/K_2   I/I_2    \n');
fprintf('-----------------------------------------------------------------------------------\n');
for tt=0:TT
    if tt==0
        fprintf('IN   %5.4f   %5.4f   %6.5f     %5.4f  %5.4f        %5.4f        %5.4f      \n',pars.zeta,pars.chi_L,output(1).pars.r_b,output(1).aggregates.BS,...
                                        relative_Y_path(tt+1),relative_K_path(tt+1),relative_I_path(tt+1));
    elseif tt<TT
        if tt==1
            previous_cap = output(1).aggregates.capital;
        else
            previous_cap = capital_cell{tt-1};
        end;
        fprintf('%2.0f   %5.4f   %5.4f   %6.5f     %5.4f  %5.4f        %5.4f        %5.4f      \n',tt,pars.zeta,pars.chi_L,pars.r +shock_vec(tt),BS_cell{tt},...
                                        relative_Y_path(tt+1),relative_K_path(tt+1),relative_I_path(tt+1));
    else
        fprintf('FI   %5.4f   %5.4f   %6.5f     %5.4f  %5.4f        %5.4f        %5.4f      \n',pars.zeta,pars.chi_L,output(2).pars.r_b,output(2).aggregates.BS,...
                                        relative_Y_path(tt+1),relative_K_path(tt+1),relative_I_path(tt+1));
    end;
end;

results_transition_path.relative_Y_path         = relative_Y_path;
results_transition_path.relative_I_path         = relative_I_path;
results_transition_path.delta_k                 = delta_k;
results_transition_path.BS_cell                 = BS_cell;
results_transition_path.capital_cell            = capital_cell;
results_transition_path.output_cell             = output_cell;
results_transition_path.S_B_cell                = S_B_cell;
results_transition_path.S_M_cell                = S_M_cell;
results_transition_path.L_B_cell                = L_B_cell;
results_transition_path.L_M_cell                = L_M_cell;
results_transition_path.shock_vec               = shock_vec;
results_transition_path.pars                    = pars;

end
