function fig11(results_transition_path,figname,savepath,fignum)

graphname           = strcat(savepath,'/',figname);
pathQFRData         = [pwd '/data sources/QFR/'];
addpath(pathQFRData)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Download, format and extract relevant info from QFR
% Done externally by the stata script extractQFR.do
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

cd(pathQFRData);
out = csv2mat_numeric([pathQFRData 'matlabExtractQFR.csv']);
year = out.year;
quarter = out.quarter;
TT1 = out.tt1;
TTFIN = out.ttfin;
clear out

NumberOfAssetClasses = 8;
numberOfPeriods = length(year)/NumberOfAssetClasses;
InitialDate = year(1)+(quarter(1)-1)*0.25;
FinalDate = year(end) + (quarter(end)-1)*0.25;
datesQFR = linspace(InitialDate,FinalDate,numberOfPeriods)';
clear Year Quarter InitialDate FinalDate

TT1   = reshape(TT1,[NumberOfAssetClasses numberOfPeriods])';
TTFIN = reshape(TTFIN,[NumberOfAssetClasses numberOfPeriods])';

% Path of QFR bank share
FirstDateDisplay = find(datesQFR == 2007.5);
LastDateDisplay = find(datesQFR == 2011.5);
BS_XX = (0:0.25:4)';
BS_YY = 100*(sum(TT1(FirstDateDisplay:LastDateDisplay,7:8),2))./(sum(TTFIN(FirstDateDisplay:LastDateDisplay,7:8),2));

BS_cell             = results_transition_path.BS_cell;

f1 = figure(fignum);
TT = length(BS_cell);
fontsize = 13;
widthline = 2.5;
mrksize = 7;
ffont = 'Palatino';
display_periods = 1:5;
ccolor = 'black';

subplot(2,3,1)
xx = 0:(TT-1);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%.
    % old model
    yy = 100*cell2mat(BS_cell);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
hh1 = plot(xx(display_periods),yy(display_periods),'-x','LineWidth',widthline,'Color',ccolor,'MarkerSize',mrksize);
grid on;
hold on;
xlabel('Year','fontsize',fontsize-2,'fontname',ffont);
ylabel('%','fontsize',fontsize-2,'fontname',ffont,'Rotation',90);
xlim([display_periods(1)-1 display_periods(end)-1]);
textCell = {'2007Q3','2008Q3','2009Q3','2010Q3','2011Q3'};
hold on;
line([1 1],[18.5 26],'Color','k','Linewidth',1.5,'Linestyle','--');
hold on;
hh2 = plot(BS_XX,BS_YY,'-','Color',[0.6;0.6;0.6],'LineWidth',widthline-0.5);
hold on;
plot([0 1 2 3 4],[24.0 24.7 23.5 19.7 19.8],'x','Color',[0.6;0.6;0.6],'MarkerFaceColor',[0.6;0.6;0.6],'MarkerEdgeColor',[0.6;0.6;0.6],'LineWidth',widthline-0.5);
for ii = 1:4:17
    if ii<21
        text(BS_XX(ii)-0.75, BS_YY(ii),textCell{(ii-1)/4+1},'FontSize',8,'FontName','Palatino','Color',[0.6;0.6;0.6]) 
    else
        text(BS_XX(ii)-0.5, BS_YY(ii)-0.5,textCell{(ii-1)/4+1},'FontSize',8,'FontName','Palatino','Color',[0.6;0.6;0.6]) 
    end;
end
l = legend([hh1 hh2],{'Model','Data, US'});
set(gca,'Xtick',[0:4])
set(gca,'fontsize',fontsize-3,'fontname',ffont);
title({'\bf{Aggregate bank share}'},'FontSize',fontsize,'fontname',ffont,'Rotation',0,'FontWeight','bold');
xlim([0 4]);
ylim([19 25.5]);
set(gca,'YTick',[20:1:26]);
set(l,'FontSize',fontsize,'FontName',ffont);
set(l,'Location','NorthEast');

subplot(2,3,4)
shock_vec = results_transition_path.shock_vec;
yy = 10000*shock_vec;
plot(xx(display_periods),yy(display_periods),'-x','LineWidth',widthline,'Color',ccolor,'MarkerSize',mrksize);
hold on;
grid on;
hold on;
xlabel('Year','fontsize',fontsize-2,'fontname',ffont);
ylabel('bps','fontsize',fontsize-2,'fontname',ffont,'Rotation',90);
title({'Path of bank intermediation cost \gamma_{b,t}'},'FontSize',fontsize,'fontname',ffont,'Rotation',0,'FontWeight','bold');
set(gca,'fontsize',fontsize-2,'fontname',ffont);
xlim([display_periods(1)-1 display_periods(end)-1]);
%ylim([155 170])
xlim([0 4]);
set(gca,'Xtick',[0:5])
%set(gca,'Ytick',[155:(5):170])
ax = gca;
line([1 1],ax.YLim,'Color','k','Linewidth',1.5,'Linestyle','--');

subplot(2,3,5)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Y = cell2mat(results_transition_path.output_cell);
yy = 100*((Y-Y(1))/Y(1));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
plot(xx(display_periods),yy(display_periods),'-x','LineWidth',widthline,'Color',ccolor,'MarkerSize',mrksize);
hold on;
plot(xx(display_periods),0*ones(length(xx(display_periods)),1),'LineWidth',widthline-1,'Color','black');
grid on;
hold on;
xlabel('Year','fontsize',fontsize-2,'fontname',ffont);
ylabel('% relative to year 0','fontsize',fontsize,'fontname',ffont,'Rotation',90);
title({'Output'},'FontSize',fontsize,'fontname',ffont,'Rotation',0,'FontWeight','bold');
set(gca,'fontsize',fontsize-2,'fontname',ffont);
xlim([display_periods(1)-1 display_periods(end)-1]);
%ylim([-3.5 1.5])
xlim([0 4]);
set(gca,'Xtick',[0:5])
%set(gca,'Ytick',[(-3):1:1])
ax = gca;
line([1 1],ax.YLim,'Color','k','Linewidth',1.5,'Linestyle','--');

subplot(2,3,6)
xx = 0:(TT-2);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
K = cell2mat(results_transition_path.capital_cell);
I = NaN(size(K));
I(2:end) = K(2:end)-(1-(results_transition_path.delta_k))*K(1:(end-1));
I(1) = results_transition_path.delta_k*K(1);
yy = 100*((I-I(1))/I(1));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
plot(xx(display_periods),yy(display_periods),'-x','LineWidth',widthline,'Color',ccolor,'MarkerSize',mrksize);
hold on;
plot(xx(display_periods),zeros(length(xx(display_periods)),1),'LineWidth',widthline-1,'Color','black');
grid on;
hold on;
xlabel('Year','fontsize',fontsize-2,'fontname',ffont);
ylabel('% relative to year 0','fontsize',fontsize,'fontname',ffont,'Rotation',90);
title({'Investment'},'FontSize',fontsize,'fontname',ffont,'Rotation',0,'FontWeight','bold');
set(gca,'fontsize',fontsize-2,'fontname',ffont);
xlim([display_periods(1)-1 display_periods(end)-1]);
%ylim([-17.5 7.5])
xlim([0 4]);
set(gca,'Xtick',[0:5])
ax = gca;
line([1 1],ax.YLim,'Color','k','Linewidth',1.5,'Linestyle','--');

% Small vs. large firms variables

BS_vec = cell2mat(results_transition_path.S_B_cell); MS_vec = cell2mat(results_transition_path.S_M_cell);
BL_vec = cell2mat(results_transition_path.L_B_cell); ML_vec = cell2mat(results_transition_path.L_M_cell);

DB_SMALL = 100*(BS_vec(1))/(BS_vec(1)+MS_vec(1))*(BS_vec(1:end)/BS_vec(1)-1);
DM_SMALL = 100*(MS_vec(1))/(BS_vec(1)+MS_vec(1))*(MS_vec(1:end)/MS_vec(1)-1);
DB_LARGE = 100*(BL_vec(1))/(BL_vec(1)+ML_vec(1))*(BL_vec(1:end)/BL_vec(1)-1);
DM_LARGE = 100*(ML_vec(1))/(BL_vec(1)+ML_vec(1))*(ML_vec(1:end)/ML_vec(1)-1);

subplot(2,3,2)
yy1 = DB_LARGE;
yy2 = DM_LARGE;
h1 = plot(xx(display_periods),yy1(display_periods),'-','LineWidth',widthline,'Color',[0.6;0.6;0.6]);
hold on;
h2 = plot(xx(display_periods),yy2(display_periods),'-','LineWidth',widthline,'Color','black');
hold on;
plot(xx(display_periods),zeros(length(xx(display_periods)),1),'LineWidth',widthline-1,'Color','black');
grid on;
hold on;
%line([1 1],[-10 20],'Color','k','Linewidth',1.5,'Linestyle','--');
xlabel('Year','fontsize',fontsize-2,'fontname',ffont);
ylabel('% relative to year 0','fontsize',fontsize,'fontname',ffont,'Rotation',90);
title({'\fontsize{13pts} \bf{Debt composition of medium firms}'},'FontSize',fontsize+2,'FontName',ffont,'FontWeight','bold');
set(gca,'fontsize',fontsize-2,'fontname',ffont);
xlim([display_periods(1)-1 display_periods(end)-1]);
%ylim([-10 10]);
hh1 = legend([h1 h2],'Bank debt','Market debt');
set(hh1,'Location','NorthWest','fontsize',fontsize,'fontname',ffont);
xlim([0 4]);
ylim([-10 20]);
set(gca,'Xtick',[0:4])
ax = gca;
line([1 1],ax.YLim,'Color','k','Linewidth',1.5,'Linestyle','--');

subplot(2,3,3)
yy1 = DB_SMALL;
yy2 = DM_SMALL;
h1 = plot(xx(display_periods),yy1(display_periods),'-','LineWidth',widthline,'Color',[0.6;0.6;0.6]);
hold on;
h2 = plot(xx(display_periods),yy2(display_periods),'-','LineWidth',widthline,'Color','black');
hold on;
plot(xx(display_periods),zeros(length(xx(display_periods)),1),'LineWidth',widthline-1,'Color','black');
grid on;
hold on;
%line([1 1],[-10 20],'Color','k','Linewidth',1.5,'Linestyle','--');
xlabel('Year','fontsize',fontsize-2,'fontname',ffont);
ylabel('% relative to year 0','fontsize',fontsize,'fontname',ffont,'Rotation',90);
title({'\fontsize{13pts} \bf{Debt composition of large firms}'},'FontSize',fontsize+2,'FontName',ffont,'FontWeight','bold');
set(gca,'fontsize',fontsize-2,'fontname',ffont);
xlim([display_periods(1)-1 display_periods(end)-1]);
%ylim([-10 10]);
hh1 = legend([h1 h2],'Bank debt','Market debt');
set(hh1,'Location','NorthWest','fontsize',fontsize,'fontname',ffont);
xlim([0 4]);
ylim([-10 20]);
set(gca,'Xtick',[0:5])
ax = gca;
line([1 1],ax.YLim,'Color','k','Linewidth',1.5,'Linestyle','--');

set(f1, 'PaperUnits', 'inches') 
set(f1, 'PaperOrientation', 'portrait');
set(f1, 'PaperSize',[11.5 9]);
papersize = get(gcf,'PaperSize');
width  = 11.5;        
height = 9;         
bottom = (papersize(2)- height)/2;
left = (papersize(1)- width)/2;
myfiguresize = [left-1.25, bottom-0.5, width+2.25, height+0.5];
set(gcf, 'PaperPosition', myfiguresize);
print(f1, '-dpng',  graphname);