function fig12(output_nosubsidy,output_subsidy,figname,savepath,fignum,has_arrows)

graphname           = strcat(savepath,'/',figname);

% Baseline
z_to_plot           = 2;
e_bas               = output_nosubsidy.policy_functions.e(:,z_to_plot);
k_r_bas             = output_nosubsidy. policy_functions.k_hat_r(:,z_to_plot);
index_r_bas         = ~isnan(k_r_bas);
k_nr_bas            = output_nosubsidy.policy_functions.k_hat_nr(:,z_to_plot);
index_nr_bas        = ~isnan(k_nr_bas);
index_switch_bas    = find(~isnan(k_r_bas),1,'last');
k_bas = k_r_bas; k_bas(isnan(k_r_bas)) = k_nr_bas(isnan(k_r_bas)); d_bas = k_bas - e_bas;

% With subsidy
e                   = output_subsidy.policy_functions.e(:,z_to_plot);
k_r                 = output_subsidy.policy_functions.k_hat_r(:,z_to_plot);
index_r             = ~isnan(k_r);
k_nr                = output_subsidy.policy_functions.k_hat_nr(:,z_to_plot);
index_switch        = find(~isnan(k_r),1,'last');
index_nr            = (~isnan(k_nr));
k = k_r; k(isnan(k_r)) = k_nr(isnan(k_r)); d = k-e;

% Make graph
lw = 3;
f1 = figure(fignum);
grey = [0.6;0.6;0.6];
% For visibility
k_r = 1.05*k_r; k_nr = 1.05*k_nr;
subplot(121)
plot(e_bas(index_r_bas),k_r_bas(index_r_bas),'-o','LineWidth',lw,'MarkerSize',3,'Color',grey);
hold on;
plot(e(index_r),k_r(index_r),'-','LineWidth',lw,'Color',[0.3,0.3,0.3]);
hold on;
plot(e_bas(index_nr_bas),k_nr_bas(index_nr_bas),'-o','Color',grey,'LineWidth',lw,'MarkerSize',3);
hold on;
plot(e_bas(index_switch_bas)*ones(50,1),linspace(0,10,50),'--','Color',grey,'LineWidth',lw-1);
hold on;
plot(e(index_nr),k_nr(index_nr),'Color',[0.3,0.3,0.3],'LineWidth',lw);
hold on;
plot(e(index_switch)*ones(50,1),linspace(0,10,50),'--','Color',[0.3,0.3,0.3],'LineWidth',lw-1);
hold on;
ll = legend('Baseline','Subsidy for market debt');
set(ll,'location','NorthWest','Interpreter','latex');
xlabel('Internal finance (levels)','FontSize',13,'FontWeight','Bold','Interpreter','Latex');
title('Total assets $\hat{k}(e_t,z_t) = e_t + \hat{b}(e_t,z_t) + \hat{m}(e_t,z_t)$','Fontsize',13,'fontname','Palatino','Interpreter','Latex')
set(gca,'Fontsize',11,'fontname','Palatino','TickLabelInterpreter','latex');
xlim([0,3]);

if has_arrows

    textar1 = text(1.05*e_bas(index_switch_bas),6,'Hello');
    set(textar1,...
        'String',{'   $e^{*}(z_t)$ '},...
        'FontSize',13,...
        'FontName','Palatino',...
        'Interpreter','Latex',...
        'color',grey);
    
    textar1 = text(0.5*e(index_switch),6,'Hello');
    set(textar1,...
        'String',{'   $e^{*}(z_t)$ '},...
        'FontSize',13,...
        'FontName','Palatino',...
        'Interpreter','Latex',...
        'color',[0.3,0.3,0.3]);
    
end;

% Output from no-subsidy model
e_bas               = output_nosubsidy.policy_functions.e;
mu_bas              = output_nosubsidy.distribution.mu;
s_hat_r_bas         = output_nosubsidy.policy_functions.s_hat_r;
% Sort equity vector
[e_sorted,indexes_sorted] = sort(e_bas(:));
mu_sorted = mu_bas(indexes_sorted);
mu_sorted = mu_sorted/sum(mu_sorted);
s_hat = s_hat_r_bas;
s_hat(isnan(s_hat)) = 0;
s_sorted = s_hat(indexes_sorted);
N_sorted = numel(e_sorted);
II = datasample(1:N_sorted,1e7,'weights',mu_sorted);
Y_bas = e_sorted(II);
S = s_sorted(II);
p = [0.10;0.20;0.30;0.40;0.50;0.60;0.70;0.80;0.90;1.00];
quantiles_e = quantile(Y_bas,p);
averag_share_E = zeros(size(p));
for ii=1:numel(p)
    if ii==1
        indexes = find(Y_bas <= quantiles_e(ii));
    else
        indexes = find( (Y_bas > quantiles_e(ii-1))&(Y_bas <= quantiles_e(ii)) );
    end;
    averag_share_E(ii) = mean(S(indexes));
end;

% Output from subsidy model
e               = output_subsidy.policy_functions.e;
mu              = output_subsidy.distribution.mu;
s_hat_r         = output_subsidy.policy_functions.s_hat_r;
% Sort equity vector
[e_sorted,indexes_sorted] = sort(e(:));
mu_sorted = mu(indexes_sorted);
mu_sorted = mu_sorted/sum(mu_sorted);
s_hat = s_hat_r;
s_hat(isnan(s_hat)) = 0;
s_sorted = s_hat(indexes_sorted);
N_sorted = numel(e_sorted);
II = datasample(1:N_sorted,1e7,'weights',mu_sorted);
Y = e_sorted(II);
S = s_sorted(II);
p = [0.10;0.20;0.30;0.40;0.50;0.60;0.70;0.80;0.90;1.00];
quantiles_e = quantile(Y,p);
averag_share_E = zeros(size(p));
for ii=1:numel(p)
    if ii==1
        indexes = find(Y <= quantiles_e(ii));
    else
        indexes = find( (Y > quantiles_e(ii-1))&(Y <= quantiles_e(ii)) );
    end;
    averag_share_E(ii) = mean(S(indexes));
end;

grey = [0.6 0.6 0.6];
ffont = 'Palatino';
fontsize = 13;

subplot(122)
hh = histogram(Y,300,'Normalization','probability'); hold on;
set(hh,'FaceColor',grey,'EdgeColor',grey);
hh = histogram(Y_bas,300,'Normalization','probability'); hold on;
set(hh,'FaceColor',[0.15 0.15 0.15],'EdgeColor',[0.15 0.15 0.15]);
title('Invariant internal finance distribution','FontSize',fontsize,'fontname',ffont,'rotation',0,'Interpreter','Latex');
ylabel('Fraction of total mass (\%)','FontSize',fontsize-2,'fontname',ffont,'Rotation',90,'Interpreter','Latex');
xlabel('Internal finance (levels)','FontSize',fontsize-2,'fontname',ffont,'Rotation',0,'Interpreter','Latex');
xlim([e(1) - 5 e(end) + 5]);
set(gca,'Fontsize',fontsize- 2,'fontname','Palatino','TickLabelInterpreter','latex');
ylim([0 0.051])
xlim([0 4])
drawnow; 
pause(.1)
legend('Baseline','Subsidy for market debt')

% Uncomment to get decomposition of effects

%{

% Find equity levels which would have been in R under no subsidy
N_z = size(e,2); N = size(e,1);
k_r_bas = output_nosubsidy.policy_functions.k_hat_r;
k_nr_bas = output_nosubsidy.policy_functions.k_hat_nr;
s_r_bas = output_nosubsidy.policy_functions.s_hat_r;
k_r = output_subsidy.policy_functions.k_hat_r;
k_nr = output_subsidy.policy_functions.k_hat_nr;
s_r = output_subsidy.policy_functions.s_hat_r;
total_k_bas = 0;
mass_bas = reshape(output_nosubsidy.distribution.mu,[N,N_z]);
mass_bas = mass_bas/sum(mass_bas(:));
total_k = 0;
mass = reshape(output_subsidy.distribution.mu,[N,N_z]);
mass = mass/sum(mass(:));
diff_k = 0;
for jj=z_to_plot
    for ii=1:N
        index_bas               = find( abs(e(ii,jj) - e_bas(:,jj))==min(abs(e(ii,jj) - e_bas(:,jj))) );
        dummy_s_hat_bas         = s_r_bas(index_bas,jj);
        dummy_s_hat             = s_r(ii,jj);
        if isnan(dummy_s_hat) && ~isnan(dummy_s_hat_bas)
            fprintf('Hello \n');
            diff_k              = mass(index_bas,jj)*(k_r_bas(index_bas,jj) - k_nr(ii,jj)) + diff_k; 
        end;
        if isnan(s_r(ii,jj))
            total_k                 = total_k + mass(ii,jj)*k_nr(ii,jj);
        else
            total_k                 = total_k + mass(ii,jj)*k_r(ii,jj);
        end;
        if isnan(s_r_bas(ii,jj))
            total_k_bas             = total_k_bas + mass_bas(ii,jj)*k_nr_bas(ii,jj);
        else
            total_k_bas             = total_k_bas + mass_bas(ii,jj)*k_r_bas(ii,jj);
        end;
    end;
end;

clc;

fprintf('\n')
fprintf(' Decomposition of real effects of subsidy : \n');
fprintf(' ------------------------------------------ \n');
fprintf(' Change in aggregate investment:                %2.1f %% \n', (total_k/total_k_bas - 1)*100);
fprintf('    of which, change due to switching firms:    %2.1f %% \n',(diff_k/total_k_bas-1)*100);
fprintf('\n')
%}

set(f1, 'PaperUnits', 'inches') 
set(f1, 'PaperOrientation', 'portrait');
set(f1, 'PaperSize',[10 4]);
papersize = get(gcf,'PaperSize');
width  = 10;         % Initialize a variable for width.
height = 4;          % Initialize a variable for height.
bottom = (papersize(2)- height)/2;
left = (papersize(1)- width)/2;
myfiguresize = [left-1, bottom, width+1.5, height];
set(gcf, 'PaperPosition', myfiguresize);
print(f1, '-depsc2',  graphname); 
    



