function fig5a(policy_functions,figname,savepath,fignum,has_arrows)

graphname = strcat(savepath,'/',figname);

f1 = figure(fignum);

ffont = 'Palatino';
fontsize = 12;
widthline = 2.5;

e = policy_functions.e;

e_H = e(:,3);
s_H = policy_functions.s_hat_r(:,3);
index_last_H = sum(~isnan(s_H));

e_L = e(:,2);
s_L = policy_functions.s_hat_r(:,2);
index_last_L = sum(~isnan(s_L));

nn_e = size(e,1);

colour = '-+k';

plot(e_H,0*ones(nn_e,1),'-','LineWidth',0.1,'Color','black');
hold on;
h1 = plot(e_H,100*s_H+0.25,'Color',[0 0 0],'LineWidth',widthline,'Marker','+','MarkerEdgeColor','k','MarkerFaceColor','k','MarkerSize',4);
hold on;
plot(e_H(index_last_H),100*s_H(index_last_H)+0.25,colour,'Marker','o','MarkerEdgeColor','k','MarkerFaceColor','k','MarkerSize',6);
hold on;
plot(e_H(index_last_H:end),0*e_H(index_last_H:end)+0.25,'Color',[0 0 0],'LineWidth',widthline,'Marker','+','MarkerEdgeColor','k','MarkerFaceColor','k','MarkerSize',4);
hold on;
plot(e_H(index_last_H),0*s_H(index_last_H)+0.25,colour,'Marker','o','MarkerEdgeColor','k','MarkerSize',6);
hold on;
plot(e_H(index_last_H)*ones(nn_e,1),linspace(0,110,nn_e),'--','LineWidth',0.5,'Color','black');

grey = [0.6;0.6;0.6];
h2 = plot(e_L(1:index_last_L),100*s_L(1:index_last_L)+0.25,'Color',grey,'LineWidth',widthline);
hold on;
plot(e_L(index_last_L),100*s_L(index_last_L)+0.25,'Color',grey,'Marker','o','MarkerEdgeColor',grey,'MarkerFaceColor',grey,'MarkerSize',6);
hold on;
plot(e_L(index_last_L:end),0*e_L(index_last_L:end)+0.25,'Color',grey,'LineWidth',widthline);
hold on;
plot(e_L(index_last_L),0*s_L(index_last_L)+0.25,'Color',grey,'Marker','o','MarkerEdgeColor',grey,'MarkerSize',6);
hold on;
plot(e_L(index_last_L)*ones(nn_e,1),linspace(0,110,nn_e),'--','LineWidth',0.5,'Color',grey);
hold on;
plot(e_L(end),0,'Color',grey,'Marker','+','MarkerFaceColor',grey,'MarkerEdgeColor',grey,'MarkerSize',5);

ylim([-2 80])
xlim([0 1.3*e_L(end)])
xlabel('Internal finance ($e_t$)','FontSize',fontsize,'FontWeight','Bold','Interpreter','Latex');
ylabel('$\% \quad$',...
       'FontSize',fontsize,'FontWeight','Bold','Interpreter','Latex','Rotation',00);
set(gca,'fontsize',fontsize - 2,'fontname',ffont,'TickLabelInterpreter', 'latex');

ll = legend([h1 h2],'Bank debt/total debt, $z_t = z_H$','Bank debt/total debt, $z_t = z_L$');
set(ll,'fontsize',fontsize - 2,'fontname',ffont,'FontWeight','Bold','Interpreter','Latex');

if has_arrows

    [arrowx,arrowy] = dsxy2figxy(gca, [e_H(index_last_H-1)+1 e_H(index_last_H-1)+0.2],[35 26]);
    textar1 = annotation('textarrow',arrowx,arrowy);
    set(textar1,...
        'String',{'   $e^{*}(z_H)$ '},...
        'FontSize',fontsize+2,...
        'FontName',ffont,...
        'Interpreter','Latex',...
        'HeadWidth',7);

    [arrowx,arrowy] = dsxy2figxy(gca, [e_L(index_last_L-1)+1 e_L(index_last_L-1)+0.2],[35 26]);
    textar1 = annotation('textarrow',arrowx,arrowy);
    set(textar1,...
        'String',{'   $e^{*}(z_L)$ '},...
        'FontSize',fontsize+2,...
        'FontName',ffont,...
        'Interpreter','Latex',...
        'HeadWidth',7,...
        'Color',grey);

    [arrowx,arrowy] = dsxy2figxy(gca, [6.4-0.3 6.4-0.1],[18 15]);
    textar1 = annotation('textarrow',arrowx,arrowy);
    set(textar1,...
        'String',{'   $\bar{e}(z_H)$ '},...
        'FontSize',fontsize+2,...
        'FontName',ffont,...
        'Interpreter','Latex',...
        'HeadWidth',7,...
        'Color','k');

    [arrowx,arrowy] = dsxy2figxy(gca, [e_L(end)+0.3 e_L(end)+0.1],[8 1]);
    textar1 = annotation('textarrow',arrowx,arrowy);
    set(textar1,...
        'String',{'   $\bar{e}(z_L)$ '},...
        'FontSize',fontsize+2,...
        'FontName',ffont,...
        'Interpreter','Latex',...
        'HeadWidth',7,...
        'Color',grey);
    
end;    
    
set(f1, 'PaperUnits', 'inches') 
set(f1, 'PaperOrientation', 'portrait');
set(f1, 'PaperSize',[7.5 4]);
papersize = get(gcf,'PaperSize');
width  = 7.5;         % Initialize a variable for width.
height = 4;          % Initialize a variable for height.
bottom = (papersize(2)- height)/2;
left = (papersize(1)- width)/2;
myfiguresize = [left, bottom, width, height];
set(gcf, 'PaperPosition', myfiguresize);
print(f1, '-depsc2',  graphname);
%print(f1,'-dpng',  graphname,'-r300');  
