function fig5b(policy_functions,figname,savepath,fignum,has_arrows)

graphname = strcat(savepath,'/',figname);

e           = policy_functions.e;
s_hat_r     = policy_functions.s_hat_r;
k_hat_r     = policy_functions.k_hat_r;
k_hat_nr    = policy_functions.k_hat_nr;

e_H = e(:,3);
s_R_H = s_hat_r(:,3);
d_R_H = k_hat_r(:,3) - e_H;
d_K_H = k_hat_nr(:,3) - e_H;
index_last_H = sum(~isnan(s_R_H));

e_L = e(:,2);
s_R_L = s_hat_r(:,2);
d_R_L = k_hat_r(:,2) - e_L;
d_K_L = k_hat_nr(:,2) - e_L;
index_last_L = sum(~isnan(s_R_L));

nn_e = size(e_H,1);

% Make figure
f1 = figure(fignum);

ffont = 'Palatino';
fontsize = 9;
widthline = 2;

subplot(231)
h1 = plot(e_H(1:index_last_H),s_R_H(1:index_last_H).*d_R_H(1:index_last_H),'LineWidth',widthline,'Color','black');
hold on;
plot(e_H(index_last_H:end),0.*d_K_H(index_last_H:end),'LineWidth',widthline,'Color','black');
hold on;
plot(e_H(index_last_H),0,'-k','Marker','o','MarkerEdgeColor','black','MarkerSize',4);
hold on;
ylim([0 14])
plot(e_H(index_last_H),s_R_H(index_last_H).*d_R_H(index_last_H),'Color','black','Marker','o','MarkerEdgeColor','black','MarkerFaceColor','black','MarkerSize',4);
yl = get(gca,'ylim');
plot(e_H(index_last_H)*ones(nn_e,1),linspace(0,yl(2),nn_e),'--','LineWidth',0.5,'Color','black');
xlabel('Internal finance ($e_t$)','FontSize',fontsize,'FontWeight','Bold','Interpreter','Latex');
title('Bank debt $\hat{b}(e_t,z_H)$ (levels)','FontSize',fontsize,'fontname',ffont,'rotation',0,'Interpreter','Latex');
set(gca,'fontsize',fontsize-2,'fontname','Palatino','TickLabelInterpreter', 'latex');
xlim([0 6])

if has_arrows

    textar1 = text(e_H(index_last_H)*1.1,12,'Hello');
    set(textar1,...
        'String',{'   $e^{*}(z_H)$ '},...
        'FontSize',fontsize,...
        'FontName',ffont,...
        'Interpreter','Latex');

end;    
    
subplot(232)
plot(e_H(1:index_last_H),(1-s_R_H(1:index_last_H)).*d_R_H(1:index_last_H),'-','LineWidth',widthline,'Color','black');
hold on;
plot(e_H(index_last_H:end),1.*d_K_H(index_last_H:end),'LineWidth',widthline,'Color','black');
hold on;
plot(e_H(index_last_H),1.*d_K_H(index_last_H),'Color','black','Marker','o','MarkerEdgeColor','black','MarkerSize',4);
hold on;
ylim([0 14])
plot(e_H(index_last_H),(1-s_R_H(index_last_H)).*d_R_H(index_last_H),'-k','Marker','o','MarkerEdgeColor','black','MarkerFaceColor','black','MarkerSize',4);
yl = get(gca,'ylim');
plot(e_H(index_last_H)*ones(nn_e,1),linspace(0,yl(2),nn_e),'--','LineWidth',0.5,'Color','black');
xlabel('Internal finance ($e_t$)','FontSize',fontsize,'FontWeight','Bold','Interpreter','Latex');
title('Market debt $\hat{m}(e_t,z_H)$ (levels)','FontSize',fontsize,'fontname',ffont,'rotation',0,'Interpreter','Latex');
set(gca,'fontsize',fontsize-2,'fontname','Palatino','TickLabelInterpreter', 'latex');
xlim([0 6])

if has_arrows

    textar1 = text(e_H(index_last_H)*1.1,12,'Hello');
    set(textar1,...
        'String',{'   $e^{*}(z_H)$ '},...
        'FontSize',fontsize,...
        'FontName',ffont,...
        'Interpreter','Latex');

end;

subplot(233)
plot(e_H(1:index_last_H),e_H(1:index_last_H)+d_R_H(1:index_last_H),'-','LineWidth',widthline,'Color','black');
hold on;
plot(e_H(index_last_H:end),e_H(index_last_H:end)+d_K_H(index_last_H:end),'LineWidth',widthline,'Color','black');
hold on;
plot(e_H(index_last_H),e_H(index_last_H)+d_K_H(index_last_H),'Color','black','Marker','o','MarkerEdgeColor','black','MarkerSize',4);
hold on;
plot(e_H(index_last_H),e_H(index_last_H)+d_R_H(index_last_H),'Color','black','Marker','o','MarkerEdgeColor','black','MarkerFaceColor','black','MarkerSize',4);
ylim([0 14]);
yl = get(gca,'ylim');
plot(e_H(index_last_H)*ones(nn_e,1),linspace(0,yl(2),nn_e),'--','LineWidth',0.5,'Color','black');
xlabel('Internal finance ($e_t$)','FontSize',fontsize,'FontWeight','Bold','Interpreter','Latex');
title('Total assets $\hat{k}(e_t,z_H)$ (levels)','FontSize',fontsize,'fontname',ffont,'rotation',0,'Interpreter','Latex');
set(gca,'fontsize',fontsize-2,'fontname','Palatino','TickLabelInterpreter', 'latex');
xlim([0 6])

if has_arrows
    
    textar1 = text(e_H(index_last_H)*1.1,12,'Hello');
    set(textar1,...
        'String',{'   $e^{*}(z_H)$ '},...
        'FontSize',fontsize,...
        'FontName',ffont,...
        'Interpreter','Latex');

end;

grey = [0.6 0.6 0.6];
subplot(234)
h1 = plot(e_L(1:index_last_L),s_R_L(1:index_last_L).*d_R_L(1:index_last_L),'LineWidth',widthline,'Color',grey);
hold on;
plot(e_L(index_last_L:end),0.*d_K_L(index_last_L:end),'LineWidth',widthline,'Color',grey);
hold on;
plot(e_L(index_last_L),0,'-k','Marker','o','MarkerEdgeColor',grey,'MarkerSize',4);
hold on;
plot(e_L(index_last_L),s_R_L(index_last_L).*d_R_L(index_last_L),'Color',grey,'Marker','o','MarkerEdgeColor',grey,'MarkerFaceColor',grey,'MarkerSize',4);
ylim([0 5])
yl = get(gca,'ylim');
plot(e_L(index_last_L)*ones(nn_e,1),linspace(0,yl(2),nn_e),'--','LineWidth',0.5,'Color',grey);
xlabel('Internal finance ($e_t$)','FontSize',fontsize,'FontWeight','Bold','Interpreter','Latex');
title('Bank debt $\hat{b}(e_t,z_L)$ (levels)','FontSize',fontsize,'fontname',ffont,'rotation',0,'Interpreter','Latex');
set(gca,'fontsize',fontsize-2,'fontname','Palatino','TickLabelInterpreter', 'latex');
xlim([0 2])

if has_arrows

    textar1 = text(e_L(index_last_L)*1.1,4,'Hello');
    set(textar1,...
        'String',{'   $e^{*}(z_L)$ '},...
        'FontSize',fontsize,...
        'FontName',ffont,...
        'Interpreter','Latex',...
        'Color',grey);
    
end;    

subplot(235)
plot(e_L(1:index_last_L),(1-s_R_L(1:index_last_L)).*d_R_L(1:index_last_L),'-','LineWidth',widthline,'Color',grey);
hold on;
plot(e_L(index_last_L:end),1.*d_K_L(index_last_L:end),'LineWidth',widthline,'Color',grey);
hold on;
plot(e_L(index_last_L),1.*d_K_L(index_last_L),'Color',grey,'Marker','o','MarkerEdgeColor',grey,'MarkerSize',4);
hold on;
plot(e_L(index_last_L),(1-s_R_L(index_last_L)).*d_R_L(index_last_L),'Color',grey,'Marker','o','MarkerEdgeColor',grey,'MarkerFaceColor',grey,'MarkerSize',4);
ylim([0 5])
yl = get(gca,'ylim');
plot(e_L(index_last_L)*ones(nn_e,1),linspace(0,yl(2),nn_e),'--','LineWidth',0.5,'Color',grey);
xlabel('Internal finance ($e_t$)','FontSize',fontsize,'FontWeight','Bold','Interpreter','Latex');
title('Market debt $\hat{m}(e_t,z_L)$ (levels)','FontSize',fontsize,'fontname',ffont,'rotation',0,'Interpreter','Latex');
set(gca,'fontsize',fontsize-2,'fontname','Palatino','TickLabelInterpreter', 'latex');
xlim([0 2])

if has_arrows

    textar1 = text(e_L(index_last_L)*1.1,4,'Hello');
    set(textar1,...
        'String',{'   $e^{*}(z_L)$ '},...
        'FontSize',fontsize,...
        'FontName',ffont,...
        'Interpreter','Latex',...
        'Color',grey);
    
end;

subplot(236)
plot(e_L(1:index_last_L),e_L(1:index_last_L)+d_R_L(1:index_last_L),'-','LineWidth',widthline,'Color',grey);
hold on;
plot(e_L(index_last_L:end),e_L(index_last_L:end)+d_K_L(index_last_L:end),'LineWidth',widthline,'Color',grey);
hold on;
plot(e_L(index_last_L),e_L(index_last_L)+d_K_L(index_last_L),'-k','Marker','o','MarkerEdgeColor',grey,'MarkerSize',4);
hold on;
plot(e_L(index_last_L),e_L(index_last_L)+d_R_L(index_last_L),'-k','Marker','o','MarkerEdgeColor',grey,'MarkerFaceColor',grey,'MarkerSize',4);
ylim([0 5]);
yl = get(gca,'ylim');
plot(e_L(index_last_L)*ones(nn_e,1),linspace(0,yl(2),nn_e),'--','LineWidth',0.5,'Color',grey);
xlabel('Internal finance ($e_t$)','FontSize',fontsize,'FontWeight','Bold','Interpreter','Latex');
title('Total assets $\hat{k}(e_t,z_L)$ (levels)','FontSize',fontsize,'fontname',ffont,'rotation',0,'Interpreter','Latex');
set(gca,'fontsize',fontsize-2,'fontname','Palatino','TickLabelInterpreter', 'latex');
xlim([0 2])

if has_arrows

    textar1 = text(e_L(index_last_L)*1.1,4,'Hello');
    set(textar1,...
        'String',{'   $e^{*}(z_L)$ '},...
        'FontSize',fontsize,...
        'FontName',ffont,...
        'Interpreter','Latex',...
        'Color',grey);

end;

% Export figure to *.eps
set(f1, 'PaperUnits', 'inches') 
set(f1, 'PaperOrientation', 'portrait');
set(f1, 'PaperSize',[8.5 4]);
papersize = get(gcf,'PaperSize');
width  = 8.5;         
height = 4;          
bottom = (papersize(2)- height)/2;
left = (papersize(1)- width)/2;
myfiguresize = [left, bottom, width, height];
set(gcf, 'PaperPosition', myfiguresize);
print(f1, '-depsc2',  graphname);
%print(f1,'-dpng',  graphname,'-r300'); 