function fig6(policy_functions,distribution,figname,savepath,fignum,has_arrows)

graphname = strcat(savepath,'/',figname);

% Output from model
e               = policy_functions.e;
mu              = distribution.mu;
s_hat_r         = policy_functions.s_hat_r;

% Sort equity vector
[e_sorted,indexes_sorted] = sort(e(:));
mu_sorted = mu(indexes_sorted);
mu_sorted = mu_sorted/sum(mu_sorted);
s_hat = s_hat_r;
s_hat(isnan(s_hat)) = 0;
s_sorted = s_hat(indexes_sorted);
N_sorted = numel(e_sorted);
II = datasample(1:N_sorted,1e7,'weights',mu_sorted);
Y = e_sorted(II);
S = s_sorted(II);
p = [0.10;0.20;0.30;0.40;0.50;0.60;0.70;0.80;0.90;1.00];
quantiles_e = quantile(Y,p);
averag_share_E = zeros(size(p));
for ii=1:numel(p)
    if ii==1
        indexes = find(Y <= quantiles_e(ii));
    else
        indexes = find( (Y > quantiles_e(ii-1))&(Y <= quantiles_e(ii)) );
    end;
    averag_share_E(ii) = mean(S(indexes));
end;

grey = [0.6 0.6 0.6];
ffont = 'Palatino';

f1 = figure(fignum);
fontsize = 12;

subplot(121)
hh = histogram(Y,300,'Normalization','probability');
set(hh,'FaceColor',grey,'EdgeColor',grey,'FaceAlpha',1);
title('Invariant internal finance distribution','FontSize',fontsize,'fontname',ffont,'rotation',0,'Interpreter','Latex');
ylabel('Fraction of total mass (\%)','FontSize',fontsize-2,'fontname',ffont,'Rotation',90,'Interpreter','Latex');
xlabel('Internal finance (levels)','FontSize',fontsize-2,'fontname',ffont,'Rotation',0,'Interpreter','Latex');
xlim([e(1) - 5 e(end) + 5]);
set(gca,'Fontsize',fontsize- 2,'fontname','Palatino','TickLabelInterpreter', 'latex');
ylim([0 0.051])
xlim([0 4])
drawnow; 
pause(.1)

if has_arrows

    [arrowx,arrowy] = dsxy2figxy(gca, [2.3+0.3 2.3+0.05],[0.028 0.026]);
    textar1 = annotation('textarrow',arrowx,arrowy);
    set(textar1,...
        'String',{'   $\bar{e}(z_{min})$ '},...
        'FontSize',fontsize,...
        'FontName',ffont,...
        'Interpreter','Latex',...
        'HeadWidth',4,...
        'Color',grey);
    ax = gca;
    ax.YTickLabel = {'0','0.5','1','1.5','2','2.5','3','3.5','4','4.5','5'};

end;

subplot(122)
ss = 40;
scatter(100*p,100*averag_share_E,ss,'o','MarkerEdgeColor','black','MarkerFaceColor','black','LineWidth',1.5)
title('Distribution of bank share','FontSize',fontsize,'fontname',ffont,'rotation',0,'Interpreter','Latex');
ylabel('Average bank share (\%)','FontSize',fontsize-2,'fontname',ffont,'Rotation',90,'Interpreter','Latex');
xlabel('Percentile of internal finance distribution','FontSize',fontsize-2,'fontname',ffont,'Rotation',0,'Interpreter','Latex');
set(gca,'Fontsize',fontsize- 2,'fontname','Palatino','TickLabelInterpreter', 'latex');
ylim([0 80])
grid on

set(f1, 'PaperUnits', 'inches') 
set(f1, 'PaperOrientation', 'portrait');
set(f1, 'PaperSize',[8.5 3.5]);
papersize = get(gcf,'PaperSize');
width  = 8.5;         % Initialize a variable for width.
height = 3.5;          % Initialize a variable for height.
bottom = (papersize(2)- height)/2;
left = (papersize(1)- width)/2;
myfiguresize = [left-0.25, bottom, width+0.25, height];
set(gcf, 'PaperPosition', myfiguresize); 
%print(f1, '-dpng',  graphname); 
print(f1, '-depsc2',  graphname);
