function [] = fig9(savepath,figname,fignum)

%%% fig9.m
%%%
%%% Generates figure 9 (debt composition during the US Great Recession)
%%% from "Aggregate implications of corporate debt choices"
%%%
%%% Uncommenting some entries below can call a number of stata scripts filed 
%%% under /data sources/QFR and /data sources/Flow of Funds/, which download 
%%% and format the QFR and Flow of Funds data:
%%%     - extractQFR.do
%%%     - extractFoF.do
%%%
%%% Nicolas Crouzet
%%% Last modified: September 2017
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set up file paths and file names
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Name and directory to save figure to be produced
% Add Stata to the executable path; may need to change this for other
% versions of Stata; may not work with non-unix systems
%{
PATH = getenv('PATH');
setenv('PATH', [PATH ':/Applications/Stata/StataSE.app/Contents/MacOS/']);
clear PATH
%}
% Create filepaths
dir_current = pwd;
path_script = pwd;
clear script_name
pathQFRData = [pwd '/data sources/QFR/'];
addpath(pathQFRData)
pathFoFData = [pwd '/data sources/Flow of Funds/'];
addpath(pathFoFData)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Download, format and extract relevant info from QFR
% Done externally by the stata script extractQFR.do
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cd(pathQFRData);
% Uncomment to download directly the QFR tables
% Will likely not work on non-unix systems
%{
fprintf('\n')
fprintf('Downloading and extracting QFR data ... \n');
system(['stata-se -b ' '"' pathQFRData 'extractQFR.do' '"' ' ' '"matlabExtractQFR"']);
%}
out = csv2mat_numeric([pathQFRData 'matlabExtractQFR.csv']);
year = out.year;
quarter = out.quarter;
assetSize = out.assetSize;
CC1 = out.cc1;
CC3 = out.cc3;
TT1 = out.tt1;
TT3 = out.tt3;
CCFIN = out.ccfin;
TTFIN = out.ttfin;
clear out

NumberOfAssetClasses = 8;
numberOfPeriods = length(year)/NumberOfAssetClasses;
InitialDate = year(1)+(quarter(1)-1)*0.25;
FinalDate = year(end) + (quarter(end)-1)*0.25;
datesQFR = linspace(InitialDate,FinalDate,numberOfPeriods)';
clear Year Quarter InitialDate FinalDate

CC1   = reshape(CC1,[NumberOfAssetClasses numberOfPeriods])';
CC3   = reshape(CC3,[NumberOfAssetClasses numberOfPeriods])';
TT1   = reshape(TT1,[NumberOfAssetClasses numberOfPeriods])';
TT3   = reshape(TT3,[NumberOfAssetClasses numberOfPeriods])';
CCFIN = reshape(CCFIN,[NumberOfAssetClasses numberOfPeriods])';
TTFIN = reshape(TTFIN,[NumberOfAssetClasses numberOfPeriods])';

graphFirstDate = 2007.5;
graphLastDate = 2011.5;

% Uncomment to compute the Gertler-Gilchrist definition of small and large
% firms

%{
% Computes the thirty percent cut-off for sales.
sales_shares = zeros(timespan,8);
index_shares = zeros(timespan,1);
weight = zeros(timespan,1);
G_C1S = zeros(timespan-1,1);
G_C1L = zeros(timespan-1,1);
G_C3S = zeros(timespan-1,1);
G_C3L = zeros(timespan-1,1);
G_T1S = zeros(timespan-1,1);
G_T1L = zeros(timespan-1,1);
G_T3S = zeros(timespan-1,1);
G_T3L = zeros(timespan-1,1);
G_CFS = zeros(timespan-1,1);
G_CFL = zeros(timespan-1,1);
G_TFS = zeros(timespan-1,1);
G_TFL = zeros(timespan-1,1);
for tt=1:timespan
    dummy = cumsum(SS(tt,:));
    tots = sum(SS(tt,:));
    sales_shares(tt,:) = dummy./tots;
    index_shares(tt) = find(sales_shares(tt,:) <= 0.3,1,'last');
    if index_shares(tt) < 7
        weight(tt) = (0.3*tots - dummy(index_shares(tt)+1))/(dummy(index_shares(tt)) - dummy(index_shares(tt)+1));
    else
        weight(tt) = 1;
    end;
    if tt > 1
        
        dummy = cumsum(CC1(tt,:));
        dummy_p = cumsum(CC1(tt-1,:));
        if index_shares(tt) < 7
        G_C1S(tt-1) = weight(tt)*dummy(index_shares(tt))/dummy_p(index_shares(tt)) + ...
                      (1-weight(tt))*dummy(index_shares(tt)+1)/dummy_p(index_shares(tt)+1);
        G_C1L(tt-1) = weight(tt)*(dummy(end) - dummy(index_shares(tt)))/(dummy_p(end) - dummy_p(index_shares(tt))) + ...
                     (1-weight(tt))*(dummy_p(end) - dummy(index_shares(tt)+1))/(dummy_p(end) - dummy_p(index_shares(tt)+1)); 
        else
        G_C1S(tt-1) = dummy(index_shares(tt))/dummy_p(index_shares(tt));
        G_C1L(tt-1) = (dummy(end) - dummy(index_shares(tt)))/(dummy_p(end) - dummy_p(index_shares(tt)));
        end;
        
        dummy = cumsum(CC3(tt,:));
        dummy_p = cumsum(CC3(tt-1,:));
        if index_shares(tt) < 7
        G_C3S(tt-1) = weight(tt)*dummy(index_shares(tt))/dummy_p(index_shares(tt)) + ...
                      (1-weight(tt))*dummy(index_shares(tt)+1)/dummy_p(index_shares(tt)+1);
        G_C3L(tt-1) = weight(tt)*(dummy(end) - dummy(index_shares(tt)))/(dummy_p(end) - dummy_p(index_shares(tt))) + ...
                     (1-weight(tt))*(dummy_p(end) - dummy(index_shares(tt)+1))/(dummy_p(end) - dummy_p(index_shares(tt)+1)); 
        else
        G_C3S(tt-1) = dummy(index_shares(tt))/dummy_p(index_shares(tt));
        G_C3L(tt-1) = (dummy(end) - dummy(index_shares(tt)))/(dummy_p(end) - dummy_p(index_shares(tt)));
        end;
        
        dummy = cumsum(TT1(tt,:));
        dummy_p = cumsum(TT1(tt-1,:));
        if index_shares(tt) < 7
        G_T1S(tt-1) = weight(tt)*dummy(index_shares(tt))/dummy_p(index_shares(tt)) + ...
                      (1-weight(tt))*dummy(index_shares(tt)+1)/dummy_p(index_shares(tt)+1);
        G_T1L(tt-1) = weight(tt)*(dummy(end) - dummy(index_shares(tt)))/(dummy_p(end) - dummy_p(index_shares(tt))) + ...
                     (1-weight(tt))*(dummy_p(end) - dummy(index_shares(tt)+1))/(dummy_p(end) - dummy_p(index_shares(tt)+1)); 
        else
        G_T1S(tt-1) = dummy(index_shares(tt))/dummy_p(index_shares(tt));
        G_T1L(tt-1) = (dummy(end) - dummy(index_shares(tt)))/(dummy_p(end) - dummy_p(index_shares(tt)));
        end;
        
        dummy = cumsum(TT3(tt,:));
        dummy_p = cumsum(TT3(tt-1,:));
        if index_shares(tt) < 7
        G_T3S(tt-1) = weight(tt)*dummy(index_shares(tt))/dummy_p(index_shares(tt)) + ...
                      (1-weight(tt))*dummy(index_shares(tt)+1)/dummy_p(index_shares(tt)+1);
        G_T3L(tt-1) = weight(tt)*(dummy(end) - dummy(index_shares(tt)))/(dummy_p(end) - dummy_p(index_shares(tt))) + ...
                     (1-weight(tt))*(dummy_p(end) - dummy(index_shares(tt)+1))/(dummy_p(end) - dummy_p(index_shares(tt)+1)); 
        else
        G_T3S(tt-1) = dummy(index_shares(tt))/dummy_p(index_shares(tt));
        G_T3L(tt-1) = (dummy(end) - dummy(index_shares(tt)))/(dummy_p(end) - dummy_p(index_shares(tt)));
        end;
        
        dummy = cumsum(CCFIN(tt,:));
        dummy_p = cumsum(CCFIN(tt-1,:));
        if index_shares(tt) < 7
        G_CFS(tt-1) = weight(tt)*dummy(index_shares(tt))/dummy_p(index_shares(tt)) + ...
                      (1-weight(tt))*dummy(index_shares(tt)+1)/dummy_p(index_shares(tt)+1);
        G_CFL(tt-1) = weight(tt)*(dummy(end) - dummy(index_shares(tt)))/(dummy_p(end) - dummy_p(index_shares(tt))) + ...
                     (1-weight(tt))*(dummy_p(end) - dummy(index_shares(tt)+1))/(dummy_p(end) - dummy_p(index_shares(tt)+1)); 
        else
        G_CFS(tt-1) = dummy(index_shares(tt))/dummy_p(index_shares(tt));
        G_CFL(tt-1) = (dummy(end) - dummy(index_shares(tt)))/(dummy_p(end) - dummy_p(index_shares(tt)));
        end;
        
        dummy = cumsum(TTFIN(tt,:));
        dummy_p = cumsum(TTFIN(tt-1,:));
        if index_shares(tt) < 7
        G_TFS(tt-1) = weight(tt)*dummy(index_shares(tt))/dummy_p(index_shares(tt)) + ...
                      (1-weight(tt))*dummy(index_shares(tt)+1)/dummy_p(index_shares(tt)+1);
        G_TFL(tt-1) = weight(tt)*(dummy(end) - dummy(index_shares(tt)))/(dummy_p(end) - dummy_p(index_shares(tt))) + ...
                     (1-weight(tt))*(dummy_p(end) - dummy(index_shares(tt)+1))/(dummy_p(end) - dummy_p(index_shares(tt)+1)); 
        else
        G_TFS(tt-1) = dummy(index_shares(tt))/dummy_p(index_shares(tt));
        G_TFL(tt-1) = (dummy(end) - dummy(index_shares(tt)))/(dummy_p(end) - dummy_p(index_shares(tt)));
        end;
      
    end;
end;

y = cumsum(log(G_C1S));
y = [0;y];
C1S = exp(y);

y = cumsum(log(G_C1L));
y = [0;y];
C1L = exp(y);

y = cumsum(log(G_C3S));
y = [0;y];
C3S = exp(y);

y = cumsum(log(G_C3L));
y = [0;y];
C3L = exp(y);

y = cumsum(log(G_T1S));
y = [0;y];
T1S = exp(y);

y = cumsum(log(G_T1L));
y = [0;y];
T1L = exp(y);

y = cumsum(log(G_T3S));
y = [0;y];
T3S = exp(y);

y = cumsum(log(G_T3L));
y = [0;y];
T3L = exp(y);

y = cumsum(log(G_CFS));
y = [0;y];
CFS = exp(y);

y = cumsum(log(G_CFL));
y = [0;y];
CFL = exp(y);

y = cumsum(log(G_TFS));
y = [0;y];
TFS = exp(y);

y = cumsum(log(G_TFL));
y = [0;y];
TFL = exp(y);

% Compute the weight w_t

%}
            
% Aggregates for total liabilities - MA filter

TFS = sum(TTFIN(:,7),2);
maTFS = NaN(numberOfPeriods,1);
for kk = 3:(numberOfPeriods-2)
maTFS(kk) = (1/8)*TFS(kk-2) + (1/4)*TFS(kk-1) + (1/4)*TFS(kk) + (1/4)*TFS(kk+1) + (1/8)*TFS(kk+2);
end;

TFL = sum(TTFIN(:,8),2);
maTFL = NaN(numberOfPeriods,1);
for kk = 3:(numberOfPeriods-2)
maTFL(kk) = (1/8)*TFL(kk-2) + (1/4)*TFL(kk-1) + (1/4)*TFL(kk) + (1/4)*TFL(kk+1) + (1/8)*TFL(kk+2);
end;

T1S = sum(TT1(:,7),2);
maT1S = NaN(numberOfPeriods,1);
for kk = 3:(numberOfPeriods-2)
maT1S(kk) = (1/8)*T1S(kk-2) + (1/4)*T1S(kk-1) + (1/4)*T1S(kk) + (1/4)*T1S(kk+1) + (1/8)*T1S(kk+2);
end;

T1L = sum(TT1(:,8),2);
maT1L = NaN(numberOfPeriods,1);
for kk = 3:(numberOfPeriods-2)
maT1L(kk) = (1/8)*T1L(kk-2) + (1/4)*T1L(kk-1) + (1/4)*T1L(kk) + (1/4)*T1L(kk+1) + (1/8)*T1L(kk+2);
end;

T3S = sum(TT3(:,7),2);
maT3S = NaN(numberOfPeriods,1);
for kk = 3:(numberOfPeriods-2)
maT3S(kk) = (1/8)*T3S(kk-2) + (1/4)*T3S(kk-1) + (1/4)*T3S(kk) + (1/4)*T3S(kk+1) + (1/8)*T3S(kk+2);
end;

T3L = TT3(:,8);
maT3L = NaN(numberOfPeriods,1);
for kk = 3:(length(TFS)-2)
maT3L(kk) = (1/8)*T3L(kk-2) + (1/4)*T3L(kk-1) + (1/4)*T3L(kk) + (1/4)*T3L(kk+1) + (1/8)*T3L(kk+2);
end;

T1S_F = ( (maT1S - maT1S(datesQFR==2008.5)) ./ maTFS(datesQFR==2008.5)   );        
T3S_F = ( (maT3S - maT3S(datesQFR==2008.5)) ./ maTFS(datesQFR==2008.5)   ); 
T1L_F = ( (maT1L - maT1L(datesQFR==2008.5)) ./ maTFL(datesQFR==2008.5)   );
T3L_F = ( (maT3L - maT3L(datesQFR==2008.5)) ./ maTFL(datesQFR==2008.5)   ); 
BSQFR = 100*(sum(TT1(:,7:8),2))./(sum(TTFIN(:,7:8),2));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Download, format and extract relevant info from Flow of Funds
% Done externally by the stata script extractFoF.do
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cd(pathFoFData);
% Uncomment to download directly the QFR tables
% Will likely not work on non-unix systems
%{
fprintf('\n')
fprintf('Downloading and extracting FoF data ... \n');
system(['stata-se -b ' '"' pathFoFData 'extractFoF.do' '"' ' ' '"matlabExtractFoF"']);
%}
out= csv2mat_numeric([pathFoFData 'matlabExtractFoF.csv']);
year = out.year;
quarter = out.quarter;
l_depositoryinstitloans = out.l_depositoryinstitloans;
l_otherloans = out.l_otherloans;
l_creditmarketinstruments = out.l_creditmarketinstruments;
clear out
numberOfPeriods = length(year);
InitialDate = year(1)+(quarter(1)-1)*0.25;
FinalDate = year(end) + (quarter(end)-1)*0.25;
datesFoF = linspace(InitialDate,FinalDate,numberOfPeriods)';
clear Year Quarter InitialDate FinalDate
BSFoF = 100*(l_depositoryinstitloans+l_otherloans)./l_creditmarketinstruments;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make graphs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

f1 = figure(fignum);

ffont = 'Garamond';
fontsize = 13;
colour1 = 'black';
colour2 = [0.6;0.6;0.6];
colour3 = [0.4;0.4;0.4];
linewid = 2.5;    

s3 = subplot(131);
[AX,h1,h3] = plotyy(datesQFR,BSQFR,datesFoF,BSFoF);
set(h1,'Linestyle','-','Color',colour3,'MarkerFaceColor',colour3,'MarkerEdgeColor',colour3,'Linewidth',linewid);
set(h3,'Linestyle','-.','Color',colour2,'MarkerFaceColor',colour2,'MarkerEdgeColor',colour2,'Linewidth',linewid);
title({'\textbf{Aggregate bank share}',' '},'FontSize',fontsize+2,'FontName','Garamond','FontWeight','bold','interpreter','latex');
line([2008.5 2008.5],[15 35],'Color','k','Linewidth',1.5,'Linestyle','--');
grid on;
set(AX,'Fontsize',fontsize-3,'fontname','Garamond');
set(AX,'XTick',[2008:1:2011])
set(AX,'XTickLabel',{'2008Q1','2009Q1','2010Q1','2011Q1'},'TickLabelInterpreter', 'latex');
set(AX,'xlim',[graphFirstDate graphLastDate]);
set(AX(1),'ylim',[19.5 26]);
ylabel(AX(1),'\%','FontSize',fontsize,'FontName','Garamond','Interpreter','latex');
set(AX(1),'YTick',[20:1:26]);
set(AX(2),'YTick',[25:5:35]);
set(AX(2),'ylim',[23 35]);
%ylabel(AX(2),'%','FontSize',fontsize,'FontName','Garamond');
set(AX(1),'ycolor','k') 
set(AX(2),'ycolor','k')

l = legend([h1 h3],'Firms with assets $\geq$ \$0.25bn (left axis)','All firms (right axis)');
set(l,'FontSize',fontsize-4,'FontName','Garamond');
set(l,'Location','NorthEast');
set(l,'Interpreter','latex');

s3 = subplot(132);
h1  = plot(datesQFR,100*T1S_F,'-','Color',colour2,'Linewidth',linewid);
hold on;
h3 = plot(datesQFR,100*T3S_F,'-','Color',colour1,'Linewidth',linewid);
title({'\textbf{Debt composition:}','\textbf{Firms with \$0.25bn} $\leq$ \textbf{assets} $\leq$ \textbf{\$1bn}'},'FontSize',fontsize+2,'FontName','Garamond','FontWeight','bold','interpreter','latex');
ylabel('\% relative to 2008Q3','FontSize',fontsize,'FontName','Garamond','interpreter','latex');
line([2008.5 2008.5],[-20 20],'Color','k','Linewidth',1.5,'Linestyle','--');
grid on;
set(gca,'Fontsize',fontsize-3,'fontname','Garamond');
set(gca,'XTick',[2008:1:2011])
set(gca,'XTickLabel',{'2008Q1','2009Q1','2010Q1','2011Q1'},'TickLabelInterpreter', 'latex');
xlim([graphFirstDate graphLastDate]);

l = legend([h1 h3],'Bank debt','Market debt');
set(l,'FontSize',fontsize-6,'FontName','Garamond');
set(l,'Location','NorthWest');
set(l,'Interpreter','latex');

s3 = subplot(133);
h2  = plot(datesQFR,100*T1L_F,'-','Color',colour2,'Linewidth',linewid);
hold on;
h4 = plot(datesQFR,100*T3L_F,'-','Color',colour1,'Linewidth',linewid);
title({'\textbf{Debt composition:}','\textbf{Firms with assets $>$ \$1bn}'},'FontSize',fontsize+2,'FontName','Garamond','FontWeight','bold','interpreter','latex');
ylabel('\% relative to 2008Q3','FontSize',fontsize,'FontName','Garamond','interpreter','latex');
ylim([-20 20]);
line([2008.5 2008.5],[-20 20],'Color','k','Linewidth',1.5,'Linestyle','--');
grid on;
set(gca,'Fontsize',fontsize-3,'fontname','Garamond');
set(gca,'XTick',[2008:1:2011])
set(gca,'XTickLabel',{'2008Q1','2009Q1','2010Q1','2011Q1'},'TickLabelInterpreter', 'latex');
xlim([graphFirstDate graphLastDate]);

l = legend([h2 h4],'Bank debt','Market debt');
set(l,'FontSize',fontsize-4,'FontName','Garamond');
set(l,'Location','NorthWest');
set(l,'Interpreter','latex');

cd(dir_current)
graphname = strcat(figname);
set(f1, 'PaperUnits', 'inches') 
set(f1, 'PaperOrientation', 'portrait');
set(f1, 'PaperSize',[11.5 4]);
papersize = get(gcf,'PaperSize');
width  = 12.5;         % Initialize a variable for width.
height = 5;          % Initialize a variable for height.
bottom = (papersize(2)- height)/2;
left = (papersize(1)- width)/2;
myfiguresize = [left-1, bottom, width+1.5, height];
set(gcf, 'PaperPosition', myfiguresize);
print(f1, '-depsc2',  [savepath graphname]);

%% Print out bank share

indexQFR1 = find(datesQFR == 2007.5);
shareQFR1 = BSQFR(indexQFR1);
indexQFR2 = find(datesQFR == 2011.5);
shareQFR2 = BSQFR(indexQFR2);
indexFoF1 = find(datesFoF == 2007.5);
shareFoF1 = BSFoF(indexFoF1);
indexFoF2 = find(datesFoF == 2011.5);
shareFoF2 = BSFoF(indexFoF2);

fprintf('\n')
fprintf('Bank share of firms with more than $250m in assets in 2007Q3: %3.1f %% \n',shareQFR1);
fprintf('\n')
fprintf('Aggregate bank share in 2007Q3: %3.1f %% \n',shareFoF1);
fprintf('\n')
fprintf('\n')
fprintf('Bank share of firms with more than $250m in assets in 2011Q3: %3.1f %% \n',shareQFR2);
fprintf('\n')
fprintf('Aggregate bank share in 2011Q3: %3.1f %% \n',shareFoF2);
fprintf('\n')

%% Clean-up
%close all
%delete([pathQFRData '/*.csv'])
delete([pathQFRData '/*.log'])
%delete([pathFoFData '/*.csv'])
delete([pathFoFData '/*.log'])

cd(dir_current);

end

